        Subroutine Floodfill(i,j)
        
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!       This subroutine determines all contiguous nodes in same subset as
!       seed, ii,jj. Nodes must be connected either vertically or horizontally.
!
!       Derived from C++ algorithm written by: 
!       Edwin P. Berlin, Jr.
!       Copyright (c) 2020 Secret Software
!
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!
!        Called by Ellipsets, Ellipwatersets, Spheresets
!
!        VARIABLES
!
!        i1,i2,j1,j1 -- array bounds for current set
!        icont -- flag for whether node is included in spheroid 
!        inscoop(nx,ny) -- indicates subset membership of DEM column nodes
!        maxspan -- number of spans used in floodfill routine used to dimension arrays
!        numset -- current set number
!        spanleft,spanright -- arrays containing bounds of continuous set of included nodes 
!           to left or right of included nodes on previous row
!        spanrow -- array of positive or negative row count representing number of rows from
!           seed row, i,j
!
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

        USE SetData, ONLY: i1,j1,i2,j2,numset,icont,spanrow,spanleft,spanright,maxspan
        USE ColumnData, ONLY: inscoop        
        
        IMPLICIT NONE

        INTERFACE
          SUBROUTINE Nodeinout(inode,jnode) 
            INTEGER, INTENT(in) :: inode,jnode
          END SUBROUTINE Nodeinout
          
          SUBROUTINE NodeTest(inode,jnode) 
            INTEGER, INTENT(in) :: inode,jnode
          END SUBROUTINE NodeTest
          
          SUBROUTINE reallocate_1d(growarray,newupperbound)
            INTEGER,DIMENSION(:),ALLOCATABLE,INTENT(INOUT) :: growarray
            INTEGER,INTENT(IN) :: newupperbound
          END SUBROUTINE reallocate_1d
        END INTERFACE

        INTEGER, INTENT(in) :: i,j 
        INTEGER :: numspans,left,right,oldleft,oldright,si,sj
        LOGICAL :: down

        si = i
        sj = j         
        
!     Starter node will always be included (i.e. initial call to NodeTest will set icont = 1. 
!     Nodes are marked as belonging to current subset by setting inscoop to numset within NodeTest.
        CALL NodeTest(si,sj)
        left = si
        right = si
        oldleft = left
        oldright = right
 
!     Scan to the left to find span of continuous nodes on row containing seed.  
        DO WHILE (left.gt.i1)
          IF (inscoop(left-1,sj).ne.0.and.inscoop(left-1,sj).ne.numset) EXIT
          CALL NodeTest(left-1,sj)
          IF (icont.eq.0) EXIT
          left = left - 1
        END DO

!     Scan to the right to find span of continuous nodes on row containing seed.         
        DO WHILE (right.lt.i2)
          IF (inscoop(right+1,sj).ne.0.and.inscoop(right+1,sj).ne.numset) EXIT
          CALL NodeTest(right+1,sj)
          IF (icont.eq.0) EXIT
          right = right + 1
        END DO

!     Set initial span of included nodes on rows above and below seed row  
        spanrow(1) = sj
        spanleft(1) = left
        spanright(1) = right        
        
        spanrow(2) = -sj
        spanleft(2) = left
        spanright(2) = right
        numspans = 3    

!     Loop until there are no more spans in the list        
        DO WHILE (numspans.ne.1)
          numspans = numspans - 1
          sj = spanrow(numspans)
!     Find continuous spans in rows below seed row
          IF (sj.lt.0) THEN
            sj = -sj
!     If no more rows below seed row will switch to rows above in next loop
            IF (sj.eq.j1) CYCLE
            sj = sj - 1
            down = .TRUE.
          ELSE
!     Find continuous spans in rows below seed row until top row j2
            IF (sj.ge.j2) CYCLE
            sj = sj + 1
            down = .FALSE.
          END IF
          
          left = spanleft(numspans)
          right = spanright(numspans)

!     Scan left of prior scan end point from prior row to create new up and down scans.
!     Nodeinout identifies whether node is contained in spheroid but does not mark inscoop.
          IF (inscoop(left,sj).eq.0) THEN
            CALL Nodeinout(left,sj)  
            IF (icont.eq.1) THEN
              oldleft = left
              DO WHILE (left.gt.1)  !  Scan left
                IF (inscoop(left-1,sj).ne.0) EXIT
                CALL Nodeinout(left-1,sj)
                IF (icont.eq.0) EXIT
                left = left - 1
              END DO

!     Check whether new line has continuous nodes more than two to the left of last span.
!     If so then there is a new span direction opposite of current direction.              
              IF (left.le.oldleft-2) THEN
                IF (down) THEN
                  spanrow(numspans) = sj
                ELSE
                  spanrow(numspans) = -sj
                END IF
                spanleft(numspans) = left
                spanright(numspans) = oldleft - 2
                numspans = numspans + 1
                IF (numspans.gt.maxspan) THEN
                  maxspan = 1.5 * maxspan
                  CALL reallocate_1d(spanrow,maxspan)
                  CALL reallocate_1d(spanleft,maxspan)
                  CALL reallocate_1d(spanright,maxspan)
                END IF
              END IF
            END IF
          END IF      

!     Scan right of prior scan end point from prior row to create new up and down scans    
          IF (inscoop(right,sj).eq.0) THEN
            CALL Nodeinout(right,sj)
            IF (icont.eq.1) THEN
              oldright = right
              DO WHILE (right.lt.i2)
                IF (inscoop(right+1,sj).ne.0) EXIT
                CALL Nodeinout(right+1,sj)
                IF (icont.eq.0) EXIT
                right = right + 1
              END DO
              
!     Check whether new line has continuous nodes more than two to the right of last span.
!     If so then there is a new span direction opposite of current direction.            
              IF (right.ge.oldright+2) THEN
                IF (down) THEN
                  spanrow(numspans) = sj
                ELSE
                  spanrow(numspans) = -sj
                END IF
                spanleft(numspans) = oldright + 2
                spanright(numspans) = right
                numspans = numspans + 1
                IF (numspans.gt.maxspan) THEN
                  maxspan = 1.5 * maxspan
                  CALL reallocate_1d(spanrow,maxspan)
                  CALL reallocate_1d(spanleft,maxspan)
                  CALL reallocate_1d(spanright,maxspan)
                END IF                
              END IF
            END IF
          END IF 
          
!     Find spans spans in up or down direction from left to right and mark inscoop array
          DO WHILE (left.le.right)
            IF (inscoop(left,sj).eq.0) THEN
              CALL NodeTest(left,sj)
              IF (icont.ne.0) THEN
!     Left is the start of a down (or up) scan
                oldleft = left
                DO WHILE (left.lt.right)
                  IF (inscoop(left+1,sj).ne.0) EXIT
                  CALL NodeTest(left+1,sj)
                  IF (icont.eq.0) EXIT
                  left = left + 1
                END DO
!     Check new span in the current up or down direction
                IF (down) THEN
                  spanrow(numspans) = -sj
                ELSE
                  spanrow(numspans) = sj
                END IF
                spanleft(numspans) = oldleft
                spanright(numspans) = left
                numspans = numspans + 1
                IF (numspans.gt.maxspan) THEN
                  maxspan = 1.5 * maxspan
                  CALL reallocate_1d(spanrow,maxspan)
                  CALL reallocate_1d(spanleft,maxspan)
                  CALL reallocate_1d(spanright,maxspan)
                END IF                
                left = left + 1
              END IF
            END IF
            left = left + 1
          END DO
        END DO
        
        END SUBROUTINE Floodfill
