MODULE CommonData
!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
! Used in all subroutines to control precision of all real variables.
!
!        VARIABLES
!
!        bfile - location in character string of first character of 
!           input file name prefix, defined by searching from the
!           end of the string for the occurrence of a slash character 
!        filin -- name of primary input file of parameters
!        int1 -- single byte integer
!        inull -- integer null value used throughout program
!        nfile -- location in character string of last character of 
!           input file name prefix, defined by searching from the 
!           end of the string for the occurrence of a period
!        nrad -- multiplier for radius to continue searching beyond last
!           radius with valid subsets even when no smaller subsets exist
!        nullhi -- positive high null value used throughout program
!        outputdir -- optional directory path to place output files into
!        dp -- defines double precision for real variables
!        rnull --  real null value used throughout program
!        slipdirtol -- FOS tolerance for stopping search for minimum FOS slide direction.
!            Default tolerance is 0.5% change in FOS.
!
!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

  SAVE
  INTEGER, PARAMETER :: dp = SELECTED_REAL_KIND(15,307)
  INTEGER, PARAMETER :: int1 = SELECTED_INT_KIND(1)
  INTEGER :: nfile,bfile
  INTEGER, PARAMETER :: inull=-9999
  
! Change nrad to continue increasing radius to nrad*last radius that had
!  resulted in valid slip surfaces regardless of whether there are any
!  additional subsets that have not yet hit size range.  Can increase
!  runtime significantly.
  REAL(dp), PARAMETER :: nrad = 0.0_dp
  REAL(dp), PARAMETER :: slipdirtol = 0.005_dp
  REAL(dp), PARAMETER :: rnull=-9999.0_dp,nullhi=9999.0_dp,rnull2=-10000.0_dp
  CHARACTER(220) :: filin,outputdir

END MODULE CommonData


MODULE GridData

!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
! Used in subroutines readin, readdem, readpiezo, readpress, readstrat,
!  radius, subsets, volumes, fos, bishop, fellenius, 
!  checksets, newnodes, readstrength, and writeout.
!
!        VARIABLES
!
!        angle -- assumed angle of slip (azimuth)
!        cellarea -- area of DEM grid cell (delxy*delxy)
!        delxy -- DEM grid resolution (delta x, delta y)
!        delxyvol -- 1/12 of delxy used in column volume calculation
!        delz -- z resolution of search grid
!        demflag -- indicates whether file replacedem will be generated.
!        dz -- vertical distance from ground-centric search cell elevation to spheroid center
!        halfdelxy -- 1/2 DEM grid spacing
!        iretro -- counter for number of retrogressions
!        lastrad -- previous radius before increase by dr
!        lengthunits -- units of length, used for labels in output files 
!        nbdy(nx+1,ny+1) -- array of flags for identifying nodes adjacent to 
!           DEM boundary cells
!        nx -- number of DEM cells in x direction
!        ny -- number of DEM cells in y direction
!        nz -- number of nodes used for 3-D FOS values when isubsurf=1or2
!        oldzcen -- elevation of previous search center
!        rad -- search spheroid axes perpendicular to strike of slope
!        radsq -- square of search spheroid axes perpendicular to strike of slope
!        xcen,ycen,zcen -- center of search spheroid relative to DEM origin
!        xcenrot,ycenrot,zcenrot -- rotational center of spheroid if different
!          from geometric center.
!        xll,yll -- x and y origin of DEM grid, read in header lines
!        xllcorner,yllcorner -- character version of DEM grid origin header lines
!        zdem(nx,ny) -- DEM elevations
!        zdemnodes(nx+1,ny+1) -- DEM node elevations averaged from surrounding cells
!        zmin -- minimum DEM elevation
!        zmax -- maximum DEM elevation
!
!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  USE CommonData, ONLY: dp,int1
  IMPLICIT NONE
  SAVE
  
  INTEGER :: nx,ny,nz,demflag,iretro
  
  REAL(dp) :: delxy,halfdelxy,xcen,ycen,zcen,zmin,delz,invdelxy,cmang,rad2d,rad2dsq
  REAL(dp) :: rad,radsq,angle,zmax,xll,yll,lastrad,oldzcen,dz,angslope
  REAL(dp) :: xcenrot,ycenrot,zcenrot,cellarea,Piover180,invPiover180,Piover2
  REAL(dp) :: halfcellarea,quartercellarea,threequartercellarea,delxyvol
  
  CHARACTER(2) :: lengthunits
  CHARACTER(60)  :: xllcorner,yllcorner
  
  REAL(dp), PARAMETER :: pi = 3.1415926535897932384626_dp,inv3 = 1._dp/3._dp
  REAL(dp), PARAMETER :: twoinv3 = 2._dp/3._dp,exp1 = EXP(1._dp),inv12 = 1._dp/12._dp
  REAL(dp), PARAMETER :: fourpiover3 = 4._dp*pi/3._dp,twopi = 2._dp*pi,inv6 = 1._dp/6._dp
  
  INTEGER(int1), ALLOCATABLE :: nbdy(:,:)
  REAL(dp), ALLOCATABLE :: zdem(:,:),zdemnodes(:,:)
  
END MODULE GridData

  

MODULE MaterialData

!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
! Used in subroutines readin, readstrat, fos, bishop, and fellenius. 
!
!        VARIABLES
!
!        activelay(nmat-1) -- number of active (nonnull) cells in each layer
!        cee(nmat) -- cohesion of each layer
!        duwtlaydz(nx,ny,nmat-1) -- gradient of depth-weighted unit weights between layers
!        eqcoef -- seismic coefficient coefficient
!        fxa(nmat) -- curve-fitting parameter, a from the equation for the soil-water characteristic 
!            curve defined by Fredlund and Xing (1994) 
!        fxn(nmat) -- curve-fitting parameter, n from the equation for the soil-water characteristic 
!            curve defined by Fredlund and Xing (1994) 
!        fxm(nmat) -- curve-fitting parameter, m from the equation for the soil-water characteristic 
!            curve defined by Fredlund and Xing (1994) 
!        fxr(nmat) -- curve-fitting parameter, psi_r (soil suction associated with residual moisture 
!            content) from the equation for the soil-water characteristic curve defined by 
!            Fredlund and Xing (1994)
!        gamr(nmat,2) -- array of total, partially saturated, and saturated
!           unit weights for each layer
!        gamw -- water unit weight in units of problem 
!        gsameall -- flag indicating whether all unit weights are equivalent
!        gsameeach -- flag indicating whether unit weights in each layer are equivalent
!           to each other
!        layer(nmat,nx,ny) -- array of bottom elevations for material layers
!        maxlayer(nmat-1) --maximum value of each stratigraphic layer file
!        minlayer(nmat-1) -- min value of each stratigraphic layer file
!        nmat -- number of material layers
!        ru(nmat) -- pore pressure ratio approximation
!        tanphi(nmat) -- tangent of friction angle for each layer
!        thetares(nmat) -- residual water content for each layer
!        thetasat(nmat) -- saturated water content for each layer
!        uwtlay(nx,ny,nmat-1) -- depth-weighted average unit weight at each layer
!           bottom with correct moist for water conditions
!        vga(nmat) -- curve-fitting parameter, alpha from the equation for the 
!           soil-water characteristic curve defined by vanGenuchten (1980) 
!        vgn(nmat) -- curve-fitting parameter, n, from the equation for the soil-water 
!            characteristic curve defined by vanGenuchten (1980) 
!
!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  USE CommonData, ONLY: dp
  IMPLICIT NONE
  SAVE
  
  INTEGER :: gsameall,gsameeach,nmat
  INTEGER, ALLOCATABLE :: activelay(:)
  
  REAL(dp) :: gamw,eqcoef,eqland,eqmin,eqmax
  REAL(dp), ALLOCATABLE :: layer(:,:,:),uwtlay(:,:,:),cee(:),tanphi(:),invlogfxr(:)
  REAL(dp), ALLOCATABLE :: gamr(:,:),ru(:),duwtlaydz(:,:,:),thetasat(:),vgnexp(:)
  REAL(dp), ALLOCATABLE :: minlayer(:),maxlayer(:),thetares(:),satresdiff(:)
  REAL(dp), ALLOCATABLE :: fxa(:),fxn(:),fxm(:),fxr(:),vga(:),vgn(:),eqarray(:,:)
END MODULE MaterialData

  
  
MODULE WaterData

!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
! Used in subroutines readin, readpiezo, readpress, readstrat, fos, 
!  bishop, and fellenius.
!
!        VARIABLES
!
!        delzp -- vertical resolution of pressure head data
!        dpdz(nx,ny,pmaxk) -- array of pressure head gradients
!        dsedz(nx,ny,pmaxk) -- array of effective saturation gradients
!        iwater -- flag indicating method for specifying water pressure
!           0 = no water pressures, 
!           1 = ru approximation, 
!           2 = piezometric surface, 
!           3 = 3D pressure head file, 
!           4 = 3D variably saturated file containing pressure head and water content, 
!           5 = 3D variably saturated file, containing pressure head, water content 
!               will be calculated from vanGenuchten SWCC, 
!           6 = 3D variably saturated file containing pressure head, water content 
!               will be calculated from Fredlund and Xing SWCC, 
!           7 = 3D pore pressure file,
!           8 = 3D variably saturated file containing pore pressure and water content, 
!           9 = 3D variably saturated file, containing pore pressure, water content 
!               will be calculated from vanGenuchten SWCC, 
!           10 = 3D variably saturated file containing pore pressure, water content 
!               will be calculated from Fredlund and Xing SWCC
!        iwaterset -- flag for whether water columns are found within slip surface
!        maxpk(nx,ny) -- array of highest k value of pressure data at each DEM cell
!        maxtheta -- maximum water content for variably saturated conditions
!        mintheta -- minimum water content for variably saturated conditions
!        moist -- determines which element of soil unit weight array to use: 1-total weight
!           2-partially saturated, 3-fully saturated.
!        pgrid -- flag for pressure head grid, 1=regular, 2=irregular grid
!        piezo(nx,ny) -- array of piezometric elevation at each DEM cell
!        pmaxk -- maximum number of 3-d pressure values at each cell
!        press(nx,ny,pmaxk) -- array of pressure head data
!        pressz(nx,ny,pmaxk) -- array of z elevations of pressure head data
!        pzsurf -- flag for whether piezometric surface was found,1=yes,0=no
!        se(nx,ny,pmaxk) -- array of effective saturation 
!        thetaz(nx,ny,pmaxk) -- array of volumetric water content
!        zpmin -- minimum elevation of pressure head values
!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  USE CommonData, ONLY: dp
  IMPLICIT NONE
  SAVE
  
  LOGICAL :: varsat,swcc
  
  INTEGER :: iwater,pzsurf,pgrid,pmaxk,moist
  INTEGER, ALLOCATABLE :: maxpk(:,:)
  
  REAL(dp):: delzp,zpmin,maxtheta,mintheta
  
  REAL(dp), ALLOCATABLE :: press(:,:,:)
  REAL(dp), ALLOCATABLE :: piezo(:,:),dpdz(:,:,:),pressz(:,:,:)
  REAL(dp), ALLOCATABLE :: se(:,:,:),dsedz(:,:,:)
END MODULE WaterData

  

MODULE StrengthData

!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
! Used in subroutines readin, readstrength, bishop, and fellenius.
!
!        VARIABLES
!
!        cflag -- indicates whether 3-d cohesion data are provided; 0=no,1=yes
!        ceect -- total number of cohesion values read
!        ceeunits -- units for cohesion, used for labels in output files
!        cohes(nx,ny,strmaxk) -- array of 3-d cohesion data
!        dcohdz(nx,ny,strmaxk) -- array of cohesion gradients between vertical nodes
!        dflag -- indicates whether 3-d unit weight data are provided; 0=no,1=yes
!        dfricdz(nx,ny,strmaxk) -- array of friction gradients between vertical nodes
!        duwtdz(nx,ny,strmaxk) -- array of unit weight gradients between vertical nodes 
!        dzstr -- z spacing of 3-d strength data
!        fflag -- indicates whether 3-d friction data are provided; 0=no,1=yes
!        fricct -- total number of friction angles read
!        linterp -- linear interpolation flag; 1=use linear interpolation, other=use
!          nearest node below slip surface base for strength data.
!        maxstrk(nx,ny) -- array of highest k value of strength data at each DEM cell
!        mincee,maxcee -- overall min and max cohesion
!        minfric,maxfric -- overall min and max friction angles
!        minuwt(2),maxuwt(2) -- overall min and max unit weights for each saturation
!        str3d -- flag for using 3-d strength file, 1=3D strengths
!        strgrid -- flag for strength data grid; 1=regular, 2=irregular grid
!        strmaxk -- number of 3-d strength values at each cell
!        strz(nx,ny,strmaxk) -- array of z locations of irregular strength data
!        tfric(nx,ny,strmaxk) -- array of 3-d friction data
!        uwt(nx,ny,strmaxk,2) -- array of unit weights
!        uwtct -- total number of unit weights read
!        uwtunits -- units for unit weight, used for labels in output files
!        uwt3d(nx,ny,strmaxk) -- unit weight at each cell (depth weighted average)
!          with correct moist for water conditions
!        zstrmin -- minimum elevation of strength data
!
!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  USE CommonData, ONLY: dp
  IMPLICIT NONE
  SAVE
  
  INTEGER :: str3d,cflag,fflag,dflag,strgrid,linterp
  INTEGER :: ceect,fricct,uwtct,strmaxk
  INTEGER, ALLOCATABLE :: maxstrk(:,:)
  
  REAL(dp) :: zstrmin,dzstr
  REAL :: mincee,maxcee,minfric,maxfric,minuwt(2),maxuwt(2)
  
  REAL(dp), ALLOCATABLE :: cohes(:,:,:),tfric(:,:,:)
  REAL(dp), ALLOCATABLE :: strz(:,:,:),dcohdz(:,:,:),dfricdz(:,:,:),duwtdz(:,:,:)
  REAL(dp), ALLOCATABLE :: uwt3d(:,:,:),uwt(:,:,:,:)
  
  CHARACTER(8) :: ceeunits, uwtunits
  
END MODULE StrengthData
 


MODULE SearchData

!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
! Used in subroutines readin, radius, criteria, subsets, and fos.
!
!        VARIABLES
!
!        armin,armax -- minimum and maximum area bounds for valid slip surface
!        critmet -- flag for meeting size/depth criteria, 1 = depth is met, 2 = primary
!           size is met, 3 = depth and primary size are met
!        fostol -- maximum percent difference in FOS between seed iterations to
!            not generate a new seed
!        goal -- vmin or armin + half of tol
!        icen,jcen -- center of search spheroid in i,j coordinates for lattice search
!        iradflag -- indicates whether valid initial range subset has been found.
!           (1=no, 0=yes, 2=no valid sets after 10 radius adjustments.)
!        irefine -- flag for implementing coarse to fine search iterations
!        islipdirflag -- flag for automated search for minimum FOS slide direction
!        ismin,ismax -- minimum and maximum boundaries of search grid on x-axis,
!           specified relative to DEM columns i=1 through nx
!        isqout -- flag for creating search quality output files, 1=create files
!        isrchmin,isrchmax -- array bounds of searchout output file.
!        jsmin,jsmax -- minimum and maximum boundaries of search grid on y-axis,
!           specified relative to DEM rows j=1 through ny
!        jsrchmin,jsrchmax -- array bounds of searchout output file
!        ksmax -- maximum boundary of search grid on z-axis calculated from 
!           user-specified zsmax and z resolution
!        linenum(nx,ny) -- line number of search center file associated with critical center
!           at DEM cell i,j
!        multres -- resolution multiplier for coarse search lattice
!        nkseed(nxsrch,nysrch) -- number of k values for current seed at each i,j
!           search node
!        nrec -- number of search centers written to file if ilattice or icritlattice = 1
!        nsrchpt -- total number of search points
!        nsrchres -- resolution of search grid relative to DEM resolution.  Acts
!           as a multiplier of DEM resolution delxy
!        nxsrchout,nysrchout -- number of search nodes in x and y directions in output
!           search grid file, includes intersection of search range with DEM range.
!           only, 0 = primary and secondary criteria
!        radcrit -- minimum radius to meet current size/depth criteria
!        radiniprev -- used in determination of first radius estimate at new search
!        searchgrid - array to specify search grid points
!           which grid points were searched and whether they fell inside or outside
!           the DEM.
!        searchout -- output array of area covered by search grid and indicating
!           which grid points were searched and whether they fell inside or outside
!           the DEM.
!        srchfile -- flag indicating whether using search file to specify valid
!           search grid nodes, 1=search file provided
!        srchlatt(nxsrch,nysrch,nzsrch+1) -- search lattice array with seed
!           iteration number at each searched node. 
!        slipdirflag -- flag for automated search for minimum FOS slide direction
!        tol -- tolerance on primary control criterion for calculating initial 
!           radius.  (i.e. initial volume must fall between volume and volume+tol)
!        va1max -- max. volume or area (primary size criterion) of a subset
!        va2max -- max. volume or area (secondary size criterion) of a subset
!        vadcrit -- indicates primary control for initial radius (1 for volume,
!           2 for area, and 3 for depth)
!        vmin,vmax -- minimum and maximum volume bounds for valid slip surface
!        zd -- elevation of DEM grid at same x,y as spheroid center
!        zsave -- representative distance from spheroid center to ground that yields
!           good initial volume or area
!        zsmin,zsmax -- minimum and maximum elevations of search grid
!        zsrchres -- resolution of search grid on z-axis
!
!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  USE CommonData, ONLY: dp,int1
  IMPLICIT NONE
  SAVE

  INTEGER :: ismin,ismax,jsmin,jsmax,nsrchpt,vadcrit,critmet
  INTEGER :: ksmax,isrchmin,jsrchmin,isrchmax,jsrchmax
  INTEGER :: nxsrchout,nysrchout,nsrchres,multres
  INTEGER :: srchfile,isqout,irefine
  INTEGER :: iradflag,icen,jcen,ncount,nrec,islipdirflag
  INTEGER, ALLOCATABLE :: nkseed(:,:),searchgrid(:,:)
  INTEGER, ALLOCATABLE :: srchlatt(:,:,:),linenum(:,:)
  INTEGER(int1), ALLOCATABLE :: searchout(:,:)
  
  REAL(dp) :: armin,armax,vmin,vmax,tol,zsmin,zsmax,zsrchres,goal,fostol
  REAL(dp) :: zd,va1max,va2max,rmax,radcrit,radiniprev,zsave,zzsrchres

END MODULE SearchData


MODULE FOSData

!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
! Used in subroutines readin, fos, and writeout.
!
!        VARIABLES
!        absminma -- abs. value of minimum m-alpha allowed in Bishop FOS calculation
!        arclength -- arc length of current 2d slice
!        area2d -- cross-sectional area of current 2d slice
!        dr -- radius increment for search
!        diter -- convergence requirement in terms of percent difference between
!           last two iterations for factor of safety using Bishop's Method 
!        felfoso -- Ordinary Method factor of safety of current scoop
!        felfsmin(nx,ny) -- array of minimum Ordinary Method FOS at each DEM cell
!        felminfos -- overall minimum FOS using Ordinary Method
!        fgrid -- minimum factor of safety at search grid point
!        filtcount(nx,ny) -- counter of number of times each cell was filtered by
!           minma.
!        filter -- indicates whether solution filters are applied (malpha and fos)
!        ifos2d -- flag for calculating 2-d factors of safety
!        foscut -- factor of safety cutoff for removing slip surface from DEM
!        foslocal -- flag for writing local factors of safety to file, only
!           available for single failure option
!        foso -- factor of safety of current scoop
!        foso2d -- 2-d factor of safety of current scoop
!        fsangle(nx,ny) -- array of fall angles of minimum scoop at each DEM cell
!        fsangle2d(nx,ny) -- array of fall angles of minimum 2d slice at each DEM cell
!        fsarclength(nx,ny) -- array of arclengths of minimum 2d slice at each DEM cell
!        fsarea2d(nx,ny) -- array of areas of minimum 2d slice at each DEM cell
!        fsdepth(nx,ny) -- array of maximum depth of minimum scoop at each DEM cell
!        fsdepth2d(nx,ny) -- array of maximum depth of minimum 2d slice at each DEM cell
!        fsmin(nx,ny) -- array of minimum FOS at each DEM cell
!        fsminold(i,j) -- last minimum factor of safety at each DEM cell used to
!           compare change in FOS during seed iterations to see if criterion is met
!        fsmin2d(nx,ny) -- 2-d minimum factor of safety at each DEM cell
!        fsmin2d3d(nx,ny) -- 3-d minimum factor of safety associated with min 2d slice
!           at each DEM cell
!        fsmin3d2d(nx,ny) -- 2-d minimum factor of safety associated with min 3d scoop
!           at each DEM cell
!        fsrad(nx,ny) -- array of radius of minimum scoop at each DEM cell
!        fsrad2d(nx,ny) -- array of radius of minimum 2d slice at each DEM cell
!        fsvol(nx,ny) -- volume of minimum FOS scoop at each DEM cell
!        fswidth2d(nx,ny) -- array of widths of minimum 2d slice at each DEM cell
!        fsx(nx,ny),fsy(nx,ny),fsz(nx,ny) -- search spheroid center of minimum FOS 
!           scoop at each DEM cell
!        fsx2d(nx,ny),fsy2d(nx,ny),fsz2d(nx,ny) -- search spheroid center of minimum FOS 
!           2d slice at each DEM cell
!        icritlattice -- flag for creating file critfoslattice_out.3D for minimum FOS of 
!          critical surfaces at each search  grid node
!        ilattice -- flag for creating file foslattice_out.3D for minimum FOS at each search
!           grid node
!        irelfos -- flag for creating relative factor of safety grids,1=create files
!        isizefiles -- flag for writing volume, area, and depth files for min FOS at each 
!           DEM column
!        isubsurf -- flag for creating 3-d file of FOS below DEM surface, 1=create file&
!          1=create file in ijk format, 2=create file in xyz format
!        icrit(nx) -- array of volume or area flags indicating proximity to limits
!            of criteria range for each DEM cell
!        limcol -- optimal number of columns per slip surface.  Fewer columns
!           will generate error in ncolerr.out file
!        linein(nx,ny) -- array of flags for whether 2-D slide directon line intersects
!           column
!        mcol(nx,ny) -- number of columns in minimum FOS scoop at each DEM cell
!        mcol2d(nx,ny) -- number of columns in minimum 2d FOS slice at each DEM cell
!        mfile -- flag indicating whether filter file was opened
!        method -- B for Bishops Simplified, F for Fellenius method for
!           calculation of factor of safety.
!        tanmindip -- minimum apparent dip of slip surface column base
!        minma -- min absolute value of m-alpha of all iterations for a surface
!        ncol2d -- number of columns in current 2-D slice
!        nretro -- number of failure retrogressions to compute
!        ntry -- total number of slip surfaces for which FOS is calculated
!        oangle -- slip angle associated with overall minimum factor of safety
!        oangle2d -- slip angle associated with overall minimum 2d factor of safety
!        oarclength -- arc length of overall minimum 2d factor of safety
!        oarclength3d2d -- arc length of 2d surface associated with overall min 3d scoop
!        oarea -- area of overall minimum scoop
!        oarea2d -- area of overall minimum 2d slice
!        oarea2d3d -- area of 3d surface associated with overall min 2d slice
!        oarea3d2d -- area of 2d surface associated with overall min 3d scoop
!        ocol -- number of columns in overall minimum scoop
!        ocol2d -- number of columns in overall minimum 2-D scoop
!        ocol2d3d -- number of columns in 3d surface associated with min 2d slice
!        ocol3d2d -- number of columns in 2d surface associated with min 3d scoop
!        odepth -- maximum depth of overall minimum scoop
!        odepth2d -- maximum depth of overall minimum 2-D scoop
!        odepth2d3d -- maximum depth of 3d surface associated with min 2d slice
!        odepth3d2d -- maximum depth of 2d surface associated with min 3d scoop
!        ofos -- overall minimum factor of safety
!        ofos2d -- 2-d FOS associated with overall minimum 3-d factor of safety
!        ofos2d3d -- FOS of 3d slice associated with overall minimum 2d FOS
!        ofos3d2d -- FOS of 2d slice associated with overall minimum 3d FOS
!        ofsx,ofsy,ofsz -- search spheroid center of minimum FOS scoop
!        ofsx2d,ofsy2d,ofsz2d -- search spheroid center of minimum 2d FOS scoop
!        ominma -- overall min absolute value of m-alpha of all iterations for a surface
!        omset -- set number of overall minimum scoop
!        orad -- radius of sphere associated with overall minimum factor of safety
!        orad2d -- radius of arc associated with overall minimum 2d factor of safety
!        osliparea -- slip surface area of overall min 3d scoop
!        osliparea2d3d -- slip surface area of 3d slice associated with overall minimum 2d FOS
!        ovangle -- slip angle of largest volume scoop with FOS < cutoff
!        ovarea -- area of largest volume scoop with FOS < cutoff
!        ovdepth -- depth of largest volume scoop with FOS < cutoff
!        ovmset -- set number of largest volume scoop with FOS < cutoff
!        ovfos -- factor of safety of largest volume scoop with FOS < cutoff
!        ovrad -- radius of largest volume scoop with FOS < cutoff
!        ovfsx,ovfsy,ovfsz -- search spheroid center of largest volume scoop with 
!           FOS < cutoff
!        ovol, oarea - volume and area of minimum  FOS scoop
!        ovol2d3d -- volume of 3d surface associated with overall min 2d slice
!        ovsliparea -- slip surface area of largest volume scoop with 
!           FOS < cutoff
!        ovvol -- volume of largest volume scoop with FOS < cutoff
!        ovwt -- weight of largest volume scoop with FOS < cutoff
!        owidth2d -- wisth of overall min 2d FOS
!        owidth3d2d -- width of 2d surface associated with overall min 3d scoop
!        owt -- weight of overall min 3d scoop
!        owt2d3d -- weight of 3d slice associated with overall minimum 2d FOS
!        ozb(nx,ny) -- array of DEM after scoops < FOS cut off removed.
!        remove -- A= all scoops < FOS cutoff will be removed from new
!           DEM.  L= only largest volume scoop < FOS cutoff will be 
!           removed. M= the scoop with the minimum FOS will be removed,
!           N= no scoops removed.
!        single -- flag for calculating single slip surface
!        width2d -- width of current 2d slice
!        xcount -- number of surfaces eliminated due to m-alpha limits and/or nonconvergence
!        zbot -- elevation of bottom of 3-d fos grid array
!        zfos(nx,ny,nz) -- 3-d array of minimum factor of safety below DEM surface
!
!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  USE CommonData, ONLY: dp,int1
  IMPLICIT NONE
  SAVE

  CHARACTER(1) :: remove,method
  INTEGER :: nretro,icritlattice,ilattice,single,foslocal,nangle,newangle
  INTEGER :: filter,isubsurf,irelfos,mfile,ifos2d,omset,ovmset,isizefiles
  INTEGER :: limcol,xcount,ocol2d,ncol2d,ocol3d2d,ocol2d3d,ocol,ntry
  INTEGER(int1), ALLOCATABLE :: linein(:,:)
  INTEGER, ALLOCATABLE :: mcol(:,:),mcol2d(:,:),filtcount(:,:),icrit(:)
  
  REAL(dp) :: dr,ofos,foso,ovvol,foscut,zbot,diter,ofos2d,felfoso,felminfos
  REAL(dp) :: absminma,minma,tanmindip,arclength,area2d,width2d,foso2d
  REAL(dp) :: orad,ovrad,ovfsx,ovfsy,ovfsz,ofsx,ofsy,ofsz,fgrid,depth2d
  REAL :: ominma,ovwt,ovsliparea,osliparea2d3d,owt2d3d
  REAL :: ofos2d3d,ofos3d2d,oarea3d2d,oarclength3d2d
  REAL :: owidth3d2d,oarea2d3d,ovol2d3d,owidth2d,osliparea,owt
  REAL :: oangle,ovarea,ovol,oarea,ovangle
  REAL :: ovfos,orad2d,oangle2d,ofsx2d,ofsy2d,ofsz2d,oarea2d,oarclength
  REAL :: odepth,odepth2d,odepth2d3d,odepth3d2d,ovdepth
  
  REAL(dp), ALLOCATABLE :: fsmin(:,:),fsminold(:,:),ozb(:,:)
  REAL(dp), ALLOCATABLE :: fsrad(:,:),fsx(:,:),fsy(:,:),fsz(:,:),fsangle(:,:)
  REAL, ALLOCATABLE :: fsrad2d(:,:),fsangle2d(:,:),fswidth2d(:,:),fsarea2d(:,:)
  REAL, ALLOCATABLE :: fsx2d(:,:),fsy2d(:,:),fsz2d(:,:),fsmin2d3d(:,:),fsmin3d2d(:,:)
  REAL, ALLOCATABLE :: fsarea(:,:),fsmin2d(:,:),zfos(:,:,:),felfsmin(:,:)
  REAL, ALLOCATABLE :: fsvol(:,:),fsarclength(:,:),fsdepth2d(:,:),fsdepth(:,:)
  
END MODULE FOSData


MODULE SetData

!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
! Used in subroutines subsets, volumes, criteria, fos, bishop, fellenius
! checksets, and newnodes.
!
!        VARIABLES
!
!        area(nnset) -- surface area of subset
!        colfile -- flag for opening error file reporting cols < limcol
!        cosang -- cosine of slip direction angle
!        goodset -- flag for finding valid subset
!        i1,i2,j1,j1 -- array bounds for current set
!        icont -- flag for whether node is included in spheroid during flood fill routine
!        maxspan -- number of spans used in floodfill routine used to dimension arrays
!        mini(nnset),maxi(nnset) -- minimum and maximum i bounds of each subset
!        minj(nnset),maxj(nnset) -- minimum and maximum j bounds of each subset
!        mset -- current land subset number
!        ncol(nnset) -- number of columns in subset
!        nnset -- maximum number of land subsets
!        nnwaterset -- maximum number of non-land subsets
!        nodecount -- number of land nodes contained in spheroid
!        nrange(nnset) -- array of flags for whether set fits volume or area criteria 
!           range, 1=fits criteria range
!        nset -- number of land subsets found
!        nsetmax -- flag for exceeding number of land subsets allowed by nnset
!        numset -- current subset number
!        omini,omaxi,ominj,omaxj -- minimum and maximum bounds of all sets included
!           in search spheroid
!        oomini,oomaxi,oominj,oomaxj -- minimum and maximum bounds of all DEM cells 
!           included in any spheroid with a calculated FOS value
!        set(nnset) -- number of columns in a contiguous set
!        setflag(nnset) -- indicates valid subset (0), or invalid by containing
!           truncated node (1), or adjacent to DEM boundary (2), or empty (-1) 
!        sinang -- sin of slip direction angle
!        sliparea -- area of slip surface
!        spanleft,spanright -- arrays containing bounds of continuous set of included nodes 
!           to left or right of included nodes on previous row
!        spanrow -- array of positive or negative row count representing number of rows from
!           seed row, i,j
!        surfslope(nx,ny) -- slope of dem surface
!        volume(nnset) -- volume of subset
!        waterlevel(nnwaterset) -- water surface elevation of each water subset
!        weight -- weight of slide
!        xcolcount -- number of critical surfaces with total # of columns < limcol
!
!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  USE CommonData, ONLY: dp
  IMPLICIT NONE
  SAVE
  
  INTEGER, PARAMETER :: nnnset=25
  
  INTEGER :: omini,omaxi,ominj,omaxj,xcolcount,nnset,nnwaterset,nodecount,subsetwarn
  INTEGER :: nsetmax,colfile,i1,j1,i2,j2,numset,icont,maxspan
  INTEGER :: goodset,nstate,mset,nset,oomini,oomaxi,oominj,oomaxj,intersect
  INTEGER :: inscpp(4),ii1,ii2,jj1,jj2
  INTEGER, ALLOCATABLE :: ncol(:),nrange(:),spanrow(:),spanleft(:),spanright(:),contiguous(:,:)
  INTEGER, ALLOCATABLE :: mini(:),minj(:),maxi(:),maxj(:),set(:),setflag(:)

  REAL(dp),ALLOCATABLE :: area(:),volume(:)
  REAL(dp) :: sliparea,weight,cosang,sinang
  
END MODULE SetData

MODULE ColumnData

!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
! Used in subroutines newnodes, volumes, fos, bishop, ordinary, line2d,
! midpoint, intersection, and watercolvol
!
!        VARIABLES
!
!        colxy(nx,ny) -- length of each column side (delxy for full columns)
!        dx1,dx2,dy1,dy2 -- x and y grid spacing of partial column relative to inside nodes
!        incol - flag for whether column with surface water is empty (0) or on DEM boundary (-1)
!        inn(nx,ny) -- number of nodes of each column bounded by spheroid
!        inscoop(nx,ny) -- indicates subset membership of DEM column nodes
!        i2dcen,j2dcen -- i,j location of center of search spheroid intersection 
!           with DEM surface
!        line -- length of 2-D slip line in current column
!        mline -- slope of slip direction line relative to x and y grid
!        nodesin -- number of nodes of each column bounded by spheroid
!        nonode -- flag for when there is no intersection between search
!           spheroid and slope line between outside and inside nodes or partial
!           column is truncated.
!           If nonode = 1, new node can not be calculated.
!           If nonode = 2, truncated column.
!        onenode -- corner location of included column node for single node column
!        outnodes(2,nx,ny) -- corner location of column nodes which fall outside the
!           search spheroid, also used to indicate included node for single node columns
!        outnode1,outnode2 -- corner location of column nodes which fall outside the
!           search spheroid, also used to indicate included node for single node columns
!        parea -- area of column projected on DEM grid.  Parea equals delxy**2 
!             except for partial columns, which have a 3, 4, or 5 sided shape
!        subset(nx,ny) -- indicates set membership of DEM columns
!        tanalpha -- tangent of apparent dip at column base center
!        tandip2d -- tangent of true dip of 2d column base
!        vol(nx,ny) -- volume of column in subset
!        volcol -- volume of column including water and land
!        x2d, y2d -- x and y location of center of current 2d column
!        x1,x2,y1,y2 -- x and y locations of each column node
!        x2dcen,y2dcen -- perpendicular projection of x,y location of center of search spheroid  
!           intersection with DEM surface onto axis of rotation
!        xmid,ymid -- x and y locations of middle of column
!        xslope(nx,ny),yslope(nx,ny) -- x and y slope vectors of DEM surface
!        zb(nx,ny) -- base of slip surface at each column node
!        zbase(4) -- elevation of base of slip surface of each node in
!           column 
!        zmid(nx,ny) -- base of slip surface at midpoint of intersected column
!        zprime2d -- 2d z coordinate translated relative to spheroid center
!        ztop -- DEM elevation used as local variable
!
!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  USE CommonData, ONLY: dp
  IMPLICIT NONE
  SAVE
  
  INTEGER :: nodesin,nonode,outnode1,outnode2,onenode,incol,i2dcen,j2dcen
  INTEGER, ALLOCATABLE :: outnodes(:,:,:),subset(:,:),inscoop(:,:),inn(:,:)

  REAL(dp) :: zbase(4),znodes(4),dx1,dx2,dy1,dy2,volcol,tanalpha,dzdx,dzdy
  REAL(dp) :: xmid,ymid,ztop,x1,x2,y1,y2,parea,zmidbase
  REAL(dp) :: x2dcen,y2dcen,mline,line,zprime2d,tandip2d,x2d,y2d
  REAL(dp) :: xprime2d,xprime,yprime,zprime
  
  REAL(dp),ALLOCATABLE :: xslope(:,:),yslope(:,:)
  REAL(dp),ALLOCATABLE :: zmid(:,:),vol(:,:),colxy(:,:,:),zb(:,:)

END MODULE ColumnData


MODULE ResultsStats

!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
! Used in subroutine writeout
!
! Minimums and maximums of results arrays
!
!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  IMPLICIT NONE
  SAVE
  
  INTEGER :: numdatapts,minmcol,maxmcol,minfiltcount,maxfiltcount,minmcol2d,&
             maxmcol2d

  REAL :: minslope,maxslope,minfsmin,maxfsmin,minfsrelmin,maxfsrelmin,&
              minfsvol,maxfsvol,minfsarea,maxfsarea,minzdem,maxzdem,&
              minfsmin2d,maxfsmin2d,minfsrelmin2d,maxfsrelmin2d,minfsmin3d2d,&
              maxfsmin3d2d,minfsmin2d3d,maxfsmin2d3d, minfsz2d,maxfsz2d,&
              minfsx2d,maxfsx2d,minfsy2d,maxfsy2d,minfsrad2d,maxfsrad2d,&
              minfsangle2d,maxfsangle2d,minfsarclength,maxfsarclength,&
              minfswidth2d,maxfswidth2d,minfsz,maxfsz,minfsx,maxfsx,minfsy,&
              maxfsy,minfsrad,maxfsrad,minfsangle,maxfsangle,minfsarea2d,&
              maxfsarea2d,mincol2d,maxcol2d,minfelfsmin,maxfelfsmin,&
              minfsdepth,maxfsdepth,minfsdepth2d,maxfsdepth2d
  
END MODULE ResultsStats


MODULE BishopArrays

!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
! Used in subroutine Bishop
!
!        costruedip(nx,ny) -- cosine of the true dip of the sliding base at the
!           center of each column.
!        cosdip2d(nx,ny) -- cosine of dip in the slide direction at the center of 
!           each 2d column.
!        rf(nx,ny) -- resisting force of each column
!        rf2d(nx,ny) -- 2D resisting force of each column
!        sindip(nx,ny) -- sine of the dip of the sliding base in the direction of 
!           the slide at the center of each column.
!        sindip2d(nx,ny) -- sine of dip in the slide direction at the center of 
!           each 2d column. 
!
!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  USE CommonData, ONLY: dp
  IMPLICIT NONE
  SAVE
  
  REAL(dp),ALLOCATABLE :: rf(:,:),sindip(:,:),costruedip(:,:),rf2d(:,:)
  REAL(dp),ALLOCATABLE :: cosdip2d(:,:),sindip2d(:,:)

END MODULE BishopArrays

MODULE DepthData

!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
! Used in subroutines criteria, subsets, radius, readin
!
!        VARIABLES
!
!        adjustdepthrad -- flag indicating radius is being adjusted to meet the 
!           maximum depth criterion within specified tolerance (dtol)
!        depthmax(nnset) -- maximum column depth in each subset
!        depthmin -- minimum depth criterion used for initial radius estimate if vadcrit=3
!        depthrad(nnset) -- estimated radius to meet minimum or maximum depth criterion
!        dmax -- maximum depth criterion, scalar if > 0, array if < 0
!        dmaxarray(nx,ny) -- array of maximum depths for each DEM column
!        dmin -- minimum depth criterion, scalar if > 0, array if < 0
!        dminarray(nx,ny) -- array of minimum depths for each DEM column
!        dtol -- tolerance on depth criterion for calculating initial radius to meet
!           minimum depth or for meeting maximum depth if required
!        idepthflag -- flag indicating whether depth criteria entered as depth (1) 
!           or elevation using stratigraphic layer (2)
!        imaxdepth -- flag for requiring a slip surface to fall within dtol of the
!           maximum depth
!        maxddiff(nnset) -- maximum difference between subset column bottoms and maximum depth 
!           criterion (negative if subset column bottom falls above maximum depth)
!        maxdepthrad -- radius calculated in Subsets to hit maximum depth critera
!        minddiff(nnset) -- maximum difference between subset column bottoms and minimum depth 
!           criterion(negative if subset column bottom falls below minimum depth)
!        nstate200 -- count of number of radius tries to hit minimum depth target
!        numdepth -- number of trial surfaces requested between min and max depth
!
!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  USE CommonData, ONLY: dp
  IMPLICIT NONE
  SAVE
  
  INTEGER :: nstate200,idepthflag,numdepth,adjustdepthrad,imaxdepth
  
  REAL(dp) :: dmin,dmax,dtol,maxdepthrad,depthmin

  REAL(dp), ALLOCATABLE :: dminarray(:,:),dmaxarray(:,:),depthrad(:)
  REAL(dp), ALLOCATABLE :: depthmax(:),minddiff(:),maxddiff(:)
  
END MODULE DepthData