        Subroutine Criteria(crit1,crit2,c1min,c1max,c2min,c2max,tol)
        
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!       This subroutine determines if intersected subsets meet volume and area 
!       constraints specified by user.  If constraints are met, factors of safety 
!       will be calculated next. Otherwise, the spheroid axes will be adjusted in 
!       a way that depends on the current circumstances.  This subroutine is used
!       only for the lattice search option. 
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!
!       Called by Scoops3D
!
!       VARIABLES
!
!        c1min -- min. area or volume for search, primary criterion
!        c1max -- max. area or volume for search, primary criterion
!        c2min -- min. area or volume for search, not primary criterion
!        c2max -- max. area or volume for search, not primary criterion
!        crit1(nset) -- variable for volume or area, whichever is 
!           primary criterion
!        crit2(nset) -- variable for volume or area, whichever is 
!           not primary criterion
!        ddiff -- flag indicating whether depth criteria are met if vadcrit = 3
!        depthdiff -- maximum difference between column bottoms and minimum depth criterion
!           of deepest subset(negative if column bottom falls below minimum depth)
!        depthmax(nnset) -- maximum column depth in each subset
!        depthrad(nnset) -- estimated radius to meet minimum or maximum depth criterion
!        dtol -- tolerance on depth criterion for calculating initial radius to meet
!           minimum depth or for meeting maximum depth if required
!        idepthflag -- flag indicating whether depth criteria entered as depth (1) 
!           or elevation using stratigraphic layer (2)
!        iradflag -- indicates whether valid initial range subset has been found.
!           (1=no, 0=yes, 2=no valid sets after 10 radius adjustments.)
!        m -- set number
!        maxddiff(nnset) -- maximum difference between subset column bottoms and maximum depth 
!           criterion (negative if subset column bottom falls above maximum depth)
!        mmax -- set number of subset with maximum primary criterion
!        minddiff(nnset) -- maximum difference between subset column bottoms and minimum depth 
!           criterion(negative if subset column bottom falls below minimum depth)
!        nin -- number of subsets that fall within criteria range
!        nrad -- multiplier for radius to continue searching beyond last
!           radius with valid subsets even when no smaller subsets exist
!        nrange(nnset) -- array of flags for whether set fits volume or area criteria 
!           range
!        nnset -- maximum number of subsets
!        nset -- number of land subsets found
!        nstate -- flag used to indicate whether valid sets
!           were found at a particular search node.
!           100 = recalculate radius to find first good set at a search node.
!           300 = increase radius by dr and continue. This flag occurs if good
!                 sets were found, or only sets not grown out of range are too
!                 small to be in range.
!           400 = met size constraints but contains truncated column
!           500 = go to next search grid point because no good sets possible.
!        nstate200 -- count of number of radius tries to hit minimum depth target
!        rad -- search spheroid axes perpendicular to strike of slope
!        radiniprev -- used in determination of first radius estimate at new search
!           lattice elevation. Equals previous radius  at current search lattice x-y  
!           location that met criteria constrains, or = -1 if radius not yet calculated.
!        radsave --  last good radius at this search location
!        setflag(ntotset) -- indicates valid subset (0), or invalid by containing
!           truncated node (1), or adjacent to DEM boundary (2), or empty (-1)
!        va1max -- maximum volume or area of subsets, whichever is 
!           primary criterion
!        va2max -- max area or volume of subsets, whichever is 
!           not primary criterion
!        vadcrit -- indicates primary control for initial radius (1 for volume,
!           2 for area, and 3 for depth)
!        zsave -- representative distance from spheroid center to ground that yields
!           good initial volume or area
!
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

        USE CommonData, ONLY: dp,nullhi,rnull,nrad
        USE GridData, ONLY: rad,radsq,pi,twopi,inv3
        USE SetData, ONLY: nrange,nnset,setflag,nstate,nset
        USE FOSData, ONLY: dr
        USE SearchData, ONLY: vadcrit,zsave,radiniprev,iradflag,va1max
        USE DepthData, ONLY: nstate200,depthrad,minddiff,maxddiff,depthmax,&
                           dtol,idepthflag
        
        IMPLICIT NONE
      
        INTEGER :: mmax,m,nin,setflg,ddiff
        REAL(dp), INTENT(in) :: crit1(nnset),crit2(nnset)
        REAL(dp), INTENT(in) :: c1min,c1max,c2min,c2max,tol
        REAL(dp) :: va2max,z1,z2,z3,theta,p,radsave,crt1,crt2
        REAL(dp) :: mxddiff,mnddiff
        
        SAVE radsave
        
        nstate = 0
        nin = -1
        mmax = 0
!     If initial radius not able to be found for minimum depth criterion
!     after 10 adjustments use last radius and continue.
        IF (nstate200.gt.10) THEN
          iradflag = 2
          nstate200 = 0
        END IF
        IF (nstate200.gt.0) nstate200 = nstate200 + 1

!     If at a new search point, or no valid initial criteria range radius
!     yet found, find subset with maximum primary criterion. This set
!     will be used to control the initial radius value for this
!     search node (unless another set outgrows this one during the 
!     initial radius adjustment cycle.)
        IF (iradflag.eq.0) THEN
          radsave = 0.0_dp
          zsave = -1._dp
!     If area or volume are primary, or depth criteria was met and trying 
!     to meet volume or area as secondary
          IF (vadcrit.ne.3) THEN
            mmax = MAXLOC(crit1(1:nset),1)
            va1max = crit1(mmax)        
            va2max = crit2(mmax)
            setflg = setflag(mmax)
!     If criteria 1 is too big and criteria 2 not too small
            IF ((va1max.gt.c1min+tol).and.(va2max.ge.c2min)) THEN 
!     If depth is not a criterion or not too shallow recalculate smaller radius
              IF (nstate200.eq.0) nstate = 100            
!     Check whether depth criteria are met if required
              IF (idepthflag.gt.0) THEN
!     If too shallow increase radius to depth radius
                IF (minddiff(mmax).gt.0.0_dp.and.minddiff(mmax).ne.nullhi) THEN
                  nstate = 200
                END IF
              END IF
            END IF
!     If criteria 1 is too small and criteria 2 not too big and
!     DEM boundary not included radius (nstate = 100)
            IF (va1max.lt.c1min.and.(va2max.le.c2max.or.c2max.eq.0.0_dp)) THEN
              IF (setflg.ne.2) THEN
                nstate = 100
!     Check whether depth criteria are met if required
                IF (idepthflag.gt.0) THEN
!     If too deep check for other subsets that may meet criteria and continue 
!     to increase radius by delr until no more possible good sets (nstate=300)
                  IF (maxddiff(mmax).gt.0.0_dp.and.maxddiff(mmax).ne.nullhi) THEN
                    nstate = 300
                  END IF
                END IF
              END IF
            END IF
          ELSE  !   depth is primary criterion
!     Find subset with greatest depth to control initial radius
            mmax = MAXLOC(depthmax(1:nset),1)        
            va1max = crit1(mmax)        
            va2max = crit2(mmax)
            setflg = setflag(mmax)
            mnddiff = minddiff(mmax)
            mxddiff = maxddiff(mmax)
!     If too deep to meet initial tolerance and volume/area not too small use smaller   
!     depth radius calculated in Subsets (nstate=200)
            IF (mnddiff.lt.-dtol) THEN  ! too deep
              IF ((va1max.ge.c1min).and.(va2max.ge.c2min)) nstate = 200
!     If too shallow 
            ELSE IF (mnddiff.gt.0._dp) THEN
!     If doesn't contain boundary and area/volume not too big set radius to value
!     calculated in subsets
              IF (((va1max.le.c1max).or.(c1max.eq.0.0_dp)).and.&
                  ((va2max.le.c2max).or.(c2max.eq.0.0_dp))) THEN
                IF (setflg.ne.2) THEN
                  nstate = 200
                END IF
              END IF
            END IF
          END IF
!     If minimum depth is a criteria and too deep or too shallow
!     set radius to value calculated in subsets.
          IF (nstate.eq.200) THEN
            IF (nstate200.eq.0) nstate200 = 1
            IF (depthrad(mmax).ne.rnull) THEN
              IF (rad.eq.depthrad(mmax)) THEN
                rad = rad + 0.1_dp*dtol
              ELSE
                rad = depthrad(mmax)
              END IF
            ELSE
!    depthrad could be rnull if no full columns were contained, so make radius bigger
              rad = rad + dr
            END IF
            RETURN
          END IF
        END IF  !  If iradflag=0
        
!     If initial criteria ranges are just right at this radius, or
!     can't be adjusted to make them right, or
!     prior good radius has been incremented by delr, or
!     radius has been adjusted > ncount (10) times and still can't hit
!     initial ranges, check for any and all valid sets between min                        
!     and max ranges. Set nrange=1 for all valid sets.
        IF (nstate.eq.0) THEN
          nin = 0
          DO m = 1,nset
            IF (idepthflag.gt.0) THEN
              mxddiff = maxddiff(m)
              mnddiff = minddiff(m)
            END IF
            crt1 = crit1(m)
            crt2 = crit2(m)
            setflg = setflag(m)
!     If slope of ellipsoid set is flat then go to next set.            
            IF (nrange(m).eq.-1) CYCLE
            nrange(m) = 0
            ddiff = 1
!     Check whether depth criteria are met if required
            IF (idepthflag.gt.0) THEN
              IF (mxddiff.le.0.0_dp.and.(mnddiff.le.0.0_dp.or.mnddiff.eq.nullhi)) THEN
                ddiff = 1  ! depth criteria met
              ELSE
                ddiff = 0
              END IF
            END IF
!     If area/volume criteria are met
            IF ((crt1.ge.c1min).and.((crt1.le.c1max).or.(c1max.eq.0.0_dp))) THEN
              IF ((crt2.ge.c2min).and.((crt2.le.c2max).or.(c2max.eq.0.0_dp))) THEN
                IF (ddiff.ne.0) THEN  !  depth criteria met
!     If first good radius found at this search center save information to speed 
!     up next radius calculation
                  IF (iradflag.ne.1.and.m.eq.mmax) THEN
                    IF (iradflag.eq.2) THEN
                      mmax = MAXLOC(crit1(1:nset),1)
                      IF (m.ne.mmax) CYCLE
                    END IF
                    iradflag = 1
                    nstate200 = 0
                    radiniprev = rad
                    IF (vadcrit.ne.3) THEN
!     If met initial criteria, calculate equivalent z for future radius calculations
!     at new search elevations or new axis ratios if area or volume are primary
                      IF (vadcrit.eq.1) THEN
                        p = 1._dp - 3._dp*va1max/(radsq*rad*twopi)
                        p = MIN(ABS(p),1._dp)
                        theta = ACOS(p)
                        z1 = -2._dp*rad * COS(inv3*theta)
                        z2 = -2._dp*rad * COS(inv3*(theta+twopi))
                        z3 = -2._dp*rad * COS(inv3*(theta-twopi))
                        zsave = 0.0_dp
                        IF (z1.gt.0._dp.and.z1.lt.rad) THEN
                          zsave = z1
                        ELSE IF (z2.gt.0._dp.and.z2.lt.rad) THEN
                          zsave = z2
                        ELSE IF (z3.gt.0._dp.and.z3.lt.rad) THEN
                          zsave = z3
                        END IF
                      ELSE
                        zsave = sqrt(MAX(radsq - va1max/(pi),0._dp))                       
                      END IF
                      IF (zsave.lt.0.0_dp) THEN
                        print *, 'error calculating zsave in Criteria'
                        Stop
                      END IF
                    END IF
                  END IF
                  IF (setflg.eq.0) THEN  !  does not contain DEM boundary or truncated column
!     Radius will now only change by increments of delr until out of range
!     or criteria cannot be met.
                    nrange(m) = 1
                    nin = nin + 1
                    radsave = rad
                  END IF
                END IF
              END IF
            END IF
          END DO
        END IF
           
!     If no sets fit range, check to see if there are any sets
!     that are too small and don't contain boundary nodes. 
!     If so, continue to increase radius by delr until no more
!     possible good sets (nstate=300) or radius is nrad times the
!     last radius with a good subset. 
!     If not, move to next search node (nstate=500).
        IF (nin.eq.0) THEN
          IF (iradflag.ne.1) THEN  ! largest subset could not fit criteria
            iradflag = 2
            radsave = 0._dp
          END IF
          nstate200 = 0
          IF (radsave.gt.0._dp.and.rad.lt.nrad*radsave) THEN
            nstate = 300
          ELSE
            nstate = 500
          END IF
          DO m = 1,nset
            IF (idepthflag.gt.0) THEN
              mxddiff = maxddiff(m)
              mnddiff = minddiff(m)
            END IF
            crt1 = crit1(m)
            crt2 = crit2(m)
            setflg = setflag(m)
            IF (idepthflag.eq.0) THEN
              IF ((crt1.le.c1max).and.((crt2.le.c2max).or.(c2max.eq.0.0_dp)).and.&
                  (setflg.ne.2)) THEN
                nstate = 300
              END IF
            ELSE
              IF ((mxddiff.ne.rnull.and.mxddiff.le.0.0_dp).and.&
                  ((crt1.le.c1max).or.(c1max.eq.0.0_dp)).and.&
                  ((crt2.le.c2max).or.(c2max.eq.0.0_dp)).and.&
                  (setflg.ne.2)) THEN
                nstate = 300
              END IF
            END IF
          END DO
        END IF           
                      
        END SUBROUTINE criteria
            
