        SUBROUTINE iwater4wt
             
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!        This subroutine calculates unit weights at 3D pressure file
!        locations for columns with an unsaturated portion.
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!
!       Called in Readin
!
!       VARIABLES
!

!        ddepth -- depth between last and current unit weight calculation elevations
!        depth -- depth from DEM to pressure data elevations
!        diff -- distance from pressure node to layer boundary
!        dsedz(nx,ny,pmaxk) -- array of effective saturation gradients
!        dthetazdz(nx,ny,pmaxk) -- array of volumetric water content gradients
!        duwtdz(nx,ny,strmaxk) -- array of vertical unit weight gradients
!        gamr(nmat,2) -- array of total, partially saturated, and saturated
!            unit weights for each layer 
!        i,j -- DEM grid array location
!        ktop -- max k value at particular i,j
!        l -- Material layer counter
!        layer(nmat,nx,ny) -- array of bottom elevations for material layers
!        maxpk(nx,ny) -- array of highest k value of pressure data at each DEM cell
!        pdepth -- depth between pressure data elevations
!        pmaxk -- number of 3-d pressure values at each cell
!        pressz(nx,ny,pmaxk) -- array of z elevations of pressure head data
!        rnull -- real null value used throughout program 
!        se(nx,ny,pmaxk) -- array of effective saturation 
!        thetares(nmat) -- residual water content for each layer
!        thetasat(nmat) -- saturated water content for each layer
!        thetaz(nx,ny,pmaxk) -- array of volumetric water content
!        tres -- residual water content for current layer
!        tsat -- saturated water content for current layer
!        uwt3d(nx,ny,strmaxk) -- unit weight at each depth (depth weighted average)
!        uwtsat -- saturated unit weight for current layer
!        uwtsum -- sum of unit weights weighted for depth
!        zdem(nx,ny) -- DEM elevations
!        zlast -- elevation of previous data point or layer boundary
!
!       OUTPUT FILES
!           20 'inputfilename_out.txt' -- echo of input and results containing
!                overall minimum slip surface data, written in subroutines
!                Readin, Readpiezo, Readpress, Readsearch, Readstrat,
!                Readstrength, and Writeout.
!
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

        USE CommonData, ONLY: dp,nullhi,rnull
        USE GridData, ONLY: nx,ny,zdem,exp1
        USE MaterialData, ONLY: nmat,gamw,gamr,thetasat,thetares,layer,satresdiff,&
                vga,vgn,vgnexp,fxr,invlogfxr,fxa,fxm,fxn
        USE WaterData, ONLY: pmaxk,maxpk,se,dsedz,pressz,press,maxtheta,mintheta,&
                swcc,iwater
        USE StrengthData, ONLY: uwt3d,duwtdz
        
        IMPLICIT NONE

        INTERFACE
          SUBROUTINE WriteError(errtype,problem,problemtype,solution,extraflag,extra1int,extra1ch)
            INTEGER, INTENT(in) :: errtype,extra1int
            CHARACTER(len=*), INTENT(in) :: extraflag,extra1ch,problemtype,problem,solution
          END SUBROUTINE WriteError
        END INTERFACE
        
        INTEGER :: ktop,i,j,k,l,error,last
        REAL(dp) :: zlast,ddepth,pdepth,uwtsum,depth,diff,ppz,pp,cfx,dzp
        REAL(dp) :: zlay(nmat),tsat,tres,uwtz,uwtzlast,uwtsat,theta,dthetazdz

        CHARACTER(70)  :: problemtype
        CHARACTER(120)  :: errmessage,solution          

        ALLOCATE (uwt3d(nx,ny,pmaxk),STAT=error) 
        IF (error.eq.0) ALLOCATE (duwtdz(nx,ny,pmaxk),STAT=error) 
        IF (error.eq.0) ALLOCATE (dsedz(nx,ny,pmaxk),STAT=error)
        IF (error.ne.0) THEN
          errmessage = 'unit weight arrays not allocated successfully'
          problemtype = 'calculating unsaturated unit weight'          
          CLOSE(13)
          Call WriteError(1,errmessage,problemtype,solution,'no ',0,' ')          
        END IF
        uwt3d = 0.0_dp
        duwtdz = 0.0_dp
        dsedz = 0.0_dp
        mintheta = nullhi
        maxtheta = rnull
        
        DO j=ny,1,-1
          DO i=nx,1,-1
            IF (zdem(i,j).eq.rnull) CYCLE      
            IF (nmat.gt.1) THEN
              DO l=1,nmat-1
                zlay(l) = layer(l,i,j)
              END DO
            END IF 
!     Set elevation of lowest layer to be lower than 3D pressure current node.             
            zlay(nmat) = pressz(i,j,1)-1.0_dp          
            ktop = maxpk(i,j)
            last = 1
            DO k = ktop,1,-1
              pp = press(i,j,k)
              ppz = pressz(i,j,k)           
              IF (swcc) THEN  !  need to calculate water content at pressure node
!     Find layer encompassing current pressure node for SWCC variables   
                DO l = last,nmat
                  IF (zlay(l).ne.rnull.and.ppz.ge.zlay(l)) THEN
                    IF (pp.ge.0) THEN
                      theta = thetasat(l)
                    ELSE 
                      IF (iwater.eq.5.or.iwater.eq.9) THEN  ! vanGunuchten curve fit 
                        theta = thetares(l)+satresdiff(l)/(1._dp+(vga(l)*abs(pp))**vgn(l))**vgnexp(l)
                      ELSE  ! Fredlund and Xing curve fit
                        cfx = 1._dp - log(1._dp+abs(pp)/fxr(l))*invlogfxr(l)
                        theta = cfx * thetasat(l)/(log(exp1+(abs(pp)/fxa(l))**fxn(l))**fxm(l))
                      END IF
                    END IF
                    last = l
                    EXIT
                  END IF
                END DO
              ELSE
                theta = se(i,j,k)
              END IF
              IF (theta.gt.thetasat(last)) THEN
                theta = thetasat(last)  
                WRITE (39,*) 'WARNING -- water content greater than saturated water content at node: ',i,j,k 
                WRITE (39,*) '   Scoops3D will set water content equal to saturated water content'  
              END IF
              IF (theta.lt.thetares(last)) THEN
                theta = thetares(last)
                WRITE (39,*) ' WARNING -- water content less than residual water content at node: ',i,j,k 
                WRITE (39,*) '  Scoops3D will set water content equal to residual water content'  
              END IF
              maxtheta = MAX(maxtheta,theta)
              mintheta = MIN(mintheta,theta)
              se(i,j,k) = theta
!     Temporarily set se array to theta to save on array space
              IF (k.lt.ktop) THEN
                dzp = pressz(i,j,k+1) - ppz
                dsedz(i,j,k) = (se(i,j,k+1) - theta)/dzp
              END IF          
            END DO      
  
!     Calculate depth-weighted unit weights and weight gradients, as well as effective saturations
!     and gradients at each pressure head location.         
            uwtsum = 0.0_dp
            last = 1
            pdepth = 0.0_dp
            depth = 0.0_dp
            zlast = zdem(i,j)
            uwtzlast = 0.0_dp
            DO k = ktop,1,-1
              ppz = pressz(i,j,k)
              theta = se(i,j,k)
              dthetazdz = dsedz(i,j,k)
              IF (k.lt.ktop) THEN
                depth = zdem(i,j) - ppz
                pdepth = pressz(i,j,k+1)-ppz 
              END IF
!    Keep running sum of weights until hit layer containing pressure node        
              DO l = last,nmat
                IF (zlay(l).ne.rnull) THEN
                  uwtsat = gamr(l,2)
                  tsat = thetasat(l)
                  tres = thetares(l)
                  IF (ppz.lt.zlay(l)) THEN  ! If 3D node below this layer
                    ddepth = zlast - zlay(l)
                    diff = zlay(l) - ppz
!    Find water content at layer boundary using interpolation of 3D values and add to
!    running sum of column wt.                    
                    uwtz = uwtsat - (tsat-(theta+ dthetazdz*diff))*gamw
                    uwtsum = uwtsum + ((uwtz+uwtzlast)*0.5_dp) * ddepth
                    zlast = zlay(l)
                    uwtzlast = uwtz
                  ELSE  !   Node is in this layer
                    ddepth = zlast - ppz
                    se(i,j,k) = (theta - tres)/satresdiff(l)
                    uwtz = uwtsat - (tsat-theta)*gamw
                    IF (uwtzlast.gt.0._dp) THEN
                      uwtsum = uwtsum + (uwtz+uwtzlast)*0.5_dp * ddepth
                    ELSE
                      uwtsum = uwtsum + uwtz * ddepth                    
                    END IF
                    IF (depth.gt.0.0_dp) THEN
                      uwt3d(i,j,k) = uwtsum/depth
                    ELSE
                      IF (k.lt.ktop) THEN
                        uwt3d(i,j,k) = uwt3d(i,j,k+1)
                      ELSE
                        uwt3d(i,j,k) = uwtz
                      END IF
                    END IF
                    IF (pdepth.eq.0.0_dp) THEN  !  Set gradient to zero for top node.
                      duwtdz(i,j,k) = 0.0_dp
                      dsedz(i,j,k) = 0.0_dp
                    ELSE
                      duwtdz(i,j,k) = (uwt3d(i,j,k+1) - uwt3d(i,j,k))/pdepth
                      dsedz(i,j,k) = (se(i,j,k+1) - se(i,j,k))/pdepth
                    END IF
                    uwtzlast = uwtz
                    last = l
                    EXIT
                  END IF
                END IF
              END DO                                   
            END DO
          END DO
        END DO

        RETURN
        
1000    FORMAT(A)

        END
        
