        SUBROUTINE Line2d (i,j)
     
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!     This subroutine determines whether the 2-D slip line intersects the
!     column and calculates the length of the slip line.
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!
!     Called by Ordinary and Bishop
!
!     VARIABLES
!
!        cosang -- cosine of slip direction angle
!        deltax -- width of column used to calculate length of intersecting
!           line using slope from slip angle.
!        delxy -- DEM grid resolution (delta x, delta y)
!        dx1,dx2,dy1,dy2 -- x and y grid spacing of new column relative to inside nodes
!        i2dcen,j2dcen -- i,j location of center of search spheroid intersection 
!           with DEM surface
!        ipar -- flag for slip line parallel to side
!        line -- length of 2-D slip line in current column
!        mline -- slope of slip direction line relative to x and y grid
!        mset -- current land subset numbering
!        outnode1,outnode2 -- corner location of column nodes which fall outside the
!           search spheroid, also used to indicate included node for single node columns
!        nodesin -- number of land and water nodes of each column bounded by spheroid
!        radsq -- square of search spheroid axes perpendicular to strike of slope
!        rnull -- real null value used throughout program 
!        sinang -- sin of slip direction angle
!        xn1,xn2,xn3,xn4 -- x location of each column corner node
!        xl,x2,yl,y2 -- coordinates of column edges intersected by 2d line
!        x2dcen,y2dcen -- perpendicular projection of x,y location of center of search spheroid  
!           intersection with DEM surface onto axis of rotation
!        xl1,yl1 -- x,y intersection of slip line with one side of column
!        xl2,yl2 -- x,y intersection of slip line with another side of column
!        xcen,ycen,zcen -- center of search spheroid relative to DEM origin
!        xlow,xhi -- min and max x of the two column intersections 
!        xy1,xy2 -- x-intersection of slip line with horizontal lines at j and j+1
!        yn1,yn2,yn3,yn4 -- y location of each column corner node
!        ylow,yhi -- min and max y of the two column intersections 
!
!
!              4---3
!              |   |    numbering scheme for corners of column
!              |   |
!              1---2
!
!                3
!               ---
!              |   |    numbering scheme for sides of column
!             4|   |2
!               ---
!                1
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

      USE CommonData, ONLY: dp,rnull
      USE GridData, ONLY: delxy,xcen,ycen,radsq,rad2d,rad2dsq
      USE ColumnData, ONLY: outnode1,outnode2,nodesin,dx1,dx2,dy1,dy2,x1,&
            x2,y1,y2,x2dcen,y2dcen,mline,line,zprime2d,tandip2d,x2d,y2d,&
            i2dcen,j2dcen,xprime2d
      USE SetData, ONLY: cosang,sinang
            
      IMPLICIT NONE                 

      INTERFACE
        SUBROUTINE WriteError(errtype,problem,problemtype,solution,extraflag,extra1int,extra1ch)
          INTEGER, INTENT(in) :: errtype,extra1int
          CHARACTER(len=*), INTENT(in) :: extraflag,extra1ch,problemtype,problem,solution
        END SUBROUTINE WriteError
        
        SUBROUTINE Intersection (xx1,yy1,xx2,yy2,xi,yi,ipar)
          IMPORT :: dp
          REAL(dp), INTENT(in) :: xx1,yy1,xx2,yy2
          REAL(dp), INTENT(out) :: xi,yi
          INTEGER, INTENT(out) :: ipar
        END SUBROUTINE Intersection
      END INTERFACE
      
      INTEGER, INTENT(in) :: i,j

      REAL(dp) :: deltax,xl1,xl2,yl1,yl2,xn1,xn2,xn3,xn4,yn1,yn2,yn3,yn4
      REAL(dp) :: xlow,xhi,ylow,yhi,zz,yprime2d
      REAL(dp) :: xy1,xy2,xl,yl,xdiff,ydiff,del
      REAL(dp) :: xn1a,xn2a,xn3a,xn4a,yn1a,yn2a,yn3a,yn4a
      INTEGER :: ipar

      CHARACTER(70)  :: problemtype
      CHARACTER(120)  :: errmessage   
            
      zprime2d = 0.0_dp
      tandip2d = 0.0_dp
      x2d = 0.0_dp
      y2d = 0.0_dp
      xdiff = 0._dp
      ydiff = 0._dp
      xl1 = rnull
      xl2 = rnull
      yl1 = rnull
      yl2 = rnull
      xn1 = x1
      xn2 = x2
      xn3 = x2
      xn4 = x1
      yn1 = y1
      yn2 = y1
      yn3 = y2
      yn4 = y2
      del = .0001*delxy
      ipar = 0
      
!     If slip direction is perpendicular to the DEM grid and not in a
!     column aligned with 2D rotational centercoord in slide direction
!     2D line will not intersect column.
      IF ((abs(sinang).lt.0.00001_dp.and.j.ne.j2dcen).or.&
          (abs(cosang).lt.0.00001_dp.and.i.ne.i2dcen)) RETURN
            
!     If all four column nodes included in 3D slip surface 
      IF (nodesin.eq.4) THEN
!     If slip direction is 0 or 180 degrees 2D assign line coordinates
        IF (abs(sinang).lt.0.00001_dp) THEN
          line = delxy
          xl1 = xn1
          xl2 = xn2
          yl1 = y2dcen
          yl2 = y2dcen
        ELSE
!     If slip direction is 90 or 270 degrees
          IF (abs(cosang).lt.0.00001_dp) THEN
            line = delxy
            xl1 = x2dcen
            xl2 = x2dcen
            yl1 = yn1
            yl2 = yn3
          ELSE  ! For all other slip directions
          
!     Find x-intersection of slip direction line and horizontal lines through j and j+1
            xy1 = (yn1-y2dcen)/mline + x2dcen
            xy2 = (yn3-y2dcen)/mline + x2dcen
            
!     Determine whether current column is intersected by the slip direction line, by
!     checking if any of the four sides the of column are intersected.
            IF ((xy1.lt.xn1).and.((xy2.gt.xn1).and.(xy2.lt.xn2))) THEN
!     Line intersects sides 3 and 4.
              deltax = xy2 - xn1
              line = deltax/cosang
              xl1 = xn1
              xl2 = xy2
              yl1 = (xn1-x2dcen)*mline + y2dcen
              yl2 = yn3 
            END IF
                 
            IF (((xy1.gt.xn1).and.(xy1.lt.xn2)).and.(xy2.gt.xn2)) THEN
!     Line intersects sides 1 and 2.
              deltax = xn2 - xy1
              line = deltax/cosang
              xl1 = xy1
              xl2 = xn2
              yl1 = yn1
              yl2 = (xn2-x2dcen)*mline + y2dcen
            END IF
             
            IF (((xy1.gt.xn1).and.(xy1.lt.xn2)).and.(xy2.lt.xn1)) THEN
!     Line intersects sides 1 and 4.
              deltax = xy1 - xn1
              line = deltax/cosang 
              xl1 = xn1
              xl2 = xy1
              yl1 = (xn1-x2dcen)*mline + y2dcen
              yl2 = yn1
            END IF
             
            IF ((xy1.gt.xn2).and.((xy2.gt.xn1).and.(xy2.lt.xn2))) THEN   
!     Line intersects sides 2 and 3.             
              deltax = xn2 - xy2
              line = deltax/cosang 
              xl1 = xy2
              xl2 = xn2
              yl1 = yn3
              yl2 = (xn2-x2dcen)*mline + y2dcen 
            END IF
            
            IF (((xy1.le.xn1).and.(xy2.ge.xn2)).or. &
               ((xy1.ge.xn2).and.(xy2.le.xn1))) THEN  
!     Line intersects sides 2 and 4.   
              line = delxy/cosang
              xl1 = xn1
              xl2 = xn2
              yl1 = (xn1-x2dcen)*mline + y2dcen
              yl2 = (xn2-x2dcen)*mline + y2dcen  
            END IF
             
            IF (((xy1.ge.xn1).and.(xy1.le.xn2)).and. &
               ((xy2.ge.xn1).and.(xy2.le.xn2))) THEN
!     Line intersects sides 1 and 3.  
              line = delxy/sinang
              xl1 = xy1
              xl2 = xy2
              yl1 = yn1
              yl2 = yn3   
            END IF
          END IF   
        END IF    

      ELSE  !  If partial column, check intersection with each side.
!     If 2 nodes outside spheroid assign x and y locations of new nodes.
        IF (nodesin.eq.2) THEN
          IF (outnode1.eq.1.and.outnode2.eq.2) THEN
            yn1 = yn4-dy1
            yn2 = yn3-dy2
          END IF
          IF (outnode1.eq.2.and.outnode2.eq.3) THEN
            xn2 = xn1+dx1
            xn3 = xn4+dx2
          END IF
          IF (outnode1.eq.3.and.outnode2.eq.4) THEN
            yn3 = yn2+dy2
            yn4 = yn1+dy1
          END IF
          IF (outnode1.eq.1.and.outnode2.eq.4) THEN
            xn1 = xn2-dx1
            xn4 = xn3-dx2
          END IF
        END IF
!     If 1 node outside spheroid (3 nodes in) assign x and y location of new nodes.
        IF (nodesin.eq.3) THEN
          IF (outnode1.eq.1) THEN
            xn1a = xn2-dx1
            yn1a = yn4-dy1
          END IF
          IF (outnode1.eq.2) THEN
            xn2a = xn1+dx1
            yn2a = yn3-dy2
          END IF
          IF (outnode1.eq.3) THEN
            xn3a = xn4+dx2
            yn3a = yn2+dy2
          END IF
          IF (outnode1.eq.4) THEN
            xn4a = xn3-dx2
            yn4a = yn1+dy1
          END IF
        END IF   
!     If 1 node inside spheroid assign x and y location of two new nodes.
!     Variable outnode1 represents the included node for single node case.
        IF (nodesin.eq.1) THEN
          IF (outnode1.eq.1) THEN
            xn2 = xn1+dx1
            yn4 = yn1+dy1
          END IF
          IF (outnode1.eq.2) THEN
            xn1 = xn2-dx1
            yn3 = yn2+dy2
          END IF
          IF (outnode1.eq.3) THEN
            xn4 = xn3-dx2
            yn2 = yn3-dy2
          END IF
          IF (outnode1.eq.4) THEN
            xn3 = xn4+dx2
            yn1 = yn4-dy1
          END IF
        END IF 
        
!     If slip direction is not 0 or 180 degrees, check intersection with side 1.
!     If only one node contained check side 1 if node in is equal to 1 or 2.
        IF ((sinang.le.-0.00001_dp.or.sinang.ge.0.00001_dp).and.&
             (nodesin.gt.1.or.outnode1.lt.3)) THEN        
!     If 3 nodes in use new node on side 1.
          IF (nodesin.eq.3.and.outnode1.eq.1) xn1 = xn1a
          IF (nodesin.eq.3.and.outnode1.eq.2) xn2 = xn2a             
!     Find slip line intersection location with line defined by nodes 1 and 2.
          CALL Intersection (xn1,yn1,xn2,yn2,xl,yl,ipar)
          IF (ipar.ne.0) RETURN          
          xlow = xn1
          xhi = xn2
!     Determine which node has bigger y coordinate.
          IF (yn1.gt.yn2) THEN
            ylow = yn2
            yhi = yn1
          ELSE
            ylow = yn1
            yhi = yn2
          END IF
!     Determine if slip line intersection falls within column.
          IF (((xl.ge.xlow-del).and.(xl.le.xhi+del)).and.((yl.ge.ylow-del).and.(yl.le.yhi+del))) THEN
!     Define first column side intersection as being in side 1.
            xl1 = xl
            yl1 = yl
          END IF
        END IF          

!     If slip direction is not 90 or 270 degrees check intersection with side 2.
!     If only one node contained check side 2 if node in is equal to 2 or 3.
        IF ((cosang.le.-0.00001_dp.or.cosang.ge.0.00001_dp).and.(nodesin.gt.1.or.&
             (outnode1.eq.2.or.outnode1.eq.3))) THEN  
!     If 3 nodes in use new node on side 2.
          IF (nodesin.eq.3.and.outnode1.eq.2) yn2 = yn2a
          IF (nodesin.eq.3.and.outnode1.eq.3)yn3 = yn3a                         
!    Find slip line intersection location with line defined by nodes 2 and 3.             
          CALL Intersection (xn2,yn2,xn3,yn3,xl,yl,ipar)
          IF (ipar.ne.0) RETURN
          ylow = yn2
          yhi = yn3
!    Determine which node has bigger x coordinate.
          IF (xn2.gt.xn3) THEN
            xlow = xn3
            xhi = xn2
          ELSE
            xlow = xn2
            xhi = xn3
          END IF   
!     Determine if slip line intersection falls within column.     
          IF (((xl.ge.xlow-del).and.(xl.le.xhi+del)).and.((yl.ge.ylow-del).and.(yl.le.yhi+del))) THEN
!     If no column side intersections yet found, define first one in side 2.
            IF (xl1.lt.0.0_dp) THEN
              xl1 = xl
              yl1 = yl
            ELSE
!     If one intersection already found, second one is in side 2.
              xl2 = xl
              yl2 = yl
            END IF
          END IF
        END IF

!     If slip direction is not 0 or 180 degrees, check intersection with side 3.
!     If only one node contained check side 3 if node in is equal to 3 or 4.
        IF ((xl2.lt.0.0_dp.and.sinang.le.-0.00001_dp.or.sinang.ge.0.00001_dp).and.&
             (nodesin.gt.1.or.outnode1.gt.2)) THEN  
!     If 3 nodes in use new node on side 3.
          IF (nodesin.eq.3.and.outnode1.eq.3) xn3 = xn3a
          IF (nodesin.eq.3.and.outnode1.eq.4) xn4 = xn4a
!     Find slip line intersection location with line defined by nodes 3 and 4.        
          CALL Intersection (xn3,yn3,xn4,yn4,xl,yl,ipar)
          IF (ipar.ne.0) RETURN          
          xlow = xn4
          xhi = xn3
!    Determine which node has bigger y coordinate.
          IF (yn3.gt.yn4) THEN
            ylow = yn4
            yhi = yn3
          ELSE
            ylow = yn3
            yhi = yn4
          END IF        
          IF (((xl.ge.xlow-del).and.(xl.le.xhi+del)).and.((yl.ge.ylow-del).and.(yl.le.yhi+del))) THEN
!     If no column side intersections yet found, define first one in side 3.
            IF (xl1.lt.0.0_dp) THEN
              xl1 = xl
              yl1 = yl
            ELSE
!     If one intersection already found, second one is in side 3.
              xl2 = xl
              yl2 = yl
            END IF
          END IF
        END IF   

!     If slip direction is not 90 or 270 degrees check intersection with side 4.
!     If only one node contained check side 4 if node in is equal to 1 or 4.
        IF (xl2.lt.0.0_dp.and.(cosang.le.-0.00001_dp.or.cosang.ge.0.00001_dp).and.&
             (nodesin.gt.1.or.(outnode1.eq.1.or.outnode1.eq.4))) THEN     
!     If 3 nodes in use new node on side 4.
          IF (nodesin.eq.3.and.outnode1.eq.1) yn1 = yn1a
          IF (nodesin.eq.3.and.outnode1.eq.4) yn4 = yn4a 
!     Find slip line intersection location with line defined by nodes 4 and 1.         
          CALL Intersection (xn4,yn4,xn1,yn1,xl,yl,ipar)
          IF (ipar.ne.0) RETURN          
          ylow = yn1
          yhi = yn4
!     Determine which node has bigger x coordinate.
          IF (xn1.gt.xn4) THEN
            xlow = xn4
            xhi = xn1
          ELSE
            xlow = xn1
            xhi = xn4
          END IF        
          IF (((xl.ge.xlow-del).and.(xl.le.xhi+del)).and.((yl.ge.ylow-del).and.(yl.le.yhi+del))) THEN
!     If no column side intersections yet found and more than one included node, 
!     then this column is not intersected.
            IF (xl1.lt.0.0_dp) THEN
              IF (nodesin.eq.2.or.nodesin.eq.4) THEN
                RETURN
              ELSE
!     Second column intersection is in side 4.
                xl1 = xl
                yl1 = yl
              END IF
            ELSE
              xl2 = xl
              yl2 = yl
            END IF
          END IF 
        END IF

!     If 1 or 3 nodes in and two sides not yet intersected check triangle hypotenuse          
        IF ((nodesin.eq.1.or.nodesin.eq.3).and.xl2.lt.0.0_dp) THEN
          SELECT CASE (outnode1)
           CASE (1)
             IF (nodesin.eq.3) THEN
               xn2 = xn1a
               yn4 = yn1a
             END IF                 
!     Find slip line intersection location with line defined by nodes 2 and 4.         
             CALL Intersection (xn2,yn2,xn4,yn4,xl,yl,ipar)
             IF (ipar.ne.0) RETURN             
             ylow = yn2
             yhi = yn4
             xlow = xn4
             xhi = xn2
           CASE (2)            
             IF (nodesin.eq.3) THEN
               xn1 = xn2a
               yn3 = yn2a
             END IF
!     Find slip line intersection location with line defined by nodes 1 and 3.         
             CALL Intersection (xn1,yn1,xn3,yn3,xl,yl,ipar)
             IF (ipar.ne.0) RETURN             
             ylow = yn1
             yhi = yn3
             xlow = xn1
             xhi = xn3
           CASE (3)
             IF (nodesin.eq.3) THEN
               xn4 = xn3a
               yn2 = yn3a
             END IF           
!     Find slip line intersection location with line defined by nodes 2 and 4.         
             CALL Intersection (xn2,yn2,xn4,yn4,xl,yl,ipar)
             IF (ipar.ne.0) RETURN             
             ylow = yn2
             yhi = yn4
             xlow = xn4
             xhi = xn2
           CASE (4)          
             IF (nodesin.eq.3) THEN
               xn3 = xn4a
               yn1 = yn4a
             END IF           
!     Find slip line intersection location with line defined by nodes 1 and 3.         
             CALL Intersection (xn1,yn1,xn3,yn3,xl,yl,ipar)
             IF (ipar.ne.0) RETURN             
             ylow = yn1
             yhi = yn3
             xlow = xn1
             xhi = xn3
          END SELECT            
          IF (((xl.ge.xlow-del).and.(xl.le.xhi+del)).and.((yl.ge.ylow-del).and.(yl.le.yhi+del))) THEN
!     If no column side intersections yet found, then this column is not intersected.
            IF (xl1.lt.0.0_dp) THEN
              RETURN
            ELSE
!     Second column intersection is in hypotenuse.
              xl2 = xl
              yl2 = yl
            END IF
          END IF 
        END IF
      END IF
        
      IF (xl2.ge.0.0_dp.or.yl2.ge.0.0_dp) THEN
!    Determine x and y location of center of 2d column.
        xdiff = xl2-xl1
        ydiff = yl2-yl1
        x2d = xl1 + xdiff * 0.5_dp
        y2d = yl1 + ydiff * 0.5_dp  
!    Determine elevation of slip base at center of 2d column.        
        xprime2d = x2d-xcen
        yprime2d = y2d-ycen 
        zz = radsq-xprime2d*xprime2d-yprime2d*yprime2d
        IF (zz.lt.0.0_dp) THEN
          errmessage = 'cannot calculate column midpoint in line2d'
          problemtype = 'calculating line for 2D factor of safety'
          CLOSE (33)
          Call WriteError(1,errmessage,problemtype,'no','no ',0,' ')               
        END IF
        zprime2d = sqrt(zz)          
!     Use partial derivative of equation for the 2D circle 
!     at column center to compute true dip of base.
        xprime2d = -(xprime2d*cosang+yprime2d*sinang)
        tandip2d = xprime2d/zprime2d
        IF (rad2d.le.0._dp) THEN
          rad2dsq = xprime2d*xprime2d + zprime2d*zprime2d
          rad2d = sqrt(rad2dsq)
        END IF
!     Find length of line using pythagorean theorem.
        line = abs(line)
        IF (line.le.0.00001_dp) line = sqrt(xdiff*xdiff + ydiff*ydiff)                      
      END IF    

      END SUBROUTINE Line2d        


      SUBROUTINE Intersection (xx1,yy1,xx2,yy2,xi,yi,ipar)
     
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!      This subroutine calculates the intersection of the slip direction
!      line with the line segment defined by the column corner nodes xx1,yy1 .
!      and xx2,yy2. The intersection is (xi,yi).
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!
!      Called by Line2d
!
!      VARIABLES
!
!      det -- determinant
!      ipar -- flag for slip line parallel to side
!      mline -- slope of slip line relative to positive x of DEM grid
!      x2dcen,y2dcen -- location of center of search spheroid intersection 
!            with DEM surface
!      xx1,yy1,xx2,yy2 -- coordinates of grid line along one side of column
!      xi,yi -- intersection of slip line with grid line
!
!
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
        USE CommonData, ONLY: dp,rnull
        USE ColumnData, ONLY: x2dcen,y2dcen,mline
        
        IMPLICIT NONE

        INTERFACE
          SUBROUTINE WriteError(errtype,problem,problemtype,solution,extraflag,extra1int,extra1ch)
            INTEGER, INTENT(in) :: errtype,extra1int
            CHARACTER(len=*), INTENT(in) :: extraflag,extra1ch,problemtype,problem,solution
          END SUBROUTINE WriteError
        END INTERFACE

        REAL(dp), INTENT(in) :: xx1,yy1,xx2,yy2
        REAL(dp), INTENT(out) :: xi,yi
        INTEGER, INTENT(out) :: ipar
        REAL(dp) :: a(2,2),b(2,2),a1,a2,b1,b2,c1,c2,det,mside

        CHARACTER(70)  :: problemtype
        CHARACTER(120) :: errmessage 

        ipar = 0 
        xi = xx1
        yi = yy1        

!     If column side is parallel to slide direction move on to next side
        IF (xx1.ne.xx2) THEN  ! column side direction is not 90 or 270 degrees
          mside = (yy2-yy1)/(xx2-xx1)
          IF (abs(mside-mline).lt.0.00001_dp) THEN
            ipar = 1
            RETURN
          ELSE  ! column side direction is 90 or 270 degrees
            IF (mline.eq.rnull) THEN
              ipar = 1
              RETURN  
            END IF
          END IF
        ELSE
!     If partial column side length is zero no intersection
          IF (yy1.eq.yy2) RETURN
        END IF          
        
        IF (mline.eq.rnull) THEN  ! slide direction is 90 or 270 degrees
          b1 = yy1-yy2
          a1 = xx2-xx1
          c1 = (xx1*yy2) - (xx2*yy1)
          a2 = 0.0_dp
          b2 = 1.0_dp
          c2 = -x2dcen
        ELSE                      !  all other slide directions
          a1 = yy2-yy1
          b1 = xx1-xx2
          c1 = (xx2*yy1) - (xx1*yy2)
          a2 = -mline
          b2 = 1.0_dp
          c2 = mline*x2dcen - y2dcen
        END IF

        a(1,1) = a1
        a(1,2) = b1
        a(2,1) = a2
        a(2,2) = b2        

!  Compute the determinant.

        det = a(1,1) * a(2,2) - a(1,2) * a(2,1)

        IF (det.eq.0.0_dp) THEN           
          problemtype = 'calculating intersection for 2D factor of safety calculation'        
          errmessage = 'error in determining line intersections'
          CLOSE (33)
          Call WriteError(1,errmessage,problemtype,'no','no ',0,' ')         
        END IF

!  Compute the entries of the inverse matrix using an explicit formula.

        b(1,1) = + a(2,2) / det
        b(1,2) = - a(1,2) / det
        b(2,1) = - a(2,1) / det
        b(2,2) = + a(1,1) / det
 
!  If the inverse exists, then multiply the inverse times -C to get the intersection point.
        IF (mline.eq.rnull) THEN
          yi = - b(1,1) * c1 - b(1,2) * c2
          xi = - b(2,1) * c1 - b(2,2) * c2
        ELSE
          xi = - b(1,1) * c1 - b(1,2) * c2
          yi = - b(2,1) * c1 - b(2,2) * c2
        END IF
                        
      END SUBROUTINE Intersection
        
        
        
        

