      SUBROUTINE midpoint(i,j)
     
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!      This subroutine calculates the midpoint of full or partial DEM columns
!      using centroid formulas.  For triangle: C=(1/3(x1+x2+x3),1/3(y1+y2+y3),
!      where x1,x2,x3 and y1,y2,y3 are the coordinates of the vertices.
!      For a polynomial with n vertices: 
!        Cx = 1/6A * Sum from n=1 to n [(x(n)+x(n+1)(x(n)y(n+1)-x(n+1)y(n))]
!        Cy = 1/6A * Sum from n=1 to n [(y(n)+y(n+1)(x(n)y(n+1)-x(n+1)y(n))]
!        Area = 1/2 * Sum from n=1 to n [(x(n)y(n+1)-x(n+1)y(n))]
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!
!        a -- polygon area calculated by shoelace formula using node coordinates
!        angle -- assumed angle of slip (azimuth)
!        cellarea -- area of DEM grid cell (delxy*delxy)
!        colxy(nx,ny) -- length of each column side (delxy for full columns)
!        delxy -- DEM grid resolution (delta x, delta y)
!        dx1,dx2,dy1,dy2 -- x and y grid spacing of partial column relative to inside nodes
!        dzdx,dzdy -- x and y components of dip at center of column base
!        icont -- flag to eliminate columns where midpoint can't be found (rounding errors)
!        inv3 -- 1/3 (precalculated)
!        mset -- current land subset number
!        nnodes -- number of nodes used to calculate column area (corner nodes plus 1)
!        nodesin -- number of land and/or water nodes of each column bounded 
!           by spheroid
!        outnode1,outnode2 -- corner location of column nodes which fall outside the
!           search spheroid, also used to indicate included node for single node columns
!        parea -- area of column projected on DEM grid. 
!        radsq -- square of search spheroid axes perpendicular to strike of slope
!        xcen,ycen,zcen -- center of search spheroid relative to DEM origin
!        x1,y1 -- x and y locations of bottom left column node
!        xmid,ymid -- x and y locations of middle of column
!        xprime,yprime,zprime -- x,y, and z coordinates translated relative to spheroid center
!        zmid(i,j) -- elevation of slip surface at column midpoint
!
!
!            dx2 
!          4----3
!      dy1 |    | dy2   numbering scheme for corners of column base
!          |    | 
!          1----2
!            dx1
!
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

        USE CommonData, ONLY: dp
        USE GridData, ONLY: halfdelxy,delxy,cellarea,xcen,ycen,zcen,radsq,inv3
        USE SetData, ONLY: icont
        USE ColumnData, ONLY: nodesin,outnode1,outnode2,parea,dx1,dx2,dy1,dy2,&
              xmid,ymid,x1,y1,dzdx,dzdy,colxy,outnodes,zmid,xprime,yprime,zprime

        IMPLICIT NONE

        INTEGER, INTENT(in) :: i,j
        INTEGER :: n,nnodes       
        REAL(dp) :: x(6),y(6),a,denom
        REAL(dp) :: zz,invzprime
      
        icont = 1
        
        IF (nodesin.lt.4) THEN
          outnode1 = outnodes(1,i,j)
          outnode2 = outnodes(2,i,j)
          dx1 = abs(colxy(1,i,j))
          dx2 = abs(colxy(2,i,j))
          dy1 = abs(colxy(3,i,j))
          dy2 = abs(colxy(4,i,j))
        END IF

!     Find x,y center of column
        
        IF (nodesin.eq.4) THEN
          xmid = x1 + halfdelxy
          ymid = y1 + halfdelxy
        ELSE

          SELECT CASE (nodesin)
!     Assign local coordinates to column from 0 to delxy to help with roundoff errors            
            CASE (2)   !  Partial column with 2 nodes in
              nnodes = 4
              IF ((outnode1.eq.1.and.outnode2.eq.2).or. &
                  (outnode1.eq.3.and.outnode2.eq.4)) THEN 
                x(1) = 0._dp
                x(2) = delxy
                x(3) = delxy
                x(4) = 0._dp
                IF (outnode1.eq.1) THEN
                  y(1) = delxy - dy1
                  y(2) = delxy - dy2
                  y(3) = delxy
                  y(4) = delxy
                ELSE
                  y(1) = 0._dp
                  y(2) = 0._dp
                  y(3) = 0._dp + dy2
                  y(4) = 0._dp + dy1
                END IF
              ELSE
                y(1) = 0._dp
                y(2) = 0._dp
                y(3) = delxy
                y(4) = delxy
                IF (outnode1.eq.1) THEN ! outnodes are 1 and 4
                  x(1) = delxy - dx1
                  x(2) = delxy
                  x(3) = delxy
                  x(4) = delxy - dx2
                ELSE  !  outnodes are 2 and 3
                  x(1) = 0._dp
                  x(2) = 0._dp + dx1
                  x(3) = 0._dp + dx2
                  x(4) = 0._dp
                END IF
              END IF
              x(5) = x(1)
              y(5) = y(1)
              
            CASE (1)   !  Partial column with 1 node in
              SELECT CASE (outnode1)  ! outnode1 represents contained node for this case
              CASE (1)
                x(1) = 0._dp
                x(2) = 0._dp + dx1
                x(3) = 0._dp
                y(1) = 0._dp
                y(2) = 0._dp
                y(3) = 0._dp + dy1
              CASE (2)
                x(1) = delxy - dx1
                x(2) = delxy
                x(3) = delxy
                y(1) = 0._dp
                y(2) = 0._dp
                y(3) = 0._dp + dy2
              CASE (3)
                x(1) = delxy
                x(2) = delxy
                x(3) = delxy - dx2
                y(1) = delxy - dy2
                y(2) = delxy
                y(3) = delxy
              CASE (4)
                x(1) = 0._dp
                x(2) = 0._dp + dx2
                x(3) = 0._dp
                y(1) = delxy - dy1
                y(2) = delxy
                y(3) = delxy
              END SELECT
!     Calculate midpoint using centroid of triangle
              xmid = inv3 * (x(1) + x(2) + x(3)) + x1
              ymid = inv3 * (y(1) + y(2) + y(3)) + y1
            
            CASE (3)  !  Partial column with 3 nodes in
!     5 sides are used for column center approximation. 
              nnodes = 5          
              SELECT CASE (outnode1)
                CASE (1)
                  x(1) = delxy - dx1
                  x(2) = delxy
                  x(3) = delxy
                  x(4) = 0._dp
                  x(5) = 0._dp
                  y(1) = 0._dp
                  y(2) = 0._dp
                  y(3) = delxy
                  y(4) = delxy
                  y(5) = delxy - dy1

                CASE (2)
                  x(1) = 0._dp
                  x(2) = 0._dp + dx1
                  x(3) = delxy
                  x(4) = delxy
                  x(5) = 0._dp
                  y(1) = 0._dp
                  y(2) = 0._dp
                  y(3) = delxy - dy2
                  y(4) = delxy
                  y(5) = delxy

                CASE (3)
                  x(1) = 0._dp
                  x(2) = delxy
                  x(3) = delxy
                  x(4) = 0._dp + dx2
                  x(5) = 0._dp
                  y(1) = 0._dp
                  y(2) = 0._dp
                  y(3) = 0._dp + dy2
                  y(4) = delxy
                  y(5) = delxy 

                CASE (4)
                  x(1) = 0._dp
                  x(2) = delxy
                  x(3) = delxy
                  x(4) = delxy - dx2
                  x(5) = 0._dp
                  y(1) = 0._dp
                  y(2) = 0._dp
                  y(3) = delxy
                  y(4) = delxy
                  y(5) = 0._dp + dy1

              END SELECT
              x(6) = x(1)
              y(6) = y(1)
            END SELECT     

!     Calulate cell midpoint for partial columns with 2 or 3 nodes in
          IF (nodesin.eq.2.or.nodesin.eq.3) THEN
            xmid = 0._dp
            ymid = 0._dp
            a = 0._dp
            DO n = 1,nnodes
              xmid = xmid + (x(n)+x(n+1)) * (x(n)*y(n+1)-x(n+1)*y(n))
              ymid = ymid + (y(n)+y(n+1)) * (x(n)*y(n+1)-x(n+1)*y(n))
              a = a + x(n)*y(n+1) - x(n+1)*y(n)
            END DO
!     Eliminate columns with very small area because of potential rounding errors
            IF (a.lt.1.D-6) THEN
              icont = 0
              RETURN
            END IF
            denom = 1._dp/(3._dp*a)
            xmid = xmid*denom + x1
            ymid = ymid*denom + y1
          END IF
        END IF
       
!     Find distance to slip base at cell center for base slope calculations.
        yprime = ymid-ycen
        xprime = xmid-xcen
!     Base elevation already calculated for full columns in land
!     so use that to find z distance to rotational axis.     
        IF (nodesin.lt.4) THEN
          zz = radsq-xprime*xprime-yprime*yprime
!     If zz is negative due to rounding errors throw out columnn
          IF (zz.lt.0.0_dp) THEN
            icont = 0
            RETURN
          END IF           
          zprime = sqrt(zz)
        ELSE  ! 4-node land column
          zprime = zcen - zmid(i,j)
        END IF
        
!     Use partial derivative of equation for the search spheroid 
!     at column center to compute cosine of true dip of base (costruedip) 
!     and slope of base in slide direction (apparent dip).
!     (Downslope apparent dip is positive in the direction of the slide.)
        invzprime = 1._dp/zprime
        dzdx = -xprime*invzprime
        dzdy = -yprime*invzprime

!    Find projected cell area for land portion of columns
        IF (nodesin.eq.4) THEN
          parea = cellarea
        ELSE 
          IF (nodesin.eq.1) THEN
            parea = 0.5_dp * (dx1+dx2)*(dy1+dy2)            
          ELSE
            parea = a * 0.5_dp
          END IF
        END IF

        END SUBROUTINE midpoint