        SUBROUTINE newnodes

!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!       This subroutine calculates the location of the intersection of
!       the search spheroid with the boundary of a column that is
!       partially contained in the slip surface.
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!
!      Called by Volumes, and Watercolvol
!
!      VARIABLES
!
!        delxy -- DEM grid resolution (delta x, delta y)
!        dx1,dx2 -- x-grid spacing of new column relative to inside nodes
!        dxy -- local variable for column resolution used to solve for
!           new node location
!        dy1,dy2 -- y resolution of column using new nodes
!        nodesin -- number of land and water nodes of each column bounded by spheroid
!        nonode -- flag for when there is no intersection between search
!           spheroid and slope line between outside and inside nodes
!        onenode -- corner location of included column node for single node column
!        outnode1,outnode2 -- corner location of column nodes which fall outside the
!           search spheroid, also used to indicate included node for single node columns
!        xcen,ycen,zcen -- center of search spheroid relative to DEM origin
!        x1 -- x location of column nodes 1 and 4
!        x2 -- x location of column nodes 2 and 3
!        y1 -- y location of column nodes 1 and 2
!        y2 -- y location of column nodes 3 and 4
!        znodes(4) -- elevation of DEM nodes in column 
!
!
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
 
        USE CommonData, ONLY: dp
        USE GridData, ONLY: delxy,xcen,ycen
        USE ColumnData, ONLY: x1,x2,y1,y2,dx1,dx2,dy1,dy2,nodesin,&
              nonode,outnode1,outnode2,onenode,znodes

        IMPLICIT NONE

        INTERFACE
          SUBROUTINE getnode (xya,xyb,xy0,za,zb,xycen,yxcen,dxy,nonode)
            IMPORT :: dp
            INTEGER, INTENT(out) :: nonode     
            REAL(dp), INTENT(in) :: xya,xyb,xy0,za,zb,xycen,yxcen
            REAL(dp), INTENT(inout) :: dxy
          END SUBROUTINE getnode
        END INTERFACE
        
        nonode = 0
        
!     If only 1 column node is intersected by search spheroid calculate
!     intersection of slip surface with DEM surface.
      SELECT CASE (nodesin)
        CASE (1)
          SELECT CASE (onenode) ! select included node
            CASE (1)
              CALL getnode (x1,x2,y1,znodes(1),znodes(2),xcen,ycen,dx1,nonode)
!     If node is not included set cell edge lengths to zero so area will be zero in Bishop
              IF (nonode.ne.0) RETURN
              CALL getnode (y1,y2,x1,znodes(1),znodes(4),ycen,xcen,dy1,nonode)
              IF (nonode.ne.0) RETURN
              dx2 = 0.0_dp
              dy2 = 0.0_dp              
            CASE (2)
              CALL getnode (x1,x2,y1,znodes(1),znodes(2),xcen,ycen,dx1,nonode)
              IF (nonode.ne.0) RETURN
              CALL getnode (y1,y2,x2,znodes(2),znodes(3),ycen,xcen,dy2,nonode)
              IF (nonode.ne.0) RETURN
              dx1 = delxy - dx1  
              dx2 = 0.0_dp
              dy1 = 0.0_dp                
            CASE (3)
              CALL getnode (y1,y2,x2,znodes(2),znodes(3),ycen,xcen,dy2,nonode)
              IF (nonode.ne.0) RETURN
              CALL getnode (x1,x2,y2,znodes(4),znodes(3),xcen,ycen,dx2,nonode)
              IF (nonode.ne.0) RETURN
              dy2 = delxy - dy2  
              dx2 = delxy - dx2
              dx1 = 0.0_dp
              dy1 = 0.0_dp                
            CASE (4)
              CALL getnode (x1,x2,y2,znodes(4),znodes(3),xcen,ycen,dx2,nonode)
              CALL getnode (y1,y2,x1,znodes(1),znodes(4),ycen,xcen,dy1,nonode)
              IF (nonode.ne.0) RETURN
              dy1 = delxy - dy1
              dx1 = 0.0_dp
              dy2 = 0.0_dp                
          END SELECT
          RETURN

!     If 2 column nodes are not intersected by search spheroid calculate
!     intersection of slip surface with DEM surface.
        CASE (2)
          IF (((outnode1.eq.1).and.(outnode2.eq.4)).or.&
              ((outnode1.eq.2).and.(outnode2.eq.3))) THEN
            CALL getnode (x1,x2,y1,znodes(1),znodes(2),xcen,ycen,dx1,nonode)
!     If intersection not found, column may have only one node in            
            IF (nonode.ne.0) THEN 
              IF (outnode1.eq.1) THEN
                onenode = 3
              ELSE
                onenode = 4
              END IF
              nonode = 0
            END IF
            CALL getnode (x1,x2,y2,znodes(4),znodes(3),xcen,ycen,dx2,nonode)
            IF (nonode.ne.0) THEN
              IF (onenode.eq.0) THEN
                IF (outnode1.eq.1) THEN
                  onenode = 2
                ELSE
                  onenode = 1
                END IF
                nonode = 0
              ELSE  !  both nodes are not included
                RETURN
              END IF
            END IF
            IF (onenode.gt.0) THEN
              SELECT CASE (onenode)
                CASE (2)                   
                  dx1 = delxy - dx1
                CASE (3) 
                  dx2 = delxy - dx2
              END SELECT
            ELSE
              IF (outnode1.eq.1) THEN
!    Find distance from column node in surface to intersection with spheroid.
                dx1 = delxy - dx1
                dx2 = delxy - dx2
              END IF
              RETURN
            END IF
          ELSE IF (((outnode1.eq.1).and.(outnode2.eq.2)).or.&
              ((outnode1.eq.3).and.(outnode2.eq.4))) THEN
            CALL getnode (y1,y2,x1,znodes(1),znodes(4),ycen,xcen,dy1,nonode)
            IF (nonode.eq.1) THEN
              IF (outnode1.eq.1) THEN
                onenode = 3
              ELSE
                onenode = 2
              END IF
              nonode = 0
            END IF
            CALL getnode (y1,y2,x2,znodes(2),znodes(3),ycen,xcen,dy2,nonode)
            IF (nonode.eq.1)THEN
              IF (onenode.eq.0) THEN
                IF (outnode1.eq.1) THEN
                  onenode = 4
                ELSE
                  onenode = 1
                END IF
                nonode = 0
              ELSE  !  both nodes are not included
                RETURN
              END IF
            END IF
            IF (onenode.gt.0) THEN
              SELECT CASE (onenode)                 
                CASE (3) 
                  dy2 = delxy - dy2
                CASE (4) 
                  dy1 = delxy - dy1
              END SELECT
            ELSE
              IF (outnode1.eq.1) THEN
                dy1 = delxy - dy1
                dy2 = delxy - dy2
              END IF
              RETURN
            END IF
          ELSE
            nonode = 1  !  nodes are catty-corner. This should never happen here.
          END IF
          RETURN

!     If only one node is uncontained, need to determine new node that
!     creates larger quadrilateral.
        CASE (3)
          IF (outnode1.eq.1) THEN
            CALL getnode (x1,x2,y1,znodes(1),znodes(2),xcen,ycen,dx1,nonode)
            IF (nonode.ne.0) THEN
              dx1 = delxy
              nonode = 0
            END IF
            CALL getnode (y1,y2,x1,znodes(1),znodes(4),ycen,xcen,dy1,nonode)
            IF (nonode.ne.0) THEN
              dy1 = delxy  
              nonode = 0                
            END IF
!    Find distance from column node in surface to intersection with spheroid.
            dx1 = delxy - dx1
            dy1 = delxy - dy1
!    Determine which new node results in larger column and keep that one.
            IF (dx1.ge.dy1) THEN
              dy1 = -dy1
            ELSE
              dx1 = -dx1
            END IF 
            RETURN            
          END IF
          IF (outnode1.eq.2) THEN
            CALL getnode (x1,x2,y1,znodes(1),znodes(2),xcen,ycen,dx1,nonode)
            IF (nonode.ne.0) THEN 
              dx1 = 0.0_dp
              nonode = 0
            END IF
            CALL getnode (y1,y2,x2,znodes(2),znodes(3),ycen,xcen,dy2,nonode)
            IF (nonode.ne.0) THEN
              dy2 = delxy
              nonode = 0
            END IF
            dy2 = delxy - dy2
            IF (dx1.ge.dy2) THEN
              dy2 = -dy2
            ELSE
              dx1 = -dx1
            END IF    
            RETURN            
          END IF
          IF (outnode1.eq.3) THEN
            CALL getnode (y1,y2,x2,znodes(2),znodes(3),ycen,xcen,dy2,nonode)
            IF (nonode.ne.0) THEN
              dy2 = 0.0_dp 
              nonode = 0
            END IF
            CALL getnode (x1,x2,y2,znodes(4),znodes(3),xcen,ycen,dx2,nonode)
            IF (nonode.ne.0) THEN 
              dx2 = 0.0_dp
              nonode = 0
            END IF
            IF (dx2.ge.dy2) THEN
              dy2 = -dy2
            ELSE
              dx2 = -dx2
            END IF  
            RETURN            
          END IF
          IF (outnode1.eq.4) THEN
            CALL getnode (x1,x2,y2,znodes(4),znodes(3),xcen,ycen,dx2,nonode)
            IF (nonode.ne.0) THEN
              dx2 = delxy
              nonode = 0
            END IF
            CALL getnode (y1,y2,x1,znodes(1),znodes(4),ycen,xcen,dy1,nonode)
            IF (nonode.ne.0) THEN
              dy1 = 0.0_dp
              nonode = 0
            END IF
            dx2 = delxy - dx2
            IF (dx2.ge.dy1) THEN
              dy1 = -dy1
            ELSE
              dx2 = -dx2
            END IF                 
          END IF
        END SELECT
        
        END SUBROUTINE newnodes

  
        SUBROUTINE getnode (xya,xyb,xy0,za,zb,xycen,yxcen,dxy,nonode)
 
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!       This subroutine calculates the location of a new column node at 
!       intersection of slip base with DEM column boundary by equating the
!       spheroid equation with the equation for the line between the excluded 
!       and included nodes and solving the resulting quadratic equation
!       for the x or y intersection of the column side, depending on which
!       side is being checked.
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!
!        Called by Newnodes
!       
!        VARIABLES
!
!        a,b,c -- coefficients of quadratric used to solve for new node
!        del -- variable used to take into account precision errors
!        nonode -- flag for when there is no intersection between search
!           spheroid and DEM line between outside and inside nodes
!        nxy -- identifies which axis is set as constant (1=x, 2=y)
!        p1,p2 -- each part of quadratic solution; add or subtract to 
!           find correct solution
!        radsq -- square of search radius
!        slope -- DEM surface gradient along column border which is intersected
!           by spheroid.
!        xcen,ycen,zcen -- center of search spheroid relative to DEM origin
!        xya,xyb -- local x or y variables used to solve for new node
!        xycen,yxcen -- local variables used to solve for new node (x or y 
!           of search spheroid center)
!        xy0,yx0 -- x or y location of new node
!        za,zb -- local z variables used to solve for new node
!
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

            USE CommonData, ONLY: dp
            USE GridData, ONLY: radsq,zcen,invdelxy
            
            IMPLICIT NONE
            
            INTEGER, INTENT(out) :: nonode     
            REAL(dp), INTENT(in) :: xya,xyb,xy0,za,zb,xycen,yxcen
            REAL(dp), INTENT(inout) :: dxy
            REAL(dp) :: yx0,p1,p2,del,q,slope
            REAL(dp) :: c,a,b,disc,comp1,comp2

            nonode = 0
            slope = (zb-za)*invdelxy
            comp1 = (za-zcen)-slope*xya
            comp2 = xy0 - yxcen
            c = comp1*comp1 + comp2*comp2 + xycen*xycen-radsq            
            a = 1.0_dp+slope*slope
            b = 2.0_dp*(slope*comp1 - xycen)
            disc = b*b - 4.0_dp*a*c
            IF ((disc).lt.0.0_dp) THEN
              nonode = 1
              RETURN
            END IF
            q = -.5_dp*(b+ SIGN(sqrt(disc),b))
            p1 = q/a
            IF (b.ne.0._dp) THEN
              p2 = c/q
            ELSE
              p2 = -p1
            END IF
!     Calculate del to use for taking into account precision errors. 
!     Count very close points as coinciding with the node point.
            del = .0001_dp*(xyb-xya)
            IF (p1.ge.xya-del.and.p1.le.xyb+del) THEN
              yx0 = p1
            ELSE
              IF (p2.ge.xya-del.and.p2.le.xyb+del) THEN
                yx0 = p2
              ELSE
                nonode = 1
                RETURN
              END IF
            END IF
            IF (yx0.lt.xya) yx0 = xya
            IF (yx0.gt.xyb) yx0 = xyb
            dxy = yx0 - xya
            
            END SUBROUTINE getnode
  
