        SUBROUTINE Ordinary 
     
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!       This subroutine calculates local and overall factors of safety
!       using the Ordinary (Fellenius) method.
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!
!       Called by Fos
!
!       VARIABLES
!
!        arclength -- arc length of current 2d slice
!        area2d -- cross-sectional area of current 2d slice
!        cee(nmat) -- cohesion of each layer
!        cellarea -- area of DEM grid cell (delxy*delxy)
!        cflag -- indicates whether 3-d cohesion data are provided (0=yes,1=no)
!        coh -- cohesion at base of column
!        cohes(nx,ny,nzstr) -- array of 3-d cohesion data
!        colarea2d -- area of 2d column
!        cosang -- cosine of slip direction angle
!        cosdip -- cosine of apparent dip of slip surface in column
!        costrdip -- cosine of true dip of slip surface in a column
!        dcohdz(nx,ny,strmaxk) -- array of cohesion gradients between vertical nodes
!        delxy -- DEM grid resolution (delta x, delta y)
!        delz -- difference between elevation of slip surface base
!           and nearest pressure data node.
!        delzp -- pressure head data vertical resolution
!        dflag -- indicates whether 3-d unit weight data are provided (0=yes,1=no)
!        dfricdz(nx,ny,strmaxk) -- array of friction gradients between vertical nodes
!        df -- local driving force
!        dlayd,dlayw -- wet and dry thickness of a layer in each column 
!        dpdz(nx,ny,nz) -- array of pressure gradients
!        drive -- overall driving moment
!        duwtdz(nx,ny,strmaxk) -- array of unit weight gradients between vertical nodes
!        duwtlaydz(nx,ny,nmat-1) -- gradient of depth-weighted unit weights between layers
!        dzdx,dzdy -- x and y components of dip at center of column base
!        dzstr -- z spacing of 3-d strength data
!        earm -- moment arm of earthquake moment for land portion of column, taken from center
!           of land portion of column to elevation of search center
!        eqcoef -- seismic coefficient coefficient (read from array if < 0)           
!        fflag -- indicates whether 3-d friction data are provided (0=yes,1=no)
!        foso -- factor of safety of current scoop at each slip angle
!        foso2d -- 2-d factor of safety of current scoop at each slip angle
!        fxr(nmat) -- curve-fitting parameter, psi_r (soil suction associated with residual moisture 
!           content) from the equation for the soil-water characteristic curve defined by 
!           Fredlund and Xing (1994)
!        gamr(nmat,2) -- array of total, partially saturated, and saturated
!           unit weights for each layer
!        gamw -- water unit weight in units of problem
!        gsameall -- flag indicating whether all densities are equivalent
!        gsameeach -- flag indicating whether densities in each layer are equiv
!        i1,i2,j1,j1 -- array bounds for current set
!        ifailcol -- flag for whether slip base is from failure surface or spheroid
!        ifailsurf -- flag for whether failure surface file is used
!        ifos2d -- flag for calculating 2-d factors of safety
!        inn(nx,ny) -- number of land nodes of each column bounded by spheroid
!        iwater -- flag indicating method for specifying water pressure
!           0 = no water pressures, 
!           1 = ru approximation, 
!           2 = piezometric surface, 
!           3 = 3D pressure head file, 
!           4 = 3D variably saturated file containing pressure head and water content, 
!           5 = 3D variably saturated file, containing pressure head, water content 
!               will be calculated from vanGenuchten SWCC, 
!           6 = 3D variably saturated file containing pressure head, water content 
!               will be calculated from Fredlund and Xing SWCC, 
!           7 = 3D pore pressure file,
!           8 = 3D variably saturated file containing pore pressure and water content, 
!           9 = 3D variably saturated file, containing pore pressure, water content 
!               will be calculated from vanGenuchten SWCC, 
!           10 = 3D variably saturated file containing pore pressure, water content 
!               will be calculated from Fredlund and Xing SWCC
!        kklo -- 3D pressure array location just below slip surface
!        lnum -- layer number
!        last -- previous layer found in column
!        layer(nmat,nx,ny) -- array of bottom elevations for material layers
!        line -- length of 2-D slip line in current column
!        lineflag -- flag for determining whether 2-D line/column intersection
!           already found for current j. Skip calculation if so (=-1).
!        linterp -- linear interpolation flag; 1=use linear interpolation, other=use
!           nearest node below slip surface base for strength data.
!        mset -- current land subset number
!        maxpk(nx,ny) -- array of highest k value of pressure data at each DEM column
!        maxstrk(nx,ny) -- array of highest k value of strength data at each DEM column
!        mline -- slope of slip direction line relative to x and y grid
!        moist -- determines which element of soil unit weight array to use: 1-total weight
!           2-partially saturated, 3-fully saturated.
!        ncol2d -- number of columns in current 2-D slice
!        nmat -- number of material layers
!        nodesin -- number of land and water nodes of each column bounded by spheroid
!        pore -- pore pressure on slip surface base in column
!        parea -- area of column projected on DEM grid. 
!        pgrid -- flag for pressure head grid, 1=regular, 2=irregular grid
!        piezo(nx,ny) -- array of piezometric elevation at each DEM column
!        pmaxk -- maximum number of 3-d pressure values at each DEM column
!        press(nx,ny,pmaxk) -- array of pressure head data
!        pressz(nx,ny,pmaxk) -- array of z elevations of pressure head data
!        pvol -- volume of intersected column below piezometric surface
!        pzsurf -- flag for whether piezometric surface was found,1=yes,0=no
!        r -- moment arm for resisting moment
!        resist -- overall resisting moment
!        rfc(nx,ny) -- cohesional component of resisting force
!        rff(nx,ny) -- frictional component of resisting force
!        rnull -- real null value used throughout program 
!        ru(nmat) -- pore pressure ratio approximation
!        sinang -- sin of slip direction angle
!        sindip -- sin of apparent dip of slip base in slide direction
!        sliparea -- area of slip surface
!        str3d -- flag for using 3-d strength file
!        strgrid -- flag for strength data grid; 1=regular, 2=irregular grid
!        strz(nx,ny,strmaxk) -- array of z locations of irregular strength data
!        subset(nx,ny) -- indicates set membership of DEM columns
!        tanfric -- tangent of friction angle at base of column
!        tanphi(nnmat) -- tangent of friction angle for each layer
!        tfric(nx,ny,strmaxk) -- array of 3-d friction data
!        vol(nx,ny) -- volume of land portion of current column in slip surface
!        volcol -- total volume of column including land and surface water portions
!        weight -- weight of slide
!        width2d -- width of current 2d slice
!        wt -- weight of land portion of column
!        xcen,ycen,zcen -- center of search spheroid relative to DEM origin
!        xmid -- x location of middle of column
!        x1,x2,y1,y2 -- x and y locations of each column node
!        xprime,yprime,zprime -- x,y, and z coordinates translated relative to spheroid center
!        ymid -- y location of middle of column
!        zdiff -- difference between strength or pressure elevation and elevation 
!           of base of slip surface
!        zlay(nmat) -- local variable for layer elevation in a column
!        zmidbase -- local variable for elevation of slip surface at column midpoint
!        zpiez -- local column variable for elevation of piezometric
!           surface at each column 
!        zpmin -- minimum elevation of pressure head data
!        zstrmin -- minimum elevation of strength data
!        ztop -- DEM elevation
!
!            dx2 
!          4----3
!      dy1 |    | dy2   numbering scheme for corners of column base
!          |    | 
!          1----2
!            dx1
!
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

      USE CommonData, ONLY: dp,rnull
      USE GridData, ONLY: delxy,zcen,pi,twopi,xcen,ycen,cmang,rad2d,exp1,zdem
      USE MaterialData, ONLY: nmat,gsameall,gsameeach,gamw,layer,cee,tanphi,&
            gamr,ru,uwtlay,duwtlaydz,eqcoef,fxr,eqarray,eqland
      USE SetData, ONLY: sliparea,weight,icont,i1,i2,j1,j2,cosang,sinang,mset
      USE WaterData  
      USE StrengthData, ONLY: str3d,cflag,fflag,dflag,strgrid,linterp,&
            maxstrk,zstrmin,dzstr,cohes,tfric,strz,dcohdz,dfricdz,duwtdz,uwt3d
      USE FOSData, ONLY: foso,foso2d,ifos2d,arclength,nangle,area2d,ncol2d,&
            width2d,foslocal,depth2d
      USE ColumnData, ONLY: nodesin,xmid,ymid,ztop,x1,x2,y1,y2,parea,mline,&
             line,zprime2d,tandip2d,tanalpha,dzdx,dzdy,xprime2d,subset,inn,vol,&
             xprime,yprime,zprime
      
      IMPLICIT NONE  
      
      INTERFACE
        SUBROUTINE WriteError(errtype,problem,problemtype,solution,extraflag,extra1int,extra1ch)
          INTEGER, INTENT(in) :: errtype,extra1int
          CHARACTER(len=*), INTENT(in) :: extraflag,extra1ch,problemtype,problem,solution
        END SUBROUTINE WriteError
        
        SUBROUTINE midpoint(i,j)
          INTEGER, INTENT(in) :: i,j
        END SUBROUTINE midpoint

        SUBROUTINE Line2d(i,j)
          INTEGER, INTENT(in) :: i,j
        END SUBROUTINE Line2d
      END INTERFACE
        
      INTEGER :: lnum,kklo,last,subsetij,i,j,kk,lineflag
      
      REAL(dp) :: coh,fosl,wt,pore,zdiff,zpiez,landvol
      REAL(dp) :: pvol,rff,rfc,dlayd,dlayw,costrdip
      REAL(dp) :: xyprime,tanfric,earm,r
      REAL(dp) :: zlay(nmat),truearea,rff2d,rfc2d      
      REAL(dp) :: resist,drive,rf,df,resist2d,drive2d,rf2d,df2d,z2d,cosdip2d
      REAL(dp) :: normf,invcostrdip,effdepth,area2dcol,zmidbase
      REAL(dp) :: consta,cosdip,invcosdip2d,dpth2dcol,ztop2d
      REAL(dp) :: cm(2),totweight,invcosdip,volfrac2d

      CHARACTER(70)  :: problemtype
      CHARACTER(120)  :: errmessage,solution
      
      cm = 0._dp
      rad2d = 0.0_dp
      totweight = 0._dp        
      sliparea = 0.0_dp
      weight = 0.0_dp                                                         
      drive = 0.0_dp
      resist = 0.0_dp
      lineflag = -1
      earm = 0.0_dp
      lnum = 1
      IF (ifos2d.ne.0) THEN
        IF (cosang.ne.0.0_dp) THEN
          mline = sinang/cosang
        ELSE
          mline = rnull
        END IF 
        resist2d = 0.0_dp
        drive2d = 0.0_dp  
        area2d = 0.0_dp
        depth2d = 0.0_dp
        arclength = 0.0_dp
        width2d = 0.0_dp
        ncol2d = 0
      END IF

      IF (foslocal.ne.0) THEN
        WRITE (16,*)
        WRITE (16,1000)
      END IF

!     Loop through all columns in current set.                                         
      DO j = j1,j2
        IF (ifos2d.ne.0) lineflag = 0  
        DO i = i1,i2
          subsetij = subset(i,j)
!     Bypass if column not within slip surface.
          IF (subsetij.ne.mset) CYCLE
          IF (pzsurf.ne.0) zpiez = piezo(i,j)
          wt = 0.0_dp
          pore = 0.0_dp
!    Initialize column variables common to both land and water columns
          x1 = REAL(i-1,dp)*delxy
          x2 = x1+delxy
          y1 = REAL(j-1,dp)*delxy
          y2 = y1+delxy 
          line = 0._dp

          nodesin = inn(i,j)
!     Calculate column midpoint and other values (zmidbase,costrdip,parea)
          CALL midpoint(i,j)
!     If can't find midpoint due to precision errors skip this column
          IF (icont.eq.0) THEN
            subset(i,j) = 0
            CYCLE
          END IF
          ztop = zdem(i,j)
          landvol = vol(i,j)
!     Find effective depth and column base elevation required to produce calculated volume
          effdepth = landvol/parea
          zmidbase = ztop - effdepth
!     Base angles were calculated for column center in subroutine midpoint. 
!     Note slope calculation is based on tangent to sphere at base midpoint, 
!     but assumption is made that base is flat enough that slope applies across base.
          invcostrdip = SQRT(1.0_dp+ dzdx*dzdx + dzdy*dzdy)
          costrdip = 1.0_dp/invcostrdip
          tanalpha = dzdx*cosang + dzdy*sinang
          invcosdip = SQRT(1._dp + tanalpha*tanalpha)
          cosdip = 1._dp/invcosdip
!     Accumulate land area of slip base      
          truearea = parea * invcostrdip     
          sliparea = sliparea + truearea
          IF (nmat.gt.1) THEN
            DO lnum=1,nmat-1
              zlay(lnum) = layer(lnum,i,j)
            END DO
          END IF

!      Determine whether slip direction line intersects column for 2-D
!      solution and calculate length of line and slip base in the column.
!      Only check if line/column intersection not yet found for this j
!      or found in adjacent column.
          IF (lineflag.ge.0) THEN
            CALL Line2d(i,j)
            IF (line.gt.0.0_dp) THEN
              z2d = zcen - zprime2d
              ztop2d = zcen - zprime + effdepth
              IF (z2d.lt.ztop2d) THEN
                lineflag = 1
                invcosdip2d = sqrt(1+tandip2d*tandip2d)
                cosdip2d = 1._dp/invcosdip2d
!     Accumulate area, arc length and number of columns for output reporting             
                width2d = width2d + line
                arclength = arclength + abs(line*invcosdip2d)
                ncol2d = ncol2d + 1
                dpth2dcol = ztop2d - z2d
                depth2d = max(dpth2dcol,depth2d)
                area2dcol = line * dpth2dcol
                area2d = area2d + area2dcol
!     Find ratio of 2D area to 3D volume
                volfrac2d = area2dcol/landvol
              ELSE
                IF (lineflag.ne.0) lineflag = -1
              END IF
            ELSE 
              IF (lineflag.ne.0) lineflag = -1
            END IF
          END IF            

!     If 3D strength file was read
          IF (str3d.eq.1) THEN
!     If slip base is below bottom data point exit program.
            IF (zmidbase.lt.strz(i,j,1)) THEN
              errmessage = 'slip surface below defined strength data'
              problemtype = 'in Ordinary factor of safety calculation'
              solution = 'increase depth of strength data or decrease volume of potential failures'              
              Call WriteError(2,errmessage,problemtype,solution,'no ',0,' ')                                                
              PRINT *,'    DEM cell location, i= ',i,'j= ',j
              PRINT *,'    elevation of slip surface = ',zmidbase
              PRINT *,'    minimum elevation of strength data = ',strz(i,j,1)
              WRITE (39,*) '     DEM cell location,    i =  ',i,'j = ',j
              WRITE (39,*) '     elevation of slip surface = ',zmidbase
              WRITE (39,*) '     minimum elevation of strength data = ',strz(i,j,1)             
              CLOSE (20)
              CLOSE (33)
              CLOSE (39)
              STOP
            END IF

!     If uniform grid ...
            IF (strgrid.eq.1) THEN                   
!     Find first strength node below column center base elevation                
              kklo = INT((zmidbase-zstrmin)/dzstr) + 1 
!     If non-uniform grid
            ELSE 
              kklo = maxstrk(i,j)                                          
              DO kk = 2,maxstrk(i,j)
!     Find first strength node below column center base elevation
                IF (strz(i,j,kk).gt.zmidbase) THEN 
                  kklo = kk - 1
                  EXIT
                END IF 
              END DO  
            END IF    !  IF (strgrid.eq.1)  
            zdiff = zmidbase - strz(i,j,kklo)
!     If not using linear interpolation use strength values from first node below base.                   
            IF (linterp.eq.0) THEN                  
              IF (fflag.eq.0) tanfric = tfric(i,j,kklo)
              IF (cflag.eq.0) coh = cohes(i,j,kklo)                  
            ELSE  
!     If interpolating, use strength values at first node below base and strength gradient.           
              IF (fflag.eq.0) tanfric = tfric(i,j,kklo) + dfricdz(i,j,kklo)*zdiff
              IF (cflag.eq.0) coh = cohes(i,j,kklo) + dcohdz(i,j,kklo)*zdiff
            END IF
!     Use gradient from bottom node to find depth-averaged unit weight at slip base.
            IF (dflag.eq.0) wt = landvol * (duwtdz(i,j,kklo)*zdiff + uwt3d(i,j,kklo))                               
          END IF  ! IF (str3d.eq.1)   

!     If multiple material layers determine location of slip surface base 
          IF (nmat.gt.1) THEN 
            zdiff = 0.0_dp
            last = nmat
            zlay(nmat) = zmidbase
            DO lnum = 1,nmat 
!     If current layer is intersected by column, check if base of material is 
!     above slip surface.  
              IF (zlay(lnum).ne.rnull.and.zlay(lnum).lt.ztop) THEN
                zdiff = zmidbase - zlay(lnum)
              ELSE
                CYCLE  ! check next layer
              END IF
!     If layer bottom is above slip surface, go to next layer
              IF (zdiff.lt.0.0_dp) THEN
                last = lnum
!     If layer bottom is below slip surface, calculate column weight
              ELSE
                EXIT
              END IF
            END DO
          END IF
                
!     Compute weight of each column based on depth-weighted average unit weights
          IF (dflag.ne.0) THEN
            IF ((nmat.eq.1).or.(gsameall.ne.0)) THEN
              IF (pzsurf.eq.0) THEN
                wt = landvol*gamr(1,moist)
              ELSE
                IF (zpiez.le.zmidbase) THEN
                  wt = landvol*gamr(1,1)
                ELSE
                  IF (zpiez.ge.ztop) THEN
                    wt = landvol*gamr(1,2)
                  ELSE
                    dlayw = zpiez-zmidbase
                    dlayd = effdepth - dlayw
                    wt = parea * (dlayd*gamr(1,1) + dlayw*gamr(1,2))
                  END IF
                END IF
              END IF
            ELSE
              IF (lnum.ne.nmat.and.last.ne.nmat) THEN
                wt = landvol * (uwtlay(i,j,lnum) + duwtlaydz(i,j,lnum)*zdiff)
              ELSE 
                IF ((pzsurf.eq.0).or.(gsameeach.ne.0)) THEN
                  IF (last.eq.nmat) THEN
                    wt = landvol * gamr(lnum,moist)
                  ELSE
                    zdiff = zlay(last)-zmidbase
                    wt = parea * (zdiff*gamr(lnum,moist) + uwtlay(i,j,last)*(effdepth-zdiff))
                  END IF
                ELSE
                  IF (pzsurf.ne.0) THEN                   
                    IF (zlay(last).le.zpiez) THEN
                      IF (last.ne.nmat) THEN
                        zdiff = zlay(last)-zmidbase
                        wt = parea * (zdiff*gamr(lnum,2) + &
                           uwtlay(i,j,last)*(effdepth-zdiff))
                      ELSE
                        wt = landvol * gamr(lnum,2)
                      END IF
                    ELSE 
                      IF (zpiez.le.zmidbase) THEN
                        IF (last.ne.nmat) THEN
                          zdiff = zlay(last)-zmidbase
                          wt = parea * (zdiff*gamr(lnum,1) + &
                             uwtlay(i,j,last)*(effdepth-zdiff))
                        ELSE
                          wt = landvol * gamr(lnum,1)
                        END IF
                      ELSE
                        IF (last.ne.nmat) THEN
                          dlayd = zlay(last)-zpiez
                          dlayw = zpiez-zmidbase
                          wt = parea * (dlayd*gamr(lnum,1) + dlayw*gamr(lnum,2) +&
                             uwtlay(i,j,last)*(effdepth-zdiff))
                        ELSE
                          IF (zpiez.ge.ztop) THEN
                            wt = landvol * gamr(lnum,2)
                          ELSE
                            dlayw = zpiez-zmidbase
                            dlayd = effdepth - dlayw
                            wt = parea * (dlayd*gamr(lnum,1) + dlayw*gamr(lnum,2))
                          END IF
                        END IF
                      END IF
                    END IF
                  END IF
                END IF
              END IF
            END IF                          
          END IF  ! if dflag.ne.0  (unit weights are given as layers)

!     Assign friction and cohesion values at slip surface
          IF (fflag.ne.0) tanfric = tanphi(lnum)
          IF (cflag.ne.0) coh = cee(lnum)

          IF (iwater.ne.0) THEN  !  calculate pore pressure
!     If using piezometric surface to calculate pore pressure
            IF (iwater.eq.2) THEN                          
!     If piezometric surface exists in column find volume of saturated portion
              IF (zpiez.ge.zmidbase) THEN
!     If piezometric surface is above DEM surface, assume entire
!     column is saturated. 
                IF (zpiez.ge.ztop) THEN
                  pvol = landvol
                ELSE
                  pvol = (zpiez-zmidbase)*parea
                END IF
                pore = MAX(pvol*gamw,0.0_dp)
              END IF
            ELSE
!     If using 3d pressure head data file and data is regularly spaced in z,
!     interpolate data to estimate pressure heads at slip surface.   
              IF (iwater.gt.2) THEN
                IF (zmidbase.lt.pressz(i,j,1)) THEN
                  errmessage = 'slip surface below defined pressure head data'
                  problemtype = 'in Ordinary factor of safety calculation'
                  solution = 'increase depth of pressure head data or decrease volume of potential failures'
                  Call WriteError(2,errmessage,problemtype,solution,'no ',0,' ')                     
                  PRINT *,'    DEM cell location, i = ',i,'j = ',j  
                  PRINT *,'    elevation of slip surface = ',zmidbase
                  PRINT *,'    minimum elevation of pressure head data = ',pressz(i,j,1)          
                  WRITE (39,*) '     DEM cell location,     i =',i,'j = ',j
                  WRITE (39,*) '     elevation of slip surface = ',zmidbase
                  WRITE (39,*) '     minimum elevation of pressure head data = ',pressz(i,j,1) 
                  CLOSE (20)
                  CLOSE (33) 
                  CLOSE (39) 
                  STOP              
                END IF           
                IF (pgrid.eq.1) THEN  ! uniform pressure grid in z              
!     Find pressure head node just below average slip surface
                  kklo = INT((zmidbase-zpmin)/delzp)+1
                ELSE              
!     If irregularly spaced pressure head data (pgrid=2)              
!     find pressure node just below base.
                  kklo = maxpk(i,j)
                  DO kk = 2,maxpk(i,j)
                    IF (pressz(i,j,kk).gt.zmidbase) THEN
                      kklo = kk - 1
                      EXIT
                    END IF
                  END DO                    
                END IF
                zdiff = zmidbase - pressz(i,j,kklo)
                pore = zdiff*dpdz(i,j,kklo) + press(i,j,kklo)
!     If variably saturated use gradient from bottom node to find depth-averaged unit weight at slip base.
                IF (varsat) THEN
                  wt = landvol * (duwtdz(i,j,kklo)*zdiff + uwt3d(i,j,kklo))
                  IF (pore.lt.0.0_dp) THEN
!     If variably saturated and negative pressure, multiply by effective saturation
                    pore = parea * pore * (zdiff * dsedz(i,j,kklo) + se(i,j,kklo))
                    IF ((iwater.eq.6.or.iwater.eq.10).and.abs(pore).gt.fxr(lnum)) pore = 0.0_dp
                  ELSE
                    pore = pore * parea
                  END IF
                ELSE
                  pore = MAX(pore * parea, 0.0_dp)
                END IF       
              ELSE
!      Calculate resisting force using hydrostatic piezometric surface ru (iwater=1)
                pore = wt * ru(lnum)        
              END IF  !  if 3D pressure files (iwater.gt.2)
            END IF  ! if piezometric surface (iwater.eq.2)
          END IF  !  if calculating pore pressures (iwater.ne.0)
        
!     Calculate moment arm for earthquake force on land portion of column 
!     from rotational axis to center of column.
            IF (eqcoef.ne.0.0_dp) THEN
              earm = zprime - 0.5_dp*effdepth
              IF (eqcoef.lt.0.0_dp) eqland = eqarray(i,j)
            END IF         
!     Calculate horizontal moment arm from rotational axis to column center
          xyprime = -(xprime*cosang+yprime*sinang)
!     Calculate radial moment arm to column base
          r = zprime*zprime + xyprime*xyprime
          r = sqrt(r)
!     Calculate driving moment
          df = wt * (xyprime + eqland*earm)
!     Calculate resisting moment
          consta = cosdip*cosdip * invcostrdip
          normf = consta*wt*(1._dp-eqland*tanalpha) - pore * invcostrdip
!     Don't allow frictional resisting force to be less than 0.
          IF (normf.le.0.0_dp) THEN
            rff = 0.0_dp
          ELSE
            rff = normf * tanfric
          END IF
          rfc = coh*truearea
          rf = (rff + rfc)*r
          
!     Accumulate total resisting moment                                    
          resist = resist + rf
!     Accumulate total driving moment
          drive = drive + df                                                     
!     Accumulate total weight
          weight = weight + wt  
          
!     Accumulate forces for 2-D factors of safety
          IF (lineflag.gt.0) THEN
            IF (eqland.gt.0.0_dp) earm = zprime2d - 0.5_dp*dpth2dcol
            df2d = volfrac2d * wt * (xprime2d + eqland*earm)
            normf = volfrac2d*(cosdip2d*wt*(1._dp-eqland*tandip2d)-pore)
            rff2d = normf*tanfric
            rfc2d = coh*line*invcosdip2d
            IF (rff2d.lt.0.0_dp) rff2d = 0.0_dp
            rf2d = (rff2d + rfc2d) * rad2d 
!     Accumulate total resisting force                                    
            resist2d = resist2d + rf2d
!     Accumulate total driving force
            drive2d = drive2d + df2d 
          END IF        
          
!     Calculate local factors of safety if required.
          IF (foslocal.ne.0) THEN    
            IF (df.eq.0.0_dp) THEN                                        
              fosl = 111.0_dp
            ELSE                                                       
              fosl = rf/df
            END IF
            WRITE (16,2000) i,j,truearea,rff,rfc,rf,df,fosl
          END IF

!     If iterating on slide angles to find minimum FOS keep running sum of
!     center of mass of each column for slip direction calculation
          IF (nangle.eq.1) THEN
            cm(1) = cm(1)+wt*xmid
            cm(2) = cm(2)+wt*ymid
            totweight = weight
          END IF

        END DO   !  loop on i                                                     
      END DO     !  loop on j                 
                                   
      IF (foslocal.ne.0) WRITE (16,*)
      IF (resist.le.0.0_dp) resist = 0.0_dp                                                       
      IF (drive.lt.0.010_dp*resist.or.drive.le.0.0_dp) THEN
        foso = 100.0_dp
      ELSE
        foso = resist/drive
      END IF
      
      IF (ifos2d.ne.0) THEN
      
        IF (resist2d.le.0.0_dp) resist2d = 0.0_dp
        IF (drive2d.lt.0.010_dp*resist2d.or.drive2d.le.0.0_dp) THEN
          foso2d = 100.0_dp
        ELSE
          foso2d = resist2d/drive2d
        END IF
      END IF

!     Find slide direction based on direction from center of mass to center of rotation if 
!     iterating to find minimum FOS slide direction
      IF (nangle.eq.1) THEN   
        cm = cm/totweight
        cmang = ATAN2(ycen-cm(2),xcen-cm(1))
!     Shift cmang to be between 0 and 2*pi
        IF (cmang.lt.0.0_dp) cmang = cmang + twopi
      END IF
    
1000  FORMAT ('   i   j  sliparea    res. frict.  res. coh.  total res.  total driv. res./driv.')
2000  FORMAT (2i4,6es12.4)  
                                 
      END SUBROUTINE Ordinary                                                              
