        SUBROUTINE Radius(ierr)

!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!       This subroutine computes the radius of the slip surface for 
!       the first spheroid at the search grid point, adjusting the 
!       radius on subsequent calls to meet the volume or area constraint.
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!
!        Called by Scoops3D
!
!        VARIABLES
!
!        ddxcensq,ddycensq,ddzcensq -- square of center of DEM column to 
!            search center
!        delxy -- DEM grid resolution (delta x, delta y)
!        drad -- differential of r found from derivative of volume with respect to r.
!        dt -- difference between vmin or armin and set volume
!        dtold -- previous dt
!        dxcen -- x center of DEM grid
!        dycen -- y center of DEM grid
!        goal -- vmin or armin + half of tol
!        ierr -- error flag indicating initial radius problem (2)
!        intersect -- flag for radius intersecting DEM grid (0=no,1=yes)
!        ncount -- total number of changes in radius estimates
!        iradflag -- indicates whether valid initial range subset has been found.
!           (1=no, 0=yes, 2=no valid sets after 10 radius adjustments.)
!        nx -- number of DEM cells in x direction
!        ny -- number of DEM cells in y direction
!        oldzcen -- elevation of previous search center
!        rad -- search spheroid axes perpendicular to strike of slope
!        radhi -- the larger of the last two radii calculated
!        radini - initial radius
!        radiniprev -- used in determination of first radius estimate at new search
!           lattice elevation. Equals previous radius  at current search lattice x-y  
!           location that met criteria constrains, or = -1 if radius not yet calculated.
!        radlo -- the smaller of the last two radii calculated
!        radold -- last radius guess with non-zero volume
!        rmax -- maximum radius allowed, depending on whether using 
!           ground-centric search or search lattice.
!        rnull -- real null value used throughout program 
!        vadcrit -- indicates primary control for initial radius (1 for volume,
!           2 for area)
!        va1max -- max. volume or area (primary criterion) of a subset
!        xcen,ycen,zcen -- center of search spheroid relative to DEM origin
!        z -- effective distance from spheroid center to DEM for spheroid volume equation
!        zd -- elevation of DEM grid at same x,y as spheroid center
!        zdiff -- difference between current and prior search lattice elevation
!        zmin -- minimum DEM elevation
!        zsave -- representative distance from spheroid center to ground that yields
!           good initial volume or area
!       
!
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

        USE CommonData, ONLY: dp,rnull
        USE GridData, ONLY: xcen,ycen,zcen,rad,delxy,zmin,nx,ny,pi,twopi,inv3,&
                         radsq,oldzcen
        USE SearchData, ONLY: vadcrit,goal,zsave,radiniprev,va1max,rmax,iradflag,&
                         zd,ncount
        USE SetData, ONLY: intersect
        USE DepthData, ONLY: depthmin
        
        IMPLICIT NONE

        INTERFACE
          PURE REAL(dp) FUNCTION VolRadius(z,voltarget)
            IMPORT :: dp
            REAL(dp), INTENT(in) :: z,voltarget
          END FUNCTION VolRadius
        END INTERFACE
  
        INTEGER, INTENT(out) :: ierr
        INTEGER :: binary,size2
        
        REAL(dp) :: dt,drad,radini,ddzcensq,zdiff,radold,vamax
        REAL(dp) :: dxcen,dycen,z,ddxcensq,ddycensq,radhi,radlo
        REAL(dp) :: dtold,zsq,s1,s2,s3,theta,p,nudge
                
        SAVE radold,dtold,radini,z,binary,radhi,radlo

        dt = 0.0_dp
        size2 = 0
     
!     If this is first radius estimate at current search center
        IF (rad.lt.0.0_dp.or.ncount.lt.0) THEN
          binary = 0        
!     If prior good radius never found that could be used for new estimate
          IF (radiniprev.lt.0.0_dp) THEN
!     If search node is outside DEM, estimate the initial radius as the 
!     distance to the center of the DEM grid with minimum DEM elevation, zmin.
            IF (zd.le.-888.0_dp) THEN
              dxcen = REAL(((nx+1)*0.5_dp)- 1,dp) * delxy
              dycen = REAL(((ny+1)*0.5_dp)-1,dp) * delxy
              ddxcensq = (dxcen-xcen)*(dxcen-xcen)
              ddycensq = (dycen-ycen)*(dycen-ycen)
              ddzcensq = (zmin-zcen)*(zmin-zcen)
              rad = sqrt(ddxcensq + ddycensq + ddzcensq)
            ELSE
!     If search node is above null DEM value, estimate z as distance
!     from search lattice elevation to minimum DEM elevation. Otherwise use 
!     distance to DEM elevation directly below search center.
              IF (zd.eq.rnull) THEN
                z = zcen - zmin
              ELSE
                z = zcen - zd
              END IF
!     If search grid point is within DEM boundary and volume
!     is primary criterion or secondary criterion with depth option
!     calculate initial guess for radius assuming a flat
!     DEM surface at distance z from the search node and the volume equation
!     for a truncated spheroid.
              IF (vadcrit.eq.1) THEN
                rad = VolRadius(z,goal)
              ELSE   ! if area is primary criterion or secondary criterion with depth option
                IF (vadcrit.eq.2) THEN
                  rad = sqrt(goal/pi + z*z)
                ELSE   ! if depth is primary criterion and grnd.ne.1
                  rad = z + depthmin
                END IF
              END IF
            END IF  ! if (zd.eq.-888.)

          ELSE
!     If radius was previously calculated at this i,j,k but not for this axis ratio, 
!     use difference in axis ratios to calculate new radius initial estimate. 
!     Use z calculated for initial radius of prior axis ratio.
            IF (zcen.eq.oldzcen) THEN
              z = zsave
              IF (vadcrit.eq.1) THEN          
                rad = VolRadius(z,goal)
              ELSE
                IF (vadcrit.eq.2) THEN
                  rad = sqrt(goal/pi + z*z)
                ELSE    ! if depth is primary criterion keep old radius from prior axis ratio
                  rad = radiniprev
                END IF
              END IF
            ELSE
!     If radius was previously calculated at this i,j but not this search node elevation, 
!     use change in search node elevation to calculate new radius initial estimate.  
              zdiff = zcen - oldzcen
              z = zsave               
              IF (vadcrit.eq.1) THEN          
                drad = 0.5_dp * zdiff *(radiniprev + z) / radiniprev
              ELSE
                IF (vadcrit.eq.2) THEN
                  drad = z * zdiff/ radiniprev
                ELSE   ! if depth is primary criterion
                  drad = zdiff
                END IF
              END IF
              rad = radiniprev + drad
            END IF
          END IF  ! if (radiniprev.lt.0)

          intersect = 0
          radini = rad
          radold = rad
          ncount = 0
        
!     If not first estimate of radius adjust value to meet criterion.
        ELSE
          IF (ncount.eq.0) dtold = 0.0_dp
          ncount = ncount + 1
!     If grid not intersected, increment radius.
          IF (va1max.eq.0.0_dp) THEN
            IF (intersect.eq.0) THEN        
              radold = rad
              rad = radini + 1.50_dp**(ncount+1)*delxy
              IF (rad.gt.rmax) rad = rmax - delxy
            ELSE
              rad = 0.5_dp * (radold + rad)
            END IF
          ELSE
            intersect = 1
            vamax = va1max
            dt = vamax - goal

            IF (ncount.gt.1.and.size2.eq.0) THEN
!     If last two radii bracketed target volume use binary search 
              IF (binary.eq.0) THEN !  first binary search
                IF((dt.gt.0._dp.and.dtold.lt.0._dp).or.&
                   (dt.lt.0._dp.and.dtold.gt.0._dp)) THEN
                  binary = 1
                  If (dt.gt.0._dp) THEN
                    radhi = rad
                    radlo = radold
                  ELSE
                    radhi = radold
                    radlo = rad
                  END IF
                  radold = rad
                  dtold = dt
                  rad = (radhi+radlo)*0.5_dp 
                END IF
              ELSE  !  already doing binary search
                IF (dt.gt.0.0_dp) THEN  !  too big
                  radhi = rad
                ELSE
                  radlo = rad              
                END IF
                radold = rad
                rad = (radhi+radlo)*0.5_dp             
                dtold = dt
              END IF
            END IF
            
            IF (binary.eq.0) THEN
              radold = rad
!     Otherwise find new radius using volume or area equation, depending on primary criterion.          
              IF (vadcrit.eq.1) THEN
!     Find equivalent height between spheroid center and plane based on calculated
!     volume and last radius using solution for cubic equation
                p = 1._dp - 3._dp*vamax/(radsq*rad*twopi)
                p = MIN(ABS(p),1._dp)
                theta = ACOS(p)
                s1 = -2._dp*rad * COS(inv3*theta)
                s2 = -2._dp*rad * COS(inv3*(theta+twopi))
                s3 = -2._dp*rad * COS(inv3*(theta-twopi))
                z = 0.0_dp
                IF (s1.gt.0._dp.and.s1.lt.rad) THEN
                  z = s1
                ELSE IF (s2.gt.0._dp.and.s2.lt.rad) THEN
                  z = s2
                ELSE IF (s3.gt.0._dp.and.s3.lt.rad) THEN
                  z = s3
                END IF
                IF (z.ge.0.0_dp) THEN
                  rad = VolRadius(z,goal)
                ELSE
                  PRINT *,'Error calculating effective z in Radius. Found negative value.'
                  STOP
                END IF
              ELSE  !  change radius based on target area
                zsq = MAX(radsq - vamax/pi,0._dp)
                rad = sqrt(goal/pi + zsq)              
              END IF
!       If the new radius would be negative, use half of last radius.
              IF (rad.le.0.0_dp) THEN
                rad = 0.5_dp * radold
              ELSE IF (ncount.gt.2.and.abs(dt).lt.abs(dtold)) THEN
                nudge = MIN(MAX(dtold/(dtold-dt),1._dp),3._dp)
                nudge = (rad-radold)*nudge + radold
                IF (nudge.gt.0._dp) rad = nudge
              END IF          
              dtold = dt
            END IF              
          END IF      

          IF (ncount.gt.0.and.rad.eq.radold) iradflag = 2           
          
!     If can't hit goal criterion range and volume is non-zero
!     choose largest value of radius and continue by setting iradflag
!     to 2, so won't get sent back into Radius.f to recalculate.          
          IF (ncount.gt.10) THEN
            IF (dt.gt.0.0_dp) THEN
              rad = MIN(rad,radold)
            ELSE
              rad = MAX(rad,radold)
            END IF
            iradflag = 2
            ierr=1
          END IF
        END IF  !  if (rad.lt.0)
        
        oldzcen = zcen          

        END SUBROUTINE radius
        
        PURE REAL(dp) FUNCTION VolRadius(z,voltarget)
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!       This function uses Numerical Recipes algorithm for solution of cubic 
!       equations to solve for the radius of a spheroidal cap given the target
!       volume and effective distance from the center of rotation to the DEM grid.
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
       
        USE CommonData, ONLY: dp
        USE GridData, ONLY: pi,twopi,inv3
        
        IMPLICIT NONE
        
        REAL(dp), INTENT(in) :: z,voltarget
        REAL(dp) :: s1,s2,s3,theta,p,q,r,a,b

        IF (z.gt.0._dp) THEN
          p = 1._dp - 6._dp*voltarget/(z*z*z*pi)
          IF (ABS(p).gt.1._dp) THEN
            q = 0.25_dp * z*z
            r = 0.125_dp * z*z*z - 0.75_dp*voltarget/pi
            a = (ABS(r) + SQRT(r*r-q*q*q))**inv3
            IF (r.gt.0._dp) a = -a
            IF (a.ne.0_dp) THEN
              b = q/a
            ELSE
              b = 0._dp
            END IF
            VolRadius = a + b + 0.5_dp * z
          ELSE
            theta = ACOS(p)
            s1 = z * (0.5_dp - COS(inv3*theta))
            s2 = z * (0.5_dp - COS(inv3*(theta+twopi)))
            s3 = z * (0.5_dp - COS(inv3*(theta-twopi)))
            IF (s1.gt.z) THEN
              VolRadius = s1
            ELSE IF (s2.gt.z) THEN
              VolRadius = s2
            ELSE
              VolRadius = s3
            END IF
          END IF
        ELSE  !  if z=0, use equation for half volume of spheroid
          VolRadius = (voltarget*1.5_dp/pi)**inv3
        END IF
        
        END FUNCTION VolRadius
