        SUBROUTINE readdepth(depthfile,minmax,laydep)
        
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!      This subroutine reads and processes the ASCII array data files that 
!      contain depth criteria information.  The minimum or maximum depth
!      file must contain values greater than or equal to zero at every 
!      non-null DEM grid location. Depths may also be specified using allow
!      materials layer elevation file which will be preprocessed into 
!      depth arrays in this subroutine.
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!
!      Called in Readin
!
!      VARIABLES
!
!        delxy -- DEM grid resolution (delta x, delta y)
!        delxy2 -- resolution read for depth file
!        depthfile -- minimum or maximum depth file name
!        dminarray(nx,ny) -- array of minimum depths for each DEM column
!        dmaxarray(nx,ny) -- array of maximum depths for each DEM column
!        dtol -- tolerance on minimum depth control criterion for calculating initial 
!           radius.  (i.e. initial depth must fall between depth and depth+dtol)
!        heading -- character variable for reading input
!        i,j -- DEM grid array location
!        ios -- status of read calls
!        laydep -- identifies material layer file to be used to define depth criteria
!        layer(nmat,nx,ny) -- array of bottom elevations for material layers
!        minmax -- identifies whether reading minimum(1) or maximum(2) depth file
!        nodata2 -- no data value for depth file
!        nullsame -- 'true' if the file value for no data is the same as value of
!           rnull, set in CommonData module
!        numd -- number of depth data
!        nx -- number of DEM columns
!        nx2 -- number of columns in depth file
!        ny -- number of DEM rows
!        ny2 -- number of rows in depth file
!        readline -- counter for number of header lines read
!        rnull -- real null value used throughout program 
!        stuff -- character variable for reading input
!        vadcrit -- indicates primary control for initial radius (1 for volume,
!           2 for area, and 3 for depth)
!        xll2,yll2 -- x and y origin of depth grid, read in header lines
!        zdem(nx,ny) -- DEM elevations
!
!      INPUT FILES READ
!       unit filename
!         14 'inputfilename.#' -- file of depths at each DEM cell
!
!      OUTPUT FILES
!       unit filename
!         20 'inputfilename_out.txt' -- echo of input and results containing
!              overall minimum slip surface data, written in subroutines
!              Readin, Readpiezo, Readpress, Readsearch, Readstrat,
!              Readstrength, Readfailsurf, and Writeout.
!         39 'errors.txt' -- file containing error messages
!
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
    
        USE CommonData
        USE GridData, ONLY: nx,ny,delxy,xll,yll,zdem
        USE MaterialData, ONLY: layer
        USE DepthData, ONLY: dmin,dminarray,dmaxarray,dtol
        USE SearchData, ONLY: vadcrit,vmax,armax
            
        IMPLICIT NONE 
        
        INTERFACE
          SUBROUTINE WriteError(errtype,problem,problemtype,solution,extraflag,extra1int,extra1ch)
            INTEGER, INTENT(in) :: errtype,extra1int
            CHARACTER(len=*), INTENT(in) :: extraflag,extra1ch,problemtype,problem,solution
          END SUBROUTINE WriteError
        END INTERFACE
        
        INTEGER :: i,j,nx2,ny2,ios,numd,readline,numbad1,numbad2,nullbad
        INTEGER, INTENT(in) :: minmax,laydep
        
        REAL(dp) :: delxy2,xll2,yll2,nodata2

        CHARACTER(len=*), INTENT(in) :: depthfile
        CHARACTER(60)  :: heading
        CHARACTER(12) :: stuff
        CHARACTER(70)  :: problemtype1,problemtype2
        CHARACTER(120)  :: errmessage,solution
        
        LOGICAL :: nullsame  
              
        problemtype1 = 'reading minimum depth file' 
        problemtype2 = 'reading maximum depth file'         
        ios = 0
        numd = 0
        readline = 0
        nullsame = .TRUE.        
        
        IF (laydep.eq.0) THEN
          OPEN (14,STATUS = 'old',FILE = depthfile,IOSTAT=ios)
          IF (ios.ne.0) THEN
            IF (minmax.eq.1) THEN
              errmessage = 'error opening minimum depth file:'
              solution = 'check for existence of file with specified name and location'
              Call WriteError(1,errmessage,problemtype1,solution,'ch ',0,depthfile)
            ELSE
              errmessage = 'error opening maximum depth file:'
              solution = 'check for existence of file with specified name and location'
              Call WriteError(1,errmessage,problemtype2,solution,'ch ',0,depthfile)
            END IF
          END IF

          DO 
            READ (14,1000,IOSTAT=ios) heading
            IF (ios.ne.0.or.readline.eq.6) EXIT

            solution = 'depth file must be in Esri ASCII raster format, as described in Scoops3D manual'
            SELECT CASE (heading(1:5))
            CASE ('ncols','NCOLS')   
              BACKSPACE 14     
              READ (14,*,IOSTAT=ios) stuff,nx2
              readline = readline + 1
              IF (ios.ne.0) THEN
                IF (minmax.eq.1) THEN
                  errmessage = 'error reading "ncols" header line of minimum depth file '              
                  CLOSE(14)
                  Call WriteError(1,errmessage,problemtype1,solution,'no ',0,' ')
                ELSE
                  errmessage = 'error reading "ncols" header line of maximum depth file '              
                  CLOSE(14)
                  Call WriteError(1,errmessage,problemtype2,solution,'no ',0,' ')
                END IF              
              END IF

            CASE ('nrows','NROWS')   
              BACKSPACE 14     
              READ (14,*,IOSTAT=ios) stuff,ny2
              readline = readline + 1
              IF (ios.ne.0) THEN
                IF (minmax.eq.1) THEN
                  errmessage = 'error reading "nrows" header line of minimum depth file '              
                  CLOSE(14)
                  Call WriteError(1,errmessage,problemtype1,solution,'no ',0,' ')
                ELSE
                  errmessage = 'error reading "nrows" header line of maximum depth file '              
                  CLOSE(14)
                  Call WriteError(1,errmessage,problemtype2,solution,'no ',0,' ')
                END IF             
              END IF

            CASE ('xllco','XLLCO')   
              BACKSPACE 14     
              READ (14,*,IOSTAT=ios) stuff,xll2
              readline = readline + 1
              IF (ios.ne.0) THEN
                IF (minmax.eq.1) THEN
                  errmessage = 'error reading "xllcorner" header line of minimum depth file '              
                  CLOSE(14)
                  Call WriteError(1,errmessage,problemtype1,solution,'no ',0,' ')
                ELSE
                  errmessage = 'error reading "xllcorner" header line of maximum depth file '              
                  CLOSE(14)
                  Call WriteError(1,errmessage,problemtype2,solution,'no ',0,' ')
                END IF             
              END IF
                             
            CASE ('yllco','YLLCO')   
              BACKSPACE 14     
              READ (14,*,IOSTAT=ios) stuff,yll2
              readline = readline + 1
              IF (ios.ne.0) THEN
                IF (minmax.eq.1) THEN
                  errmessage = 'error reading "yllcorner" header line of minimum depth file '              
                  CLOSE(14)
                  Call WriteError(1,errmessage,problemtype1,solution,'no ',0,' ')
                ELSE
                  errmessage = 'error reading "yllcorner" header line of maximum depth file '              
                  CLOSE(14)
                  Call WriteError(1,errmessage,problemtype2,solution,'no ',0,' ')
                END IF             
              END IF   

            CASE ('cells','CELLS')   
              BACKSPACE 14     
              READ (14,*,IOSTAT=ios) stuff,delxy2
              readline = readline + 1
              IF (ios.ne.0) THEN
                IF (minmax.eq.1) THEN
                  errmessage = 'error reading "cellsize" header line of minimum depth file '              
                  CLOSE(14)
                  Call WriteError(1,errmessage,problemtype1,solution,'no ',0,' ')
                ELSE
                  errmessage = 'error reading "cellsize" header line of maximum depth file '              
                  CLOSE(14)
                  Call WriteError(1,errmessage,problemtype2,solution,'no ',0,' ')
                END IF             
              END IF
              
            CASE ('NODAT','nodat')   
              BACKSPACE 14     
              READ (14,*,IOSTAT=ios) stuff,nodata2
              readline = readline + 1
              IF (ios.ne.0) THEN
                IF (minmax.eq.1) THEN
                  errmessage = 'error reading "nodata" header line of minimum depth file '              
                  CLOSE(14)
                  Call WriteError(1,errmessage,problemtype1,solution,'no ',0,' ')
                ELSE
                  errmessage = 'error reading "nodata" header line of maximum depth file '              
                  CLOSE(14)
                  Call WriteError(1,errmessage,problemtype2,solution,'no ',0,' ')
                END IF             
              END IF
              EXIT
                                   
            END SELECT
          END DO   
          
          IF (readline.ne.6) THEN
            IF (minmax.eq.1) THEN
              errmessage = 'error reading minimum depth file -- missing or incorrect header data'              
              CLOSE(14)
              Call WriteError(1,errmessage,problemtype1,solution,'no ',0,' ')
            ELSE
              errmessage = 'error reading maximum depth file -- missing or incorrect header data'              
              CLOSE(14)
              Call WriteError(1,errmessage,problemtype2,solution,'no ',0,' ')
            END IF          
          END IF
          
          solution = 'check extent and cellsize of input grid files'        
!     Compare DEM and failure surface file grid dimensions as given in header info.
          IF ((nx.ne.nx2).or.(ny.ne.ny2)) THEN
            IF (minmax.eq.1) THEN
              errmessage = 'DEM file and minimum depth file must have equivalent ncols and nrows'              
              CLOSE(14)
              Call WriteError(1,errmessage,problemtype1,solution,'no ',0,' ')
            ELSE
              errmessage = 'DEM file and maximum depth file must have equivalent ncols and nrows'              
              CLOSE(14)
              Call WriteError(1,errmessage,problemtype2,solution,'no ',0,' ')
            END IF 
          END IF
          
          IF (delxy2.ne.delxy) THEN 
            IF (minmax.eq.1) THEN
              errmessage = 'DEM file and minimum depth file must have equivalent cellsize'              
              CLOSE(14)
              Call WriteError(1,errmessage,problemtype1,solution,'no ',0,' ')
            ELSE
              errmessage = 'DEM file and maximum depth file must have equivalent cellsize'              
              CLOSE(14)
              Call WriteError(1,errmessage,problemtype2,solution,'no ',0,' ')
            END IF          
          END IF
          
          IF (xll2.ne.xll.or.yll2.ne.yll) THEN
            IF (minmax.eq.1) THEN
              errmessage = 'DEM file and minimum depth file must have equivalent xllcorner and yllcorner'              
              CLOSE(14)
              Call WriteError(1,errmessage,problemtype1,solution,'no ',0,' ')
            ELSE
              errmessage = 'DEM file and maximum depth file must have equivalent xllcorner and yllcorner'              
              CLOSE(14)
              Call WriteError(1,errmessage,problemtype2,solution,'no ',0,' ')
            END IF
          END IF
                             
          IF (nodata2.ne.rnull) nullsame = .false.

          solution = 'check format of input grid data; see "Program Input" chapter of Scoops3D manual' 
                 
          DO j = ny,1,-1
            IF (minmax.eq.1) THEN
              READ (14,*,IOSTAT=ios) (dminarray(i,j),i=1,nx)
            ELSE
              READ (14,*,IOSTAT=ios) (dmaxarray(i,j),i=1,nx)
            END IF
            IF (ios.ne.0.and.i.le.nx) THEN
              IF (minmax.eq.1) THEN 
                errmessage = 'error reading minimum depth file on line number '              
                CLOSE(14)
                Call WriteError(1,errmessage,problemtype1,solution,'int',ny-j+1,' ')
              ELSE
                errmessage = 'error reading maximum depth file on line number '              
                CLOSE(14)
                Call WriteError(1,errmessage,problemtype2,solution,'int',ny-j+1,' ')
              END IF
            END IF
            numd = numd + nx   
          END DO
                                  
          solution = 'check dimensions of input data, number of data records should equal ncols*nrows'               
          IF (numd.ne.(nx*ny)) THEN
            IF (minmax.eq.1) THEN 
              errmessage = 'minimum depth file dimensions are incorrect'              
              CLOSE(14)
              Call WriteError(1,errmessage,problemtype1,solution,'no ',0,' ')
            ELSE
              errmessage = 'maximum depth file dimensions are incorrect'              
              CLOSE(14)
              Call WriteError(1,errmessage,problemtype2,solution,'no ',0,' ')
            END IF          
          END IF  
         
          IF (minmax.eq.1) THEN
            IF (.not.nullsame) WHERE (dminarray.eq.nodata2) dminarray = rnull
            numbad1 = COUNT((dminarray.lt.0.0_dp.and.dminarray.ne.rnull).and.zdem.ne.rnull) 
            IF (numbad1.gt.0) THEN        
              solution = 'check depths specified in minimum depth file'
              errmessage = 'Scoops3D found a minimum depth less than zero' 
              CLOSE(14)
              Call WriteError(1,errmessage,problemtype1,solution,'no ',0,' ')
            END IF
            nullbad = COUNT(dminarray.eq.rnull.and.zdem.ne.rnull)
            IF (nullbad.gt.0) THEN
              WHERE (dminarray.eq.rnull) dminarray = 0.0_dp
              solution = 'check depths specified in minimum depth file'
              errmessage = 'null values found in minimum depth file at non-null DEM locations'              
              Call WriteError(0,errmessage,problemtype1,solution,'no ',0,' ')                     
              PRINT *,'Scoops3D will set null minimum depth values to zero at non-null DEM locations' 
              WRITE (39,*)'Scoops3D will set null minimum depth values to zero at non-null DEM locations'
            END IF
          ELSE
            IF (.not.nullsame) WHERE (dmaxarray.eq.nodata2) dmaxarray = rnull
            numbad1 = COUNT(dmaxarray.lt.0.0_dp.and.zdem.ne.rnull)
            IF (dmin.ge.0) THEN
              numbad2 = COUNT(dmaxarray.lt.dmin+dtol.and.dmaxarray.ne.rnull)
            ELSE
              numbad2 = COUNT(dmaxarray.lt.dminarray+dtol.and.dmaxarray.ne.rnull)
            END IF
            IF (numbad1.gt.0) THEN        
              solution = 'check depths specified in maximum depth file'
              errmessage = 'Scoops3D found a maximum depth less than or equal to zero' 
              CLOSE(14)
              Call WriteError(1,errmessage,problemtype2,solution,'no ',0,' ')
            END IF
            IF (numbad2.gt.0) THEN        
              solution = 'check depths specified in maximum depth file'
              errmessage = 'Scoops3D found a maximum depth less than a minimum depth plus tolerance' 
              CLOSE(14)
              Call WriteError(1,errmessage,problemtype2,solution,'no ',0,' ')
            END IF
            nullbad = COUNT(dmaxarray.eq.rnull.and.zdem.ne.rnull)
            IF (nullbad.gt.0) THEN
              IF (vadcrit.eq.2) THEN
                WHERE (dmaxarray.eq.rnull) zdem = rnull
                solution = 'check depths specified in maximum depth file'
                errmessage = 'null values found in maximum depth file at non-null DEM locations'              
                Call WriteError(0,errmessage,problemtype2,solution,'no ',0,' ')                     
                PRINT *,'Scoops3D will set DEM values to null at null maximum depth locations' 
                WRITE (39,*)'Scoops3D will set DEM values to null at null maximum depth locations'
              ELSE
                WHERE (dmaxarray.eq.rnull) dmaxarray = 0.0_dp
                solution = 'check depths specified in maximum depth file'
                errmessage = 'null values found in maximum depth file at non-null DEM locations'              
                Call WriteError(0,errmessage,problemtype2,solution,'no ',0,' ')                     
                PRINT *,'Scoops3D will set null maximum depth values to zero at non-null DEM locations' 
                WRITE (39,*)'Scoops3D will set null maximum depth values to zero at non-null DEM locations'
              END IF
            END IF
          END IF          

!     depth defined by material layer file
        ELSE
          IF (minmax.eq.1) THEN
            WHERE (layer(laydep,:,:).ne.rnull) dminarray = zdem - layer(laydep,:,:)
            numbad1 = COUNT((dminarray.lt.0.0_dp.and.dminarray.ne.rnull).and.zdem.ne.rnull) 
            IF (numbad1.gt.0) THEN        
              solution = 'check elevations in specified layer file used to calculate minimum depth'
              errmessage = 'Scoops3D calculated a minimum depth less than zero' 
              CLOSE(14)
              Call WriteError(1,errmessage,problemtype1,solution,'no ',0,' ')
            END IF
            nullbad = COUNT(dminarray.eq.rnull.and.zdem.ne.rnull)
            IF (nullbad.gt.0) THEN
              WHERE (dminarray.eq.rnull) dminarray = 0.0_dp
              solution = 'check elevations in specified layer file used to calculate minimum depth'
              errmessage = 'null values (or values higher than DEM) found in minimum depth layer &
                  &file at non-null DEM locations'              
              Call WriteError(0,errmessage,problemtype1,solution,'no ',0,' ')                     
              PRINT *,'Scoops3D will set null minimum depth values to zero at non-null DEM locations' 
              WRITE (39,*)'Scoops3D will set null minimum depth values to zero at non-null DEM locations'
            END IF
          ELSE
            WHERE (layer(laydep,:,:).ne.rnull) dmaxarray = zdem - layer(laydep,:,:)
            numbad1 = COUNT((dmaxarray.lt.0.0_dp.and.dmaxarray.ne.rnull).and.zdem.ne.rnull)
            IF (dmin.ge.0) THEN
              numbad2 = COUNT(dmaxarray.lt.dmin+dtol.and.dmaxarray.ne.rnull)
            ELSE
              numbad2 = COUNT(dmaxarray.lt.dminarray+dtol.and.dmaxarray.ne.rnull)
            END IF
            IF (numbad1.gt.0) THEN        
              solution = 'check elevations in specified layer file used to calculate maximum depth'
              errmessage = 'Scoops3D found a maximum depth less than or equal to zero' 
              CLOSE(14)
              Call WriteError(1,errmessage,problemtype2,solution,'no ',0,' ')
            END IF
            IF (numbad2.gt.0) THEN        
              solution = 'check depths specified in maximum depth file'
              errmessage = 'Scoops3D found a maximum depth less than a minimum depth plus tolerance' 
              CLOSE(14)
              Call WriteError(1,errmessage,problemtype2,solution,'no ',0,' ')
            END IF
            nullbad = COUNT(dmaxarray.eq.rnull.and.zdem.ne.rnull)
            IF (nullbad.gt.0) THEN
              IF (vmax.eq.0._dp.and.armax.eq.0._dp) THEN
                WHERE (dmaxarray.eq.rnull) zdem = rnull
                solution = 'check elevations in specified layer file used to calculate maximum depth'
                errmessage = 'null values (or values higher than DEM) found in maximum depth layer &
                    &file at non-null DEM locations'              
                Call WriteError(0,errmessage,problemtype1,solution,'no ',0,' ')                     
                PRINT *,'Scoops3D will set null maximum depth values to zero at non-null DEM locations' 
                WRITE (39,*)'Scoops3D will set null maximum depth values to zero at non-null DEM locations'
              ELSE
                WHERE (dmaxarray.eq.rnull) dmaxarray = 0.0_dp
                solution = 'check elevations in specified layer file used to calculate maximum depth'
                errmessage = 'null values (or values higher than DEM) found in maximum depth layer file &
                    &at non-null DEM locations'              
                Call WriteError(0,errmessage,problemtype1,solution,'no ',0,' ')                     
                PRINT *,'Scoops3D will set null maximum depth values to zero at non-null DEM locations' 
                WRITE (39,*)'Scoops3D will set null maximum depth values to zero at non-null DEM locations'
              END IF
            END IF          
          END IF  
        END IF 
        CLOSE (14)

        RETURN
        
1000    FORMAT (A)   

        END SUBROUTINE readdepth
