        SUBROUTINE readeq(eqfile)
        
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!      This subroutine reads and processes the ASCII array data files that 
!      contain spatially variable seismic coefficient information. 
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!
!      Called in Readin
!
!      VARIABLES
!
!        delxy -- DEM grid resolution (delta x, delta y)
!        delxy2 -- resolution read for seismic coefficient file
!        eqarray(nx,ny) -- array of minimum depths for each DEM column
!        eqfile -- seismic coefficient file name
!        eqmax -- maximum seismic coefficient
!        eqmin -- minimum seismic coefficient
!        heading -- character variable for reading input
!        i,j -- DEM grid array location
!        ios -- status of read calls
!        nodata2 -- no data value for seismic coefficient file
!        nullsame -- 'true' if the file value for no data is the same as value of
!           rnull, set in CommonData module
!        numeq -- number of seismic coefficient data
!        nx -- number of DEM columns
!        nx2 -- number of columns in seismic coefficient file
!        ny -- number of DEM rows
!        ny2 -- number of rows in seismic coefficient file
!        readline -- counter for number of header lines read
!        rnull -- real null value used throughout program 
!        stuff -- character variable for reading input
!        xll2,yll2 -- x and y origin of seismic coefficient grid, read in header lines
!
!      INPUT FILES READ
!       unit filename
!         14 'inputfilename.#' -- file of depths at each DEM cell
!
!      OUTPUT FILES
!       unit filename
!         20 'inputfilename_out.txt' -- echo of input and results containing
!              overall minimum slip surface data, written in subroutines
!              Readin, Readpiezo, Readpress, Readsearch, Readstrat,
!              Readstrength, Readfailsurf, and Writeout.
!         39 'errors.txt' -- file containing error messages
!
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
    
        USE CommonData
        USE GridData, ONLY: nx,ny,delxy,xll,yll,zdem
        USE MaterialData, ONLY: eqarray,eqmin,eqmax
            
        IMPLICIT NONE 
        
        INTERFACE
          SUBROUTINE WriteError(errtype,problem,problemtype,solution,extraflag,extra1int,extra1ch)
            INTEGER, INTENT(in) :: errtype,extra1int
            CHARACTER(len=*), INTENT(in) :: extraflag,extra1ch,problemtype,problem,solution
          END SUBROUTINE WriteError
        END INTERFACE
        
        INTEGER :: i,j,nx2,ny2,ios,numeq,readline
        
        REAL(dp) :: delxy2,xll2,yll2,nodata2

        CHARACTER(len=*), INTENT(in) :: eqfile
        CHARACTER(60)  :: heading
        CHARACTER(12) :: stuff
        CHARACTER(70)  :: problemtype
        CHARACTER(120)  :: errmessage,solution
        
        LOGICAL :: nullsame  
              
        problemtype = 'reading seismic coefficient file'        
        ios = 0
        numeq = 0
        readline = 0
        nullsame = .TRUE.        
        
        OPEN (14,STATUS = 'old',FILE = eqfile,IOSTAT=ios)
        IF (ios.ne.0) THEN
          errmessage = 'error opening seismic coefficient file:'
          solution = 'check for existence of file with specified name and location'
          Call WriteError(1,errmessage,problemtype,solution,'ch ',0,eqfile)        
        END IF

        DO 
          READ (14,1000,IOSTAT=ios) heading
          IF (ios.ne.0.or.readline.eq.6) EXIT

          solution = 'seismic coefficient file must be in Esri ASCII raster format, as described in Scoops3D manual'
          SELECT CASE (heading(1:5))
          CASE ('ncols','NCOLS')   
            BACKSPACE 14     
            READ (14,*,IOSTAT=ios) stuff,nx2
            readline = readline + 1
            IF (ios.ne.0) THEN
              errmessage = 'error reading "ncols" header line of seismic coefficient file '              
              CLOSE(14)
              Call WriteError(1,errmessage,problemtype,solution,'no ',0,' ')              
            END IF

          CASE ('nrows','NROWS')   
            BACKSPACE 14     
            READ (14,*,IOSTAT=ios) stuff,ny2
            readline = readline + 1
            IF (ios.ne.0) THEN
              errmessage = 'error reading "nrows" header line of seismic coefficient file '              
              CLOSE(14)
              Call WriteError(1,errmessage,problemtype,solution,'no ',0,' ')             
            END IF

          CASE ('xllco','XLLCO')   
            BACKSPACE 14     
            READ (14,*,IOSTAT=ios) stuff,xll2
            readline = readline + 1
            IF (ios.ne.0) THEN
              errmessage = 'error reading "xllcorner" header line of seismic coefficient file '              
              CLOSE(14)
              Call WriteError(1,errmessage,problemtype,solution,'no ',0,' ')             
            END IF
                           
          CASE ('yllco','YLLCO')   
            BACKSPACE 14     
            READ (14,*,IOSTAT=ios) stuff,yll2
            readline = readline + 1
            IF (ios.ne.0) THEN
              errmessage = 'error reading "yllcorner" header line of seismic coefficient file '              
              CLOSE(14)
              Call WriteError(1,errmessage,problemtype,solution,'no ',0,' ')             
            END IF   

          CASE ('cells','CELLS')   
            BACKSPACE 14     
            READ (14,*,IOSTAT=ios) stuff,delxy2
            readline = readline + 1
            IF (ios.ne.0) THEN
              errmessage = 'error reading "cellsize" header line of seismic coefficient file '              
              CLOSE(14)
              Call WriteError(1,errmessage,problemtype,solution,'no ',0,' ')             
            END IF
            
          CASE ('NODAT','nodat')   
            BACKSPACE 14     
            READ (14,*,IOSTAT=ios) stuff,nodata2
            readline = readline + 1
            IF (ios.ne.0) THEN
              errmessage = 'error reading "nodata" header line of seismic coefficient file '              
              CLOSE(14)
              Call WriteError(1,errmessage,problemtype,solution,'no ',0,' ')             
            END IF
            EXIT
                                 
          END SELECT
        END DO   
        
        IF (readline.ne.6) THEN
          errmessage = 'error reading seismic coefficient file -- missing or incorrect header data'              
          CLOSE(14)
          Call WriteError(1,errmessage,problemtype,solution,'no ',0,' ')         
        END IF
        
        solution = 'check extent and cell size of input grid files'        
!     Compare DEM and seismic coefficient file grid dimensions as given in header info.
        IF ((nx.ne.nx2).or.(ny.ne.ny2)) THEN
          errmessage = 'DEM file and seismic coefficient file must have equivalent ncols and nrows'              
          CLOSE(14)
          Call WriteError(1,errmessage,problemtype,solution,'no ',0,' ')         
        END IF
        
        IF (delxy2.ne.delxy) THEN
          errmessage = 'DEM file and seismic coefficient file must have equivalent cellsize'              
          CLOSE(14)
          Call WriteError(1,errmessage,problemtype,solution,'no ',0,' ')         
        END IF
        
        IF (xll2.ne.xll.or.yll2.ne.yll) THEN
          errmessage = 'DEM file and seismic coefficient file must have equivalent xllcorner and yllcorner'              
          CLOSE(14)
          Call WriteError(1,errmessage,problemtype,solution,'no ',0,' ') 
        END IF
                           
        IF (nodata2.ne.rnull) nullsame = .false.

        solution = 'check format of input grid data; see "Program Input" chapter of Scoops3D manual'
        errmessage = 'error reading seismic coefficient file on line number '
        DO j = ny,1,-1
          READ (14,*,IOSTAT=ios) (eqarray(i,j),i=1,nx)
          IF (ios.ne.0.and.i.le.nx) THEN          
            CLOSE(14)
            Call WriteError(1,errmessage,problemtype,solution,'int',ny-j+1,' ') 
          END IF
          numeq = numeq + nx   
        END DO
                                        
        IF (numeq.ne.(nx*ny)) THEN
          solution = 'check dimensions of input data, number of data records should equal ncols*nrows'
          errmessage = 'seismic coefficient file dimensions are incorrect'        
          CLOSE(14)
          Call WriteError(1,errmessage,problemtype,solution,'no ',0,' ')                  
        END IF  
            
        IF (.not.nullsame) WHERE (eqarray.eq.nodata2) eqarray = rnull
!     Check that there are no null or negative values where DEM is non-null
        errmessage = 'Seismic coefficient is < 0 on line number '
        solution = 'make sure there are no null or negative values where DEM is non-null'
        DO j = 1,ny
          DO i = 1,nx
            IF (eqarray(i,j).lt.0._dp.and.zdem(i,j).ne.rnull) THEN              
              CLOSE(14)
              Call WriteError(1,errmessage,problemtype,solution,'int',ny-j+1,' ') 
            END IF
          END DO
        END DO           
        
!     Find minimum and maximum seismic coefficient.        
        eqmin = MINVAL(eqarray,MASK=eqarray.ne.rnull)
        eqmax = MAXVAL(eqarray,MASK=eqarray.ne.rnull)
   
        CLOSE (14)

        RETURN
        
1000    FORMAT (A)   

        END SUBROUTINE readeq
