        PROGRAM Scoops3D
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!
!      This U.S. Geological Survey program, Scoops3D, analyzes 3D slope stability 
!      throughout a digital landscape.  It determines the minimum factor of safety 
!      associated with each cell in a digital elevation model (DEM).  
!
!      More information on the theoretical basis of the program, program operation, 
!      practical considerations, and testing of Scoop3D can be found in:  
!         Reid, M.E., Christian, S.B., Brien, D.L., and Henderson, S.T., 2014, 
!         Scoops3D-Software to Analyze Three-Dimensional Slope Stability Throughout
!         a Digital Landscape: U.S. Geological Survey Techniques and Methods, book 14, 
!         chap. A1  [http://pubs.usgs.gov/tm/14/a01].
!
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!
!      Program coding: Sarah Christian, Dianne Brien, Mark Reid
! 
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!
!      LIST OF VARIABLES
!
!        area(nnset) -- surface area of subset
!        armin,armax -- minimum and maximum area bounds for valid slip surface
!        colfile -- flag for opening error file reporting cols < limcol
!        colxy(nx,ny) -- length of each column side (delxy for full columns)
!        costruedip(nx,ny) -- cosine of the true dip of the sliding base at the
!           center of each column.
!        dArray1(nx,ny),dArray2(nx,ny) -- arrays used in Bishop method for components
!            of normal force in driving moment
!        dArray1_2d(nx,ny),dArray2_2d(nx,ny) -- arrays used in Bishop method for components
!            of normal force in 2d driving moment
!        dr -- radius increment for search
!        delxy -- DEM grid resolution
!        demflag -- indicates whether file newDEM will be generated
!        error -- file allocation flag       
!        felfsmin(nx,ny) -- array of minimum Ordinary (Fellenius) FOS at each DEM column
!        fgrid -- minimum factor of safety at search grid point
!        filtcount(nx,ny) -- counter of number of times each DEM cell was filtered
!           by minma.
!        foscut -- factor of safety cutoff for removing slip surface from DEM
!        fsangle(nx,ny) -- array of fall angles of minimum scoop at each DEM column
!        fsangle2d(nx,ny) -- array of fall angles of minimum 2d slice at each DEM column
!        fsarclength(nx,ny) -- array of arclengths of minimum 2d slice at each DEM column
!        fsarea2d(nx,ny) -- array of areas of minimum 2d slice at each DEM column
!        fsmin(nx,ny) -- array of minimum FOS at each DEM column
!        fsminold(i,j) -- last minimum factor of safety at each DEM column used to
!           compare change in FOS during seed iterations to see if criterion is met
!        fsmin2d(nx,ny) -- 2D array of minimum FOS at each DEM column
!        fsmin2d3d(nx,ny) -- 3D minimum factor of safety associated with min 2D slice
!           at each DEM cell
!        fsmin3d2d(nx,ny) -- 2D minimum factor of safety associated with min 3D scoop
!          at each DEM column
!        fsrad(nx,ny) -- array of radius of minimum scoop at each DEM column
!        fsrad2d(nx,ny) -- array of radii of minimum 2D FOS slice at each DEM column
!        fsvol(nx,ny) -- volume of minimum FOS scoop at each DEM column
!        fswidth2d(nx,ny) -- array of widths of minimum 2D slice at each DEM column
!        fsx(nx,ny),fsy(nx,ny),fsz(nx,ny) -- search spheroid center of minimum FOS 
!           scoop at each DEM cell
!        fsx2d(nx,ny),fsy2d(nx,ny),fsz2d(nx,ny) -- search spheroid center of minimum FOS 
!           2D slice at each DEM column
!        halfdelxy -- 1/2 DEM grid spacing
!        icen,jcen -- center of search spheroid in i,j coordinates for lattice search
!        ierr -- error flag indicating initial radius problem (2) 
!           or too many subsets (1)
!        icritlattice -- flag for creating file critfoslattice_out.3D for minimum FOS of 
!            critical surfaces at each search grid node
!        ilatt,jlatt -- indices for search lattice files
!        ilattice -- flag for creating file foslattice_out.3D for minimum FOS at each 
!           search grid node
!        inn(nx,ny) -- number of nodes of each column bounded by spheroid or
!          equals -1 if truncated surface.
!        inscoop(nx+1,ny+1) -- indicates whether DEM column node is bounded by spheroid
!        inull -- integer null value used throughout program
!        irefine -- flag for implementing coarse to fine search iterations
!        iretro -- counter for number of retrogressions
!        iseed -- grid refinement iteration count also used to fill srchlatt
!           lattice. iseed=1 represents initial coarse grid locations
!        ismin,ismax -- minimum and maximum boundaries of search grid on x-axis,
!           specified relative to DEM cells i=1 through nx
!        isqout -- flag for creating search quality output files
!        isrchmin,isrchmax -- array bounds of searchout output file
!        isrchnum,jsrchnum -- the number of search lattice nodes in x and y directions
!        isubsurf -- flag for creating 3D file of FOS below DEM surface,
!            1=create file in ijk format, 2=create file in xyz format
!            3=create .3D format
!        jsmin,jsmax -- minimum and maximum boundaries of search grid on y-axis,
!           specified relative to DEM cells j=1 through ny
!        jsrchmin,jsrchmax -- array bounds of searchout output file
!        kcen -- search lattice z coordinate
!        kseed -- array number representing search lattice elevation at current i,j
!        kset -- number of valid subsets
!        lastrad -- previous radius before increase by dr
!        linein(nx,ny) -- array of flags for whether 2D slide directon line intersects
!           column
!        mcol(nx,ny) -- number of columns in minimum FOS scoop at each DEM column
!        mfile -- flag indicating whether filter file was opened
!        mini(nnset),maxi(nnset) -- minimum and maximum i bounds of each subset
!        minj(nnset),maxj(nnset) -- minimum and maximum j bounds of each subset
!        multres -- resolution multiplier for coarse search lattice
!        iradflag -- indicates whether valid initial range subset has been found
!           (1=no, 0=yes, 2=no valid sets after 10 radius adjustments.)
!        nkseed(nxsrch,nysrch) -- number of k values for current seed at each i,j
!           search node
!        nnset -- maximum number of subsets
!        nrange(nnset) -- array of flags for whether set fits volume or area criteria 
!           range
!        nres -- search lattice loop multiplier
!        nretro -- number of failure retrogressions to compute
!        nseed -- number of new search nodes generated from current seeds.
!        nseg -- number of search slip directions for each search spheroid
!        nset -- number of land subsets found
!        nsetmax -- flag for exceeding number of subsets allowed nnset
!        nsrchpt -- total number of search points
!        nsrchres -- resolution of finest search grid relative to DEM resolution.
!           Acts as a multiplier of DEM resolution delxy
!        nstate -- flag used by subroutine Criteria to indicate whether valid sets
!           were found at a particular search node.
!           100 = recalculate radius to find first good set at a search node.
!           300 = increase radius by dr and continue. This flag occurs if good
!                 sets were found, or only sets not grown out of range are too
!                 small to be in range.
!           500 = go to next search grid point because no good sets possible.
!        ntry -- total number of slip surfaces for which FOS is calculated
!        nullhi -- positive real null value used for initializations and set in CommonData.
!        nxsrchout,nysrchout -- number of search nodes in x and y directions in output
!           search grid file, includes intersection of search range with DEM range.
!        nx -- number of DEM cells in x direction
!        ny -- number of DEM cells in y direction
!        nz -- number of nodes used for 3D FOS values when isubsurf=1 or 2 or 3
!        oangle -- slip angle associated with overall minimum factor of safety
!        ocol -- number of columns in minimum scoop
!        ofos -- overall minimum factor of safety
!        ofsx,ofsy,ofsz -- search spheroid center of minimum FOS scoop
!        omset -- set number of overall minimum scoop
!        oldzcen -- elevation of previous search center
!        omini,omaxi,ominj,omaxj -- minimum and maximum bounds of all sets included
!           in search spheroid
!        oomini,oomaxi,oominj,oomaxj -- minimum and maximum bounds of all DEM cells 
!           included in any spheroid with a calculated FOS value
!        orad -- longest axis of spheroid associated with overall minimum factor of safety
!        outnodes(2,nx,ny) -- corner location of column nodes which fall outside the
!           search spheroid, also used to indicate included node for single node columns
!        outputdir -- optional directory path to place output files into
!        ovangle -- slip angle of largest volume scoop with FOS < cutoff
!        ovmset -- set number of largest volume scoop with FOS < cutoff
!        ovol, oarea - volume and area of minimum  FOS scoop
!        ovfos -- factor of safety of largest volume scoop with FOS < cutoff
!        ovrad -- radius of largest volume scoop with FOS < cutoff
!        ovfsx,ovfsy,ovfsz -- search spheroid center of largest volume scoop with 
!           FOS < cutoff
!        ovvol,ovarea -- volume and area of largest volume scoop with FOS < cutoff
!        ozb(nx,ny) -- array containing elevation of DEM after scoops are removed 
!           depending on value of variable 'remove' (A, L, or M)
!        r2 -- variable used for calculating maximum radius
!        rad -- search spheroid axes perpendicular to strike of slope
!        radcrit -- minimum radius to meet current size criteria
!        radiniprev -- used in determination of first radius estimate at new search
!           lattice elevation. Equals previous radius  at current search center 
!           location that met criteria constrains, or = -1 if radius not yet calculated
!        radsq -- square of search spheroid axes perpendicular to strike of slope
!        remove -- A= all scoops < FOS cutoff will be removed from new
!           DEM.  L= only largest volume scoop < FOS cutoff will be 
!           removed. M= the scoop with the minimum FOS will be removed,
!           N= no scoops removed.
!        rf(nx,ny) -- resisting force of each column
!        rf2d(nx,ny) -- 2D resisting force of each column
!        rmax -- maximum radius allowed
!        rnull -- real negative null value used throughout program and set in CommonData.
!        searchgrid - array to specify search grid points. Values less than 0 are
!           not searched.
!        searchout -- output array of area covered by search grid and indicating
!           which grid points were searched and whether they fell inside or outside
!           the DEM.
!        set(nnset) -- number of columns in a contiguous set
!        setflag(nnset) -- indicates valid subset (0), or invalid by containing
!           truncated node (1), or adjacent to DEM boundary (2), or empty (-1)
!        settype(nnset) -- indicates subset type as land (1), water (2) or air (3)
!        sindip(nx,ny) -- sine of the dip of the sliding base in the direction of 
!           the slide at the center of each column.
!        single -- flag for calculating single slip surface
!        spanleft,spanright -- arrays containing bounds of continuous set of included nodes 
!           to left or right of included nodes on previous row
!        spanrow -- array of positive or negative row count representing number of rows from
!           seed row, i,j
!        srchct -- running sum of number of search grid points completed
!        srchfile -- flag indicating whether using search file to specify valid
!           search grid nodes
!        srchlatt(isrchnum,jsrchnum,ksmax) -- search lattice array with seed
!           iteration number at each searched node. 
!        srchpct -- percentage of search file completed
!        srchprint -- flag to report percent of completed search
!        srchpt -- flag for valid search grid location, 1=valid,0=not
!        subset(nx,ny) -- indicates set membership of DEM columns
!        vadcrit -- indicates primary control for initial radius (1 for volume,
!           2 for area, and 3 for depth)
!        va1max -- maximum volume or area (primary criterion) of a subset
!        version -- version of Scoops
!        vmin,vmax -- minimum and maximum volume bounds for valid slip surface
!        vol(nx,ny) -- volume of column in subset
!        volume(nnset) -- volume of subset
!        xcen,ycen,zcen -- center of search spheroid relative to DEM origin
!        xcolcount -- number of critical surfaces with total # of columns < limcol
!        xcount -- number of surfaces eliminated due to m-alpha limits and/or nonconvergence
!        xdiff,ydiff -- variables used to calculate maximum radius
!        xlast,ylast,zlast -- last search grid node tried
!        xll,yll -- x and y origin of DEM grid, read in header lines
!        xslope(nx,ny),yslope(nx,ny) -- x and y slope vectors of DEM surface
!        zb(nx+1,ny+1) -- base of slip surface at each node
!        zmid(nx,ny) -- base of slip surface at center of intersected columns
!        zd -- elevation of DEM grid at same x,y as spheroid center
!        zdem(nx,ny) -- DEM elevations
!        zdemnodes(nx+1,ny+1) -- DEM node elevations averaged from srrounding cells
!        zfos(nx,ny,nz) -- 3D array of minimum factor of safety below DEM surface
!        zmin -- minimum DEM elevation
!        zsave -- representative distance from spheroid center to ground that yields
!           good initial volume or area
!        zsmin,zsmax -- minimum and maximum elevations of search grid
!        zsrchres -- resolution of fine search grid on z-axis
!        zzsrchres -- resolution of current search grid on z-axis
!  
!      INPUT FILES
!       unit filename
!         10 search filename -- file that defines search grid. Opened and
!              closed in Readsearch.
!         12 input filename -- file of basic input information. Opened in
!              Readin.
!         13 DEM filename -- file of DEM input information. Opened and
!              closed in Readdem.          
!         13 3D pressure filename -- file containing 3D pressure input.
!              Opened and closed in Readpress.
!         13 3D strength filename -- file of 3D strength for each DEM column.
!              Opened and closed in Readstrength.
!         14 piezometric filename -- file of piezometric elevations at each
!              DEM column.  Opened and closed in Readpiezo.
!         14 strat layers filenames -- files of layer boundary elevations.
!              Each layer should have own file with extension giving the 
!              layer number, i.e. 'layer.1'. Opened and closed in Readstrat.
!
!      OUTPUT FILES
!       unit filename
!         15 'foslattice_out.3D' -- file of minimum factor of safety   
!               at each search grid point. Opened and written in Scoops 
!              if ilattice = 1.
!         16 'foslocal_out.txt' -- file of local resisting force, components
!              of the resisting force, driving force and factors
!              of safety for each column in minimum slip surface.
!              Opened and written in Ordinary if foslocal = 1.
!         17 'filtergrid_out.txt' -- file of number of times each DEM cell was
!              filtered due to absmina.  Opened and written in
!              Writeout if filter = 1 and mflag = 1.
!         19 'fos2d_out.asc' -- file of minimum 2D factors of safety at each
!              DEM column. Opened and written in Writeout if ifos2d=1.
!         20 'inputfilename_out.txt' -- echo of input and results containing
!              overall minimum slip surface data, written in subroutines
!              Readin, Readpiezo, Readpress, Readsearch, Readstrat,
!              Readstrength, and Writeout.
!         21 'spheroids_out.okc' -- file of spheroid center coordinates, radius, 
!              fall angle, # of columns in scoop, vol, and factor of safety 
!              of the least stable scoop at each DEM column. Opened and written
!              in Writeout.
!         22 'critcheck_out.asc' -- file of volume (or area) checks.  If volume of minimum
!              FOS scoop is < vmin+tol shows 1, if > vmax-tol shows 2,
!              if not close to volume boundary shows 0. Opened and written
!              in Writeout if isqout = 1.
!	        23 'subsurffos_out.txt' or '.vtk' -- file of minimum FOS in 3D array, written
!              if isubsurf = 1 or 2 or 3. Opened and written in subroutine writeout.
!         24 'newDEM_out.asc' -- file of replace DEM with certain scoops removed 
!              depending on value of flag remove. Opened and written in Writeout.
!         26 'fos3d_out.asc' -- file of minimum factors of safety at each
!              DEM column. Opened and written in Writeout.
!         28 'searchgrid_out.asc' -- search grid file.  Record of which nodes were searched
!              and whether they fell within the DEM bounds or not. Opened
!              and written in Scoops if isqout = 1.
!         29 'numcols_out.asc' -- file of # of columns for min FOS at each 
!              DEM column. Opened and written in Writeout if isqout = 1.
!         32 'fosretro_out.txt' -- file of retrogression data. Opened in Readin
!              and written in Scoops, Readin and Fos if Remove=A and nretro>0.
!         33 'filter_out.txt' -- file of data describing surfaces which fell outside
!              of user-defined m-alpha and/or FOS limits and nonconverging surfaces.  
!              Opened and written in FOS if filter = 1.
!         34 'ncolerr_out.txt' -- file of information about critical surfaces with
!              fewer columns than user-defined limit, limcol. Opened and
!              written in Fos if colfile = 1.
!         35 'arcs2d_out.okc' -- file of spheroid center coordinates, radius, fall angle,
!              and 2D factor of safety of the least stable 2D solution at each DEM column. 
!              Opened and written in Writeout if ifos2d=1.
!         36 'slope_out.asc' -- slope of DEM. Opened and written in Writeout.
!         37 'fos3drel_out.asc' -- file of relative minimum factors of safety at each
!              DEM column. Relative factor of safety is defined as factor of safety divided
!              by the global minimum factor of safety (ofos).  Opened and written in Writeout 
!              if irelfos=1.
!         38 'fos2drel_out.txt' -- file of relative minimum 2D factors of safety at each
!              DEM column. 2D relative factor of safety is defined as factor of safety divided
!              by the global minimum 2D factor of safety (ofos2d).Opened and written in Writeout 
!              if ifos2d=1 and irelfos=1.
!         39 'errors.txt' -- file containing error messages
!         40 'spheresltcut_out.txt' --  file of sphere center coordinates, radius,
!              fall angle, # of columns in scoop, vol, and factor of safety of all
!              surfaces with F<foscut. Opened in readin and written in fos if remove='A'.
!         43 'ordfos3d_out.asc' -- file of minimum factors of safety computed by the
!              Ordinary (Fellenius) method at each DEM column. Opened and written in Writeout.
!              if method = 'b'
!         44 'boundcheck_out.asc' -- file of check for boundary limitations in
!              the search lattice. Opened and written in Writeout if isqout = 1
!         45 'critfoslattice_out.3D' -- file of minimum factor of safety   
!              for critical surfaces at each search grid point. Opened and written in Scoops 
!              if icritlattice = 1.
!         47 'fosvol_out.asc' -- file of volumes or areas for min FOS at each DEM column. 
!              Opened and written in Writeout.
!         48 'fosarea_out.asc' -- file of areas for min FOS at each DEM column.  
!              Opened and written inWriteout.
!         49 'fosdepth_out.asc' -- file of depths for min FOS at each DEM column. 
!              Opened and written in  Writeout.
!
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

        
        USE CommonData, ONLY: dp,rnull,inull,nullhi,filin,outputdir,int1,nfile,bfile
        USE GridData, ONLY: nx,ny,nz,delxy,xcen,ycen,zcen,zmin,rad,xll,yll,demflag,&
                           radsq,zdem,lengthunits,halfdelxy,lastrad,cellarea,&
                           invdelxy,pi,oldzcen,delxyvol,iretro,zdemnodes
        USE SearchData, ONLY: srchfile,ismin,ismax,jsmin,jsmax,nsrchres,irefine,nkseed,&
                           zsmin,zsmax,zsrchres,vmin,vmax,armin,armax,ksmax,&
                           isqout,vadcrit,searchgrid,nsrchpt,srchlatt,multres,&
                           isrchmin,jsrchmin,isrchmax,jsrchmax,nxsrchout,nysrchout,searchout,&
                           iradflag,zsave,radiniprev,icen,jcen,zd,va1max,tol,&
                           rmax,ncount,nrec,linenum,zzsrchres
                           
        USE FOSData, ONLY: remove,ntry,isubsurf,single,ofos,orad,oangle,omset,fgrid,&
                           ofsx,ofsy,ofsz,ovol,oarea,ovfos,ovrad,ovangle,ocol,ovmset,&
                           ovfsx,ovfsy,ovfsz,ovvol,ovarea,fsmin,fsrad,dr,xcount,&
                           fsangle,fsx,fsy,fsz,fsvol,zfos,ozb,mcol,mfile,icrit,&
                           fsmin2d,ifos2d,fsmin3d2d,fsmin2d3d,fsminold,method,felfsmin,&
                           fsx2d,fsy2d,fsz2d,fsrad2d,fsangle2d,fsarea2d,mcol2d,ofos2d,&
                           fsarea,fswidth2d,fsarclength,filtcount,icritlattice,ilattice,linein,&
                           fsdepth,fsdepth2d,odepth,ovdepth,isizefiles,nretro,foscut
        USE SetData, ONLY: nnset,nrange,area,volume,mini,maxi,minj,maxj,set,setflag,subsetwarn,&
                           ncol,i1,i2,j1,j2,nstate,spanrow,spanleft,spanright,nset,mset,xcolcount,&
                           colfile,nsetmax,maxspan
        USE ColumnData, ONLY: xslope,yslope,subset,inn,inscoop,zb,colxy,vol,zmid,outnodes
        USE BishopArrays, ONLY: rf,sindip,costruedip,rf2d,sindip2d,cosdip2d
        USE ResultsStats, ONLY: maxslope,minslope
        USE DepthData, ONLY: nstate200,adjustdepthrad,idepthflag,maxdepthrad,dminarray,dmin,&
                           depthmin
        
        IMPLICIT NONE

        INTERFACE
          SUBROUTINE Readin(version)
            CHARACTER(len=*), INTENT(in) :: version
          END SUBROUTINE Readin
        
          SUBROUTINE WriteError(errtype,problem,problemtype,solution,extraflag,extra1int,extra1ch)
            INTEGER, INTENT(in) :: errtype,extra1int
            CHARACTER(len=*), INTENT(in) :: extraflag,extra1ch,problemtype,problem,solution
          END SUBROUTINE WriteError
          
          SUBROUTINE SeedSearch(iseed,nseed,nres,nzres,isrchnum,jsrchnum)
            IMPORT :: dp
            INTEGER, INTENT(inout) :: iseed
            INTEGER, INTENT(out) :: nseed
            INTEGER, INTENT(in) :: nres,nzres,isrchnum,jsrchnum
          END SUBROUTINE SeedSearch

          SUBROUTINE Radius(ierr)
            INTEGER, INTENT(out) :: ierr
          END SUBROUTINE Radius
          
          SUBROUTINE Subsets(kset,ierr)
            INTEGER, INTENT(out) :: kset,ierr
          END SUBROUTINE Subsets
          
          SUBROUTINE Volumes
          END SUBROUTINE Volumes

          SUBROUTINE Criteria(crit1,crit2,c1min,c1max,c2min,c2max,tol)
            IMPORT :: dp,nnset
            REAL(dp), INTENT(in) :: crit1(nnset),crit2(nnset)
            REAL(dp), INTENT(in) :: c1min,c1max,c2min,c2max,tol
          END SUBROUTINE Criteria

          SUBROUTINE FOS
          END SUBROUTINE FOS

          SUBROUTINE Writeout(runtime)
            REAL, INTENT(in) :: runtime
          END SUBROUTINE Writeout
          
          REAL(dp) FUNCTION Newzcen(ilatt,jlatt,iseed)
            IMPORT :: dp
            INTEGER, INTENT(in) :: ilatt,jlatt,iseed
          END FUNCTION Newzcen
        END INTERFACE

        INTEGER :: skip,i,j,nn,m,l,mm,ll,kset,nzres,nnsrchpt
        INTEGER :: error,nseed,nres,kseed,iiseed,ierr,kcen,initmultres
        INTEGER :: srchct,srchpcnt,srchprint,isrchnum,jsrchnum,ilatt,jlatt
        INTEGER :: zcount,iseed,mmset

        REAL(dp) :: r2,xdiff,ydiff,zlast,zdemsrch
        REAL :: starttime,endtime,runtime,slope,zdn(4)
        
        CHARACTER(8)  :: date
        CHARACTER(10) :: time        
        CHARACTER(50)  :: version
        CHARACTER(4)  :: fosmeth    
        CHARACTER(70)  :: problemtype
        CHARACTER(120)  :: errmessage,solution
        CHARACTER(10) :: cnum
        CHARACTER(1) :: charetro
        
        ncount = 0
        problemtype = 'Scoops3D main program' 
        error = 0
        subsetwarn = 0       

        version = '1.3.01, rev. 03/1/2023'

!     Read basic input data.
        CALL Readin (version)

!     Allocate arrays and initialize variables and arrays.
        ALLOCATE (spanrow(maxspan),spanleft(maxspan),spanright(maxspan),STAT=error)
        ALLOCATE (ncol(nnset),nrange(nnset),area(nnset),volume(nnset),STAT=error)
        ALLOCATE (xslope(nx,ny),yslope(nx,ny),STAT=error)
        ALLOCATE (fsrad(nx,ny),fsx(nx,ny),fsy(nx,ny),fsz(nx,ny),fsvol(nx,ny),STAT=error)
        ALLOCATE (mcol(nx,ny),fsmin(nx,ny),fsangle(nx,ny),STAT=error)
        ALLOCATE (fsarea(nx,ny),filtcount(nx,ny),zmid(nx,ny),STAT=error)
        ALLOCATE (subset(nx,ny),inn(nx,ny),inscoop(nx+1,ny+1),zb(nx+1,ny+1),STAT=error)
        ALLOCATE (outnodes(2,nx,ny),colxy(4,nx,ny),vol(nx,ny),STAT=error)
        ALLOCATE (mini(nnset),minj(nnset),maxi(nnset),maxj(nnset),STAT=error)
        ALLOCATE (set(nnset),setflag(nnset), STAT=error)
        IF (idepthflag.ne.0.or.isizefiles.ne.0) ALLOCATE (fsdepth(nx,ny))          
        IF (ifos2d.eq.1) THEN
          ALLOCATE (fsrad2d(nx,ny),fsx2d(nx,ny),fsy2d(nx,ny),fsz2d(nx,ny),STAT=error)
          ALLOCATE (fsarea2d(nx,ny),mcol2d(nx,ny),fsangle2d(nx,ny),STAT=error)
          ALLOCATE (fsmin2d(nx,ny),fsmin3d2d(nx,ny),fsmin2d3d(nx,ny),STAT=error)
          ALLOCATE (fswidth2d(nx,ny),fsarclength(nx,ny),linein(nx,ny),STAT=error)
          IF (idepthflag.ne.0.or.isizefiles.ne.0) ALLOCATE (fsdepth2d(nx,ny))
        END IF
        IF (irefine.ne.0) THEN
          isrchnum = INT((ismax-ismin)/nsrchres)+1
          jsrchnum = INT((jsmax-jsmin)/nsrchres)+1 
          ALLOCATE (fsminold(nx,ny),srchlatt(isrchnum,jsrchnum,ksmax),STAT=error)
          ALLOCATE (nkseed(isrchnum,jsrchnum),STAT=error)
        END IF
        IF (method.eq.'b') THEN
          ALLOCATE (sindip(nx,ny),costruedip(nx,ny),STAT=error)
          ALLOCATE (rf(nx,ny),STAT=error)
          IF (ifos2d.eq.0) ALLOCATE (linein(nx,ny),STAT=error)
          ALLOCATE (felfsmin(nx,ny),STAT=error)!        
          IF (ifos2d.ne.0) THEN
            ALLOCATE (sindip2d(nx,ny),cosdip2d(nx,ny),rf2d(nx,ny),STAT=error)
          END IF         
        END IF
        IF (icritlattice.gt.0) ALLOCATE (linenum(nx,ny),STAT=error)
        IF (error.ne.0) THEN
          errmessage = 'arrays not allocated successfully' 
          solution = 'reduce memory requirements. See "Practical Considerations" chapter of Scoops3D manual'
          Call WriteError(1,errmessage,problemtype,solution,'no ',0,' ')         
        END IF
        
        halfdelxy = .5_dp*delxy
        delxyvol = delxy*(1._dp/12._dp)
        iretro = 0
        initmultres = multres

!     Allocate 3D array of FOS.
        IF (isubsurf.ne.0) THEN
          ALLOCATE (zfos(nx,ny,nz), STAT=error)
          IF (error.ne.0) THEN
            errmessage = 'subsurface factor of safety array not allocated successfully' 
            solution = 'reduce memory requirements - select "isubsurf"=0. See "Practical Considerations" chapter of Scoops3D manual'
            Call WriteError(1,errmessage,problemtype,solution,'no ',0,' ')                   
          END IF    
        END IF
       
!     Allocate search array for searchgrid.out
        IF (isqout.ne.0) THEN
          IF (single.eq.0) THEN
            isrchmin = MIN(1,ismin)
            jsrchmin = MIN(1,jsmin)
            isrchmax = MAX(nx,ismax)
            jsrchmax = MAX(ny,jsmax)
          END IF            
          nxsrchout=(isrchmax-isrchmin)+1
          nysrchout=(jsrchmax-jsrchmin)+1
          ALLOCATE (searchout(isrchmin:isrchmax,jsrchmin:jsrchmax),icrit(nx),STAT=error)
          IF (error.ne.0) THEN
            errmessage = 'search array not allocated successfully' 
            solution = 'reduce memory requirements. See "Practical Considerations" chapter of Scoops3D manual'
            Call WriteError(1,errmessage,problemtype,solution,'no ',0,' ')                   
          END IF 
!     Initialize output search array to -1 if inside DEM boundaries, or -11 if outside.
          DO j = jsrchmin,jsrchmax
            DO i = isrchmin,isrchmax
              IF ((i.gt.0).and.(j.gt.0).and.(i.le.nx).and.(j.le.ny)) THEN
                IF (zdem(i,j).ne.rnull) THEN
                  searchout(i,j) = -1_int1
                ELSE
                  searchout(i,j) = -11_int1
                END IF
              ELSE
                searchout(i,j) = -11_int1
              END IF   
            END DO
          END DO      
          IF (single.ne.0) THEN
            IF (xcen.gt.0) THEN
              icen = INT(xcen*invdelxy) + 1
            ELSE
              icen = INT(xcen*invdelxy)
            END IF
            IF (ycen.gt.0) THEN 
              jcen = INT(ycen*invdelxy) + 1
            ELSE
              jcen = INT(ycen*invdelxy)
            END IF  
            IF (icen.ge.isrchmin.and.icen.le.isrchmax.and.&
                jcen.ge.jsrchmin.and.jcen.le.jsrchmax)&           
              searchout(icen,jcen) = 2_int1
          END IF
        END IF    
        
        CALL cpu_time(starttime)

!     Calculate components of DEM slope
25      minslope = 180.
        maxslope = 0.        
        DO j = 1,ny
          DO i = 1,nx
            IF (i.gt.1.and.j.gt.1.and.i.lt.nx.and.j.lt.ny) THEN
              IF (ALL(zdem(i-1:i+1,j-1:j+1).gt.rnull)) THEN
                xslope(i,j)= ((zdem(i-1,j+1)+2.0_dp*zdem(i-1,j)+zdem(i-1,j-1))&
                          -(zdem(i+1,j+1)+2.0_dp*zdem(i+1,j)+zdem(i+1,j-1)))&
                          * invdelxy * 0.125_dp
                yslope(i,j)= ((zdem(i-1,j-1)+2.0_dp*zdem(i,j-1)+zdem(i+1,j-1))&
                          -(zdem(i-1,j+1)+2.0_dp*zdem(i,j+1)+zdem(i+1,j+1)))&
                          * invdelxy * 0.125_dp                          
                slope = ATAN(sqrt(xslope(i,j)*xslope(i,j)+yslope(i,j)*yslope(i,j)))* 57.2957795_dp
                minslope = MIN(minslope,slope)
                maxslope = MAX(maxslope,slope)
              ELSE
                xslope(i,j)= 0.0
                yslope(i,j)= 0.0
              END IF
            ELSE
              xslope(i,j)= 0.0
              yslope(i,j)= 0.0
            END IF     
          END DO
        END DO
        
!     If printing mimimum FOS at each search lattice node, open output file
!     and write header information.                    
        IF (ilattice.ne.0) THEN    
          IF (iretro.gt.0) THEN    
            IF (ilattice.eq.2) THEN         
               OPEN (15,STATUS='scratch')     
            ELSE
              IF (outputdir.ne.'') THEN
                OPEN (15,STATUS='replace',file = outputdir(1:LEN_TRIM(outputdir))//&
                   filin(bfile:nfile)//'_foslattice'//charetro//'_out.3D')
              ELSE
                OPEN (15,STATUS='replace',file = &
                   filin(bfile:nfile)//'_foslattice'//charetro//'_out.3D')
              END IF              
           END IF    
          ELSE     
            IF (ilattice.eq.2) THEN              
               OPEN (15,STATUS='scratch')           
            ELSE
              IF (outputdir.ne.'') THEN            
                OPEN (15,STATUS='replace',file = outputdir(1:LEN_TRIM(outputdir))//&
                    filin(bfile:nfile)//'_foslattice_out.3D')
              ELSE
                OPEN (15,STATUS='replace',file = &
                    filin(bfile:nfile)//'_foslattice_out.3D')
              END IF              
            END IF    
          END IF
          IF (method.eq.'b') then 
             fosmeth = 'Bish'
          ELSE
             fosmeth = 'Ord '
          END IF
          IF (LEN_TRIM(lengthunits).eq.1) THEN
             WRITE (15,1210) lengthunits,lengthunits,lengthunits,fosmeth
          ELSE
            IF (LEN_TRIM(lengthunits).eq.2) THEN
             WRITE (15,1215) lengthunits,lengthunits,lengthunits,fosmeth
            ELSE
             WRITE (15,1220) fosmeth
            END IF
          END IF  
        END IF
       
!     If using retrogression, write number to 'fosretro.out' if remove=A.
!     Set variables for retrogression file names
        IF (nretro.gt.0.and.single.eq.0) THEN
          IF (remove.eq.'A') WRITE (32,1000) iretro
          cnum = '0123456789'
          charetro = cnum(iretro+1:iretro+1)              
        END IF
        
!     Initialize variables needed for each retrogression
        inscoop = 0
        set = 0
        nrec = 1
        zb = rnull
        fsmin = nullhi
        IF (method.eq.'b') felfsmin = nullhi
        fsarea = rnull
        fsangle = rnull
        fsrad = rnull
        fsx = rnull
        fsy = rnull
        fsz = rnull
        ofos = nullhi
        ofos2d = nullhi
        orad = 0.0_dp
        oangle = 0.0_dp
        ofsx = 0.0_dp
        ofsy = 0.0_dp
        ofsz = 0.0_dp
        omset = 0
        ovol = 0.0_dp
        oarea = 0.0_dp
        odepth = 0.0_dp
        ocol = 0
        ovfos = nullhi
        ovrad = 0.0_dp
        ovangle = 0.0_dp
        ovmset = 0
        ovfsx = 0.0_dp
        ovfsy = 0.0_dp
        ovfsz = 0.0_dp
        ovvol = vmin
        ovarea = 0.0_dp
        ovdepth = 0.0_dp
        nrange = 0
        xcount = 0
        xcolcount = 0 
        filtcount = 0
        IF (idepthflag.ne.0.or.isizefiles.ne.0) fsdepth = rnull
        IF (ifos2d.ne.0) THEN
          fsmin2d = nullhi
          fsmin2d3d = nullhi
          fsmin3d2d = nullhi
          fsangle2d = rnull
          fsrad2d = rnull
          fsx2d = rnull
          fsy2d = rnull
          fsz2d = rnull
          fsarea2d = rnull
          mcol2d = inull
          fswidth2d = rnull
          fsarclength = rnull
          linein = 0
          IF (idepthflag.ne.0.or.isizefiles.ne.0) fsdepth2d = rnull
        END IF
        fsvol = rnull
        mcol = inull
        mfile = 0
        colfile = 0
        nsetmax = 0
        ntry = 0
        nnsrchpt = nsrchpt
        IF (isubsurf.ne.0) zfos = nullhi
        multres = initmultres
        
        IF (irefine.ne.0) THEN  ! Initial coarse search  
          nres = multres * nsrchres
          fsminold = rnull
          nzres = multres
          zzsrchres = REAL(nzres,dp) * zsrchres
          srchlatt = 0
        ELSE
          zzsrchres = zsrchres  
          nres = nsrchres   
          nzres = 1 
        END IF
        
!     If not doing single run for factors of safety
        IF (single.eq.0) THEN
          iseed = 0
        
          PRINT *,filin(bfile:LEN_TRIM(filin))," - Starting search using Scoops3D"  
          
! Do until restol is reached at every DEM cell or no new search seeds are produced
          DO      
            srchct = 0 
            srchprint = 0
!     If doing coarse to fine iterative search          
            IF (irefine.ne.0.and.iseed.gt.0) THEN
              multres = max(int(multres*0.5_dp),1)
              nres = multres * nsrchres
              nzres = multres
              zzsrchres = REAL(nzres,dp)*zsrchres
              CALL SeedSearch(iseed,nseed,nres,nzres,isrchnum,jsrchnum)
              nnsrchpt = nseed
              IF (nseed.eq.0) EXIT
            END IF
                        
            IF (iseed.le.1) THEN
              skip = nres 
            ELSE  
!     If multres is odd will need to check for new seeds at minimum resolution
              skip = nsrchres
            END IF  
            
!     Loop through search grid searching for minimum FOS.
            DO jcen = jsmin,jsmax,skip
              DO icen = ismin,ismax,skip
!     Check if using search file and then whether icen,jcen is a valid location.  
!     If using search lattice refinement method, search file only applies to
!     coarse search.        
                IF (srchfile.ne.0.and.iseed.eq.0) THEN
                  IF (searchgrid(icen,jcen).lt.1) CYCLE
                END IF
!     If doing lattice refinement                
                IF (irefine.ne.0) THEN
!     Calculate indices of search lattice array
                  ilatt = INT((icen-ismin)/nsrchres)+1
                  jlatt = INT((jcen-jsmin)/nsrchres)+1
!     Check whether at a valid i,j search location.                  
                  IF (iseed.gt.1) THEN
                    IF (nkseed(ilatt,jlatt).eq.0) CYCLE  
                  END IF
                END IF
                
                IF (icen.gt.0.and.icen.le.nx.and.jcen.gt.0.and.jcen.le.ny) &
                  zdemsrch = zdem(icen,jcen)
                oldzcen = rnull

!     Calculate percent of search completed.              
                IF (srchct.gt.0) THEN
                  srchpcnt = srchct*100/max(nnsrchpt,1)
                  srchpcnt = INT(srchpcnt/10)*10
                    
                  IF (srchpcnt.gt.srchprint.or.(srchpcnt.eq.10.and.srchprint.eq.0)) THEN
!     In cases with a search grid and search refinement, the value for
!     srchct is approximate and can result in a value of srchpcnt > 100
                    IF (srchpcnt.gt.100) srchpcnt = 100                   
                    srchprint = MAX(srchprint + 10,srchpcnt)                                    
                    IF (irefine.eq.0) THEN
                      PRINT &
                        "(A20,' - Search node:',i6,',',i6,'; search grid ,',i4,' % completed,')",&
                          filin(bfile:LEN_TRIM(filin)),icen,jcen,srchpcnt
                      PRINT "('      ',i10,' trial surfaces analyzed')",ntry     
                    ELSE
                      IF (iseed.le.1) THEN
                        PRINT &
                           "(A20,' - Search node:',i6,',',i6,'; coarse search ,',i4,' % completed,')",&
                           filin(bfile:LEN_TRIM(filin)),icen,jcen,srchpcnt
                        PRINT "('      ',i10,' trial surfaces analyzed')",ntry                         
                      ELSE
                        iiseed = iseed - 1
                        PRINT &
                           "(A20,' - Search node:',i6,',',i6,'; fine search # ',i3,' ,',i4,' % completed,')",&
                           filin(bfile:LEN_TRIM(filin)),icen,jcen,iiseed,srchpcnt
                        PRINT "('      ',i10,' trial surfaces analyzed')",ntry                        
                      END IF
                    END IF
                  END IF 
                END IF
                IF (iseed.gt.1) THEN
                  srchct = srchct + nkseed(ilatt,jlatt)
                  kseed = 1
                ELSE      
                  srchct = srchct + 1
                END IF
                
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!              
!     Find starting elevation of search node.
                IF (iseed.gt.1) THEN
                  zcen = Newzcen(ilatt,jlatt,iseed)
                ELSE
                  zcen = zsmin
                END IF   
                 
!     If above DEM and ground elevation is higher than zsmin, 
!     find starting search elevation at first search lattice node above DEM.
                IF (icen.gt.0.and.jcen.gt.0.and.icen.le.nx.and.jcen.le.ny) THEN
                  IF (zdemsrch.ne.rnull) THEN
!     If creating output search file, set array value equal to 2 if within
!     DEM boundaries, or 22 if outside boundaries or zdem = rnull.
                    IF (isqout.ne.0) searchout(icen,jcen) = 2_int1 
                    IF  (zsmin.lt.zdemsrch.and.iseed.eq.0) THEN
                      nn = INT((zdemsrch-zsmin)/zsrchres) + 1
                      zcen = zsmin + (REAL(nn,dp)*zsrchres)                     
                    END IF
                  ELSE
                    IF (isqout.ne.0) searchout(icen,jcen) = 22_int1
                  END IF  ! IF (zdem(icen,jcen).ne.rnull)
                ELSE
                  IF (isqout.ne.0) searchout(icen,jcen) = 22_int1
                END IF  ! IF (icen.gt.0.and.jcen.gt.0.and.icen.le.nx.and.jcen.le.ny)
              
!     Set flag for no prior radius estimate at current search i,j location.                
                radiniprev = -1._dp              
!     Find x and y coordinates of search node based on DEM column (1,1) lower left  
!     node at x=0., y=0. Assume search node is at center of DEM cell.        
                xcen = REAL(icen-1,dp)*delxy + halfdelxy
                ycen = REAL(jcen-1,dp)*delxy + halfdelxy 
                zlast = zcen 
                fgrid = nullhi
                zsave = rnull
!    Loop over range of zcen (search lattice elevations).                
                DO                  
                  kcen = NINT((zcen-zsmin)/zsrchres) + 1                                                     
                
!    Keep track of minimum factor of safety at each search lattice
!    node and write to file if ilattice > 0
                  IF (ilattice.ne.0) THEN
!    If at a new search lattice point, write minimum fos data for last
!    grid point to file.
                    IF (zcen.ne.zlast) THEN
                      nrec = nrec + 1
                      WRITE (15,1110) xcen+xll,ycen+yll,zlast,fgrid
                    END IF      
                  END IF

                  fgrid = nullhi 
!    If higher than maximum search elevation move to next point.                
                  IF (zcen.gt.zsmax) EXIT 
                  IF (irefine.ne.0.and.iseed.eq.0) srchlatt(ilatt,jlatt,kcen) = -1                  
                  zlast = zcen           
                  rmax = 0.0_dp
!     Calculate maximum radius to furthest border.
                  DO m = 1,2
                    DO l = 1,2
                      mm = (m-1)*(nx-1) + 1
                      ll = (l-1)*(ny-1) + 1
                      xdiff = xcen-(REAL(mm-1,dp)*delxy+halfdelxy)
                      ydiff = ycen-(REAL(ll-1,dp)*delxy+halfdelxy)                  
!     If DEM has null value at corners, approximate elevation value as zmin
!     (the minimum DEM elevation) to calculate a maximum radius.
                      IF (zdem(mm,ll).eq.rnull) THEN
                        r2 = xdiff*xdiff + ydiff*ydiff + &
                            (zcen-zmin)*(zcen-zmin)
                      ELSE
                        r2 = xdiff*xdiff + ydiff*ydiff + & 
                            (zcen-zdem(mm,ll))*(zcen-zdem(mm,ll))
                      END IF                
                      rmax = MAX(rmax,r2)
                    END DO
                  END DO

                  rmax = sqrt(rmax)
                  IF ((icen.lt.1.or.icen.gt.nx).or.(jcen.lt.1.or.jcen.gt.ny)) THEN
                    zd = -888.0_dp
                    depthmin = 0.0_dp
                  ELSE 
                    zd = zdemsrch
                    IF (vadcrit.eq.3) THEN
!     If depth is primary criterion find initial depth to be used for first radius estimate
                      IF (dmin.ge.0.0_dp) THEN
                        depthmin = dmin
                      ELSE
                        depthmin = dminarray(icen,jcen)
                      END IF
                    END IF
                  END IF

!     Initialize variables used to calculate initial radius guess.
                  ierr = 0
                  iradflag = 0
                  rad = -1.0_dp
                  nset = 1      
                  va1max = 0.0_dp
                  lastrad = rad
                  nstate200 = 0
                  adjustdepthrad = 0
                  maxdepthrad = 0._dp
              
!     Calculate radius (or longest axis) of trial spheroid.                 
100               CALL Radius (ierr) 
                    
!     If radius is too large and hit max number of iterations, move to next search center.
                  IF (ierr.eq.1.and.rad.gt.rmax) THEN
                      zcen = Newzcen(ilatt,jlatt,iseed)
                      CYCLE  !  zcen DO loop
                  END IF

!     Loop over range of radii                  
                  DO
                    nstate = 0                 
                    radsq = rad*rad 
                    
!     Find all sets of contiguous DEM columns that are intersected
!     by the trial spheroid.
                    CALL Subsets (kset,ierr)
                      
!     If trying to hit maximum depth boundary but last radius change did not hit
!     target go back to Subsets and try new radius calculated in Subsets.
!     If more than 10 tries, use current radius and keep going.
                    IF (adjustdepthrad.gt.0) CYCLE

!     If more subsets than parameter nnset move to next search center.
                    IF (ierr.gt.0.and.iradflag.ne.0) THEN
                      zcen = Newzcen(ilatt,jlatt,iseed)                         
                      EXIT  ! leave radius DO loop
                    END IF
 
!     If no intersected sets recalculate radius.
                    IF (kset.eq.0) THEN
                      IF (iradflag.eq.0) THEN
                        va1max = 0.0_dp
                        go to 100
                      ELSE
                        nstate = 500
                      END IF
                    END IF
                      
!     Calculate volumes and areas of sets, and check whether they meet
!     criteria.  
                    IF (kset.gt.0) CALL Volumes          

                    IF (nstate.eq.0) THEN
                      IF (vadcrit.eq.1.or.(vadcrit.eq.3.and.vmin.gt.0._dp)) THEN
                        CALL Criteria (volume,area,vmin,vmax,armin,armax,tol)          
                      ELSE           
                        CALL Criteria (area,volume,armin,armax,vmin,vmax,tol)           
                      END IF                         
  
                      SELECT CASE (nstate)
                        CASE (0)
!     Calculate factor of safety for all sections within correct
!     volume or area interval and keep minimum factor of safety stored.        
                          CALL Fos
                        CASE (100)  
!     If need to recalculate radius and have not hit max # tries.
                          IF (iradflag.eq.0) go to 100
!     If depth is primary criterion and current radius is too large or too small, and other
!     criteria can still be met, use depth radius calculated in Criteria and return to 
!     to find new subsets.
                        CASE (200)
                          CYCLE
                      END SELECT
                    END IF                     

!     If tolerance is set for maximum depth criterion, check whether target subset radius
!     falls between last radius and next radius and adjust radius to meet maximum depth target.
!     If more than one target radius falls between last and next radii, pick smallest first.
                        IF (iradflag.ne.0.and.maxdepthrad.gt.0._dp) THEN
                          rad = maxdepthrad
                        ELSE
                          rad = rad + dr
                        END IF
                        
!     If no good sets possible with increasing radius based on currently intersected set
!     details or radius exceeds rmax, move to next search node.                     
                    IF ((nstate.eq.500).or.(rad.gt.rmax)) THEN
                      zcen = Newzcen(ilatt,jlatt,iseed)
                      EXIT  ! leave radius DO loop
                    END IF
                  END DO !  loop over radii                    
                END DO !  loop on zcen

              END DO  ! loop on icen of search grid
            END DO  ! loop on jcen of search grid
           
            IF (irefine.eq.0) THEN
              PRINT *, filin(bfile:LEN_TRIM(filin)),' - search , ',100.,'% complete,',ntry,' trial surfaces'
              EXIT
            ELSE
              IF (iseed.le.1) THEN
                PRINT *, filin(bfile:LEN_TRIM(filin)),' - coarse search , ',100.,'% complete,',ntry,' trial surfaces'
                iseed = 1
              ELSE
                iiseed = iseed - 1
                PRINT "(A,' - fine search # ',i3,', ',f4.0,'% complete,',i10,' trial surfaces')",&
                     filin(bfile:LEN_TRIM(filin)),iiseed,100.,ntry
                iiseed = iiseed + 1
              END IF
            END IF     
          END DO  ! iteration loop on seeds
          
        ELSE   
!     If option chosen to calculate FOS for single failure option
          radsq = rad*rad
          fgrid = nullhi
          iradflag = 1
          adjustdepthrad = 0
          maxdepthrad = 0._dp
          CALL Subsets (kset,ierr)
          CALL Volumes
          nrange = 1
          CALL Fos
          IF ((ilattice.ne.0)) WRITE (15,1110) xcen+xll,ycen+yll,zcen,ofos
        END IF

!     Create a new DEM with scoop removed depending on options chosen.
!     If all scoops below FOS cutoff are to be removed, array ozb is assigned
!     in suboutine FOS as scoops are found to be below FOS cutoff. 
        IF (ofos.ne.nullhi) THEN 
          IF (remove.ne.'A'.and.remove.ne.'N') THEN
            subset = 0
            IF (remove.eq.'L') THEN
!     Assign search center variable values for largest scoop under FOS cutoff.
              xcen = ovfsx
              ycen = ovfsy
              zcen = ovfsz
              rad = ovrad
              radsq = rad*rad
              mmset = ovmset
            ELSE
              IF (remove.eq.'M') THEN
!     Assign search center variables for overall minimum FOS scoop.
                xcen = ofsx
                ycen = ofsy
                zcen = ofsz
                rad = orad
                radsq = rad*rad
                mmset = omset
              END IF
            END IF
!     Assign removed scoop base elevation values to array zmid using chosen search center.
            CALL Subsets (kset,ierr)
            CALL Volumes
            mset = mmset
            i1 = mini(mset)
            i2 = maxi(mset)
            j1 = minj(mset)
            j2 = maxj(mset)
            DO j = j1,j2
              DO i = i1,i2
                IF (subset(i,j).eq.mset) THEN
                  IF (inn(i,j).eq.4) THEN
                    zdem(i,j) = zmid(i,j)
                  ELSE
!     For partial columns find new elevation of full cell to match volume removed
                    zdem(i,j) = zdem(i,j) - vol(i,j)/cellarea
                  END IF
                END IF
              END DO
            END DO
          END IF 

!     Place new DEM in original DEM array zdem.        
          IF (demflag.ne.0.and.remove.eq.'A') zdem = ozb  

          CALL cpu_time(endtime)
          runtime = endtime - starttime

          CALL Writeout (runtime)    

!     Check to see if more retrogressions are needed and
!     if min FOS is less than foscut and continue if so.
          IF (nretro.gt.0.and.iretro.lt.nretro.and.ofos.lt.foscut.and.single.eq.0) THEN
            iretro = iretro + 1
!     Average 4 surrounding cells to find node elevations for continuous 
!     DEM surface approximation      
            DO j = ny+1,1,-1
              DO i = 1,nx+1
                zcount = 0
                zdn = 0
                IF (i.gt.1) THEN
                  IF (j.gt.1) THEN
                    IF (zdem(i-1,j-1).ne.rnull) THEN
                      zdn(1) = zdem(i-1,j-1)
                      zcount = zcount + 1
                    END IF
                  END IF
                  IF (j.lt.ny+1) THEN
                    IF (zdem(i-1,j).ne.rnull) THEN
                      zdn(4) = zdem(i-1,j)
                      zcount = zcount + 1
                    END IF
                  END IF
                END IF
                IF (i.lt.nx+1) THEN
                  IF (j.gt.1) THEN
                    IF (zdem(i,j-1).ne.rnull) THEN
                      zdn(2) = zdem(i,j-1)
                      zcount = zcount + 1
                    END IF
                  END IF
                  IF (j.lt.ny+1) THEN
                    IF (zdem(i,j).ne.rnull) THEN
                      zdn(3) = zdem(i,j)
                      zcount = zcount + 1
                    END IF
                  END IF
                END IF
                IF (zcount.gt.0) zdemnodes(i,j) = SUM(zdn)/REAL(zcount,dp)         
              END DO
            END DO
            GOTO 25
          END IF
        ELSE
          errmessage = 'no valid sets were found'
          solution = 'check size constraints of sphere'
          Call WriteError(1,errmessage,problemtype,solution,'no ',0,' ')             
        END IF
        
        CLOSE(15)
        CLOSE (16)
        CLOSE (20)
        CLOSE (32)
        CLOSE (39)
        CLOSE(45)

        CALL DATE_AND_TIME(date,time) 
        
        PRINT *, filin(bfile:LEN_TRIM(filin)),' - Successful execution of Scoops3D version number ',version
        PRINT "('Date and Time: ',A2,'/',A2,'/',A4,'  ',A2,':',A2,':',A2)",&
                 date(5:6),date(7:8),date(1:4),time(1:2),time(3:4),time(5:6)

 
1000    FORMAT ('Retrogression # = ',i5,/3x,'FOS','        ',&
                 &'x','          ','y','           ','z','       ',&
                 &'rad','        ','angle','     ','vol','     ','area') 
1110    FORMAT (3ES24.17,f11.4) 
1115    FORMAT (3f30.4,f11.4,2i6)  
1116    FORMAT (2i6,2f20.4,f11.4) 
1210    FORMAT ('x_',A1,' y_',A1,' z_',A1,' F_',A4)                
1215    FORMAT ('x_',A2,' y_',A2,' z_',A2,' F_',A4)                
1220    FORMAT ('x y z F_',A4)
1230    FORMAT ('x_',A1,' y_',A1,' z_',A1,' F_',A4,' icen ',' jcen ')                
1235    FORMAT ('x_',A2,' y_',A2,' z_',A2,' F_',A4,' icen ',' jcen ')                
1240    FORMAT ('x y z F_',A4,' icen ',' jcen ')  

2000    FORMAT (5(ES24.17),2i8)
2100    FORMAT (6(ES24.17),2i8)   


        END PROGRAM Scoops3D    
             
             
        REAL(dp) FUNCTION Newzcen(ilatt,jlatt,iseed)
      
          USE CommonData, ONLY: dp
          USE GridData, ONLY: zcen
          USE SearchData, ONLY: ksmax,zsrchres,zzsrchres,srchlatt,&
                             zsmin,zsmax
          IMPLICIT NONE
        
          INTEGER, INTENT(in) :: ilatt,jlatt,iseed
          
          INTEGER :: klast,k,kseed
          
          SAVE klast
        
          IF (iseed.gt.1) THEN
            klast = klast + 1
            kseed = 0
            IF (klast.le.ksmax) THEN
              DO k = klast,ksmax
                IF (srchlatt(ilatt,jlatt,k).eq.iseed) THEN
                  kseed = k
                  EXIT
                END IF
              END DO
            END IF
            IF (kseed.gt.0) THEN
              Newzcen = zsrchres * REAL(kseed-1,dp) + zsmin
              klast = kseed
            ELSE
              Newzcen = zsmax + zzsrchres
              klast = 0
            END IF
          ELSE
            Newzcen = zcen + zzsrchres
            klast = 0
          END IF
        
        END FUNCTION Newzcen
