        SUBROUTINE SeedSearch(iseed,nseed,nres,nzres,isrchnum,jsrchnum)
        
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!       This subroutine determines new search centers in the Search lattice
!       based on centers that produced minimum FOS values on the ground
!       in the previous iteration.
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!
!        Called by Scoops
!
!        VARIABLES
!
!        delxy -- DEM grid resolution
!        fostol -- maximum percent difference in FOS between seed iterations to
!           not generate a new seed
!        fsmin(i,j) -- minimum factor of safety at each DEM column
!        fsminold(i,j) -- last minimum factor of safety at each DEM column used to
!           compare change in FOS during seed iterations to see if criterion is met
!        fsx(nx,ny),fsy(nx,ny),fsz(nx,ny) -- search spheroid center of minimum FOS 
!           scoop at each DEM column
!        idem,jdem -- DEM grid loop counters
!        ismin,ismax -- minimum and maximum boundaries of search grid on x-axis,
!           specified relative to DEM cells i=1 through nx
!        isrch,jsrch,ksrch -- search lattice loop counters
!        isrchnum,jsrchnum -- the number of search lattice nodes in x and y directions
!        iseed -- grid refinement iteration count also used to fill srchlatt
!           lattice. iseed=1 represents initial coarse grid locations.
!        jsmin,jsmax -- minimum and maximum boundaries of search grid on y-axis,
!           specified relative to DEM cells j=1 through ny
!        ksmax -- maximum boundary of search grid on z-axis calculated from 
!           user-specified zsmax and z resolution
!        multres -- resolution multiplier for coarse search lattice
!        nkseed(nxsrch,nysrch) -- number of k values for current seed at each i,j
!           search node
!        nsrchres -- resolution of finest search grid relative to DEM resolution.
!         Acts as a multiplier of DEM resolution delxy
!        nx -- number of DEM cells in x direction
!        ny -- number of DEM cells in y direction
!        rnull -- positive real null value used for initializations and set 
!           in CommonData. 
!        srchlatt(isrchnum,jsrchnum,ksmax) -- search lattice array with seed
!           iteration number at each searched node. 
!        zdem(nx,ny) -- DEM elevations
!        zsmin,zsmax -- minimum and maximum elevations of search grid
!        zsrch -- search elevation at new search center
!        zsrchres -- resolution of finest search grid on z-axis
!        zzsrchres -- resolution of current search grid on z-axis
!
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

        USE CommonData, ONLY: dp,nullhi,rnull
        USE GridData, ONLY: nx,ny,zdem,invdelxy
        USE SearchData, ONLY: zsmin,zsmax,zsrchres,srchlatt,ksmax,fostol,&
                          nkseed,ismin,jsmin,nsrchres,multres,zzsrchres
        USE FosData, ONLY: fsmin,fsx,fsy,fsz,fsminold
        
        IMPLICIT NONE

        INTERFACE
          SUBROUTINE WriteError(errtype,problem,problemtype,solution,extraflag,extra1int,extra1ch)
            INTEGER, INTENT(in) :: errtype,extra1int
            CHARACTER(len=*), INTENT(in) :: extraflag,extra1ch,problemtype,problem,solution
          END SUBROUTINE WriteError
        END INTERFACE

        INTEGER, INTENT(inout) :: iseed
        INTEGER, INTENT(out) :: nseed
        INTEGER, INTENT(in) :: nres,nzres,isrchnum,jsrchnum
        INTEGER :: idem,jdem,isrch,jsrch,ksrch,iisrch,jjsrch,iin,jin
        INTEGER :: iii,jjj,kkk
        REAL(dp) :: fsdiff,zsrch

        CHARACTER(70)  :: problemtype
        CHARACTER(120)  :: errmessage
                         
        nkseed = 0
        nseed = 0
        iseed = iseed + 1
        fsdiff = nullhi
         
!    Check all search nodes adjacent to current seed. If they are valid nodes
!    and have not been searched before, determine whether to place new seed  
!    for next search iteration.         
        DO jdem = 1,ny
          DO idem = 1,nx
            IF (fsmin(idem,jdem).lt.nullhi) THEN
!    Compare DEM cell factor of safety to value from last iteration.
              IF (iseed.gt.2) THEN
                fsdiff = (fsminold(idem,jdem) - fsmin(idem,jdem))/fsminold(idem,jdem)
              END IF
              fsminold(idem,jdem) = fsmin(idem,jdem)
!     If % change in FOS is larger than tolerance (or this is first iteration 
!     after initial coarse iteration) assign next seed number for next iteration.
              IF (fsdiff.ge.fostol) THEN
                IF (fsx(idem,jdem).gt.0) THEN               
                  iisrch = INT(fsx(idem,jdem)*invdelxy) + 1
                ELSE 
                  iisrch = INT(fsx(idem,jdem)*invdelxy)
                END IF
                IF (fsy(idem,jdem).gt.0) THEN 
                  jjsrch = INT(fsy(idem,jdem)*invdelxy) + 1
                ELSE
                  jjsrch = INT(fsy(idem,jdem)*invdelxy)
                END IF 
                isrch = NINT(REAL(iisrch-ismin)/REAL(nsrchres))+1
                jsrch = NINT(REAL(jjsrch-jsmin)/REAL(nsrchres))+1
                ksrch = NINT((fsz(idem,jdem)-zsmin)/zsrchres) + 1 
                IF (isrch.lt.1.or.isrch.gt.isrchnum.or.&
                  jsrch.lt.1.or.jsrch.gt.jsrchnum.or.&
                  ksrch.lt.1.or.ksrch.gt.ksmax) THEN
                  errmessage = 'search lattice array out of bounds'
                  problemtype = 'identifying nodes for search refinement' 
                  Call WriteError(1,errmessage,problemtype,'no','no ',0,' ')                    
                END IF
                IF (iseed.eq.2) srchlatt(isrch,jsrch,ksrch) = 1
!     Determine if nodes around last seed have been searched and if not,
!     assign new seed to those nodes.
                DO kkk = ksrch-nzres,ksrch+nzres,nzres
                  DO jjj = jsrch-multres,jsrch+multres,multres
                    DO iii = isrch-multres,isrch+multres,multres
                      IF ((iii.gt.0.and.iii.le.isrchnum).and. &
                          (jjj.gt.0.and.jjj.le.jsrchnum).and. &
                          (kkk.gt.0.and.kkk.le.ksmax)) THEN
                        IF (srchlatt(iii,jjj,kkk).eq.0) THEN
!     Find DEM cell coordinates iin,ji associated with search lattice coordinates iii,jjj.
                          IF (iii.eq.isrch) THEN
                            iin = iisrch
                          ELSE
                            IF (iii.eq.isrch-multres) THEN
                              iin = iisrch - nres
                            ELSE
                              iin = iisrch + nres
                            END IF
                          END IF
                          IF (jjj.eq.jsrch) THEN
                            jin = jjsrch
                          ELSE
                            IF (jjj.eq.jsrch-multres) THEN
                              jin = jjsrch - nres
                            ELSE
                              jin = jjsrch + nres
                            END IF
                          END IF
                          IF (kkk.eq.ksrch) THEN
                            zsrch = fsz(idem,jdem)
                          ELSE
                            IF (kkk.eq.ksrch-nzres) THEN
                              zsrch = fsz(idem,jdem) - zzsrchres
                            ELSE
                              zsrch = fsz(idem,jdem) + zzsrchres
                            END IF
                          END IF
                          IF (zsrch.le.zsmax) THEN
!     Check that new search node is above DEM elevation if within DEM bounds.
                            IF (iin.gt.0.and.jin.gt.0.and.iin.le.nx.and.jin.le.ny) THEN
                              IF (zdem(iin,jin).ne.rnull) THEN 
                                IF (zsrch.le.zdem(iin,jin)) CYCLE
                              END IF
                            END IF
                            srchlatt(iii,jjj,kkk) = iseed
                            nkseed(iii,jjj) = nkseed(iii,jjj) + 1
                            nseed = nseed + 1
                          END IF
                        END IF
                      END IF
                    END DO  ! iii
                  END DO  ! jjj  
                END DO  ! kkk
              END IF  !  IF (fsdiff.ge.fostol)
            END IF  !  IF (fsmin(idem,jdem).lt.nullhi)
          END DO  ! idem
        END DO  !  jdem
         
        END SUBROUTINE SeedSearch
