        SUBROUTINE SphereSets (kset,ierr)
        
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!       This subroutine determines all contiguous sets of columns in the 
!       DEM that are intersected by the trial slip surface.
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!
!      Called by Subsets
!
!      VARIABLES
!
!        delxy -- DEM grid resolution (delta x, delta y)
!        foundset -- flag indicating at least one subset with included columns
!        i1,i2,j1,j1 -- array bounds for current set
!        ierr -- error flag indicating initial radius problem (2) 
!           or too many subsets (1)
!           potential failure surfaces (1 if included, 0 if not)
!        inscoop(nx,ny) -- indicates whether DEM column node is bounded by spheroid
!        kset -- number of valid subsets
!        mini(ntotset),maxi(ntotset) -- minimum and maximum i bounds of each subset
!        minj(ntotset),maxj(ntotset) -- minimum and maximum j bounds of each subset
!        mset -- current land subset number
!        mz2 -- square of elevation difference from spheroid center to spheroid border
!           at current i,j location
!        nnset -- maximum number of subsets
!        nodecount -- number of land nodes contained in spheroid
!        nodein -- indicates node type as land (1), water (2) or air (3)
!        nset -- number of land subsets found
!        numset -- current set number
!        nx -- number of DEM cells in x direction
!        ny -- number of DEM cells in y direction
!        omini,omaxi,ominj,omaxj -- minimum and maximum bounds of all sets included
!           in search spheroid
!        rad -- search spheroid axes perpendicular to strike of slope
!        radsq -- square of search spheroid axes perpendicular to strike of slope
!        rnull -- real null value used throughout program 
!        set(ntotset) -- number of columns in a contiguous set
!        setflag(ntotset) -- indicates valid subset (0), or invalid by containing
!           truncated column (1), or adjacent to DEM boundary (2), or empty (-1)
!        shadow -- estimated radius of spherical shadow intersection with DEM
!        single -- flag for calculating single slip surface
!        subset(nx,ny) -- indicates set membership of DEM columns
!        xcen,ycen,zcen -- center of search spheroid relative to DEM origin
!        xdiff,ydiff -- distance from column to spheroid center
!        zb(nx,ny) -- base of slip surface at each column node
!        zbot -- Sphere bottom elevation in column used to determine if DEM is
!            in spheroid
!        zmax -- maximum DEM elevation value
!        zdemnodes(nx+1,ny+1) -- DEM node elevations averaged from surrounding cells
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

        USE CommonData, ONLY: dp,rnull
        USE GridData, ONLY: nx,ny,xcen,ycen,zcen,rad,radsq,delxy,zdemnodes,&
                           invdelxy
        USE SetData, ONLY: omini,omaxi,ominj,omaxj,mini,minj,maxi,maxj,set,&
                           setflag,nnset,nset,numset,mset,i1,j1,i2,j2,nodecount
        USE ColumnData, ONLY: zb,inscoop,subset
        USE FosData, ONLY: single   
        USE SearchData, ONLY: iradflag
        
        IMPLICIT NONE

        INTERFACE
          SUBROUTINE WriteError(errtype,problem,problemtype,solution,extraflag,extra1int,extra1ch)
            INTEGER, INTENT(in) :: errtype,extra1int
            CHARACTER(len=*), INTENT(in) :: extraflag,extra1ch,problemtype,problem,solution
          END SUBROUTINE WriteError
          
          SUBROUTINE Floodfill(i,j)
            INTEGER, INTENT(in) :: i,j
          END SUBROUTINE Floodfill
          
          SUBROUTINE SubsetError(ierr)
            INTEGER, INTENT(in) :: ierr 
          END SUBROUTINE SubsetError
          
          SUBROUTINE checksets(i,j)
            INTEGER, INTENT(in) :: i,j
          END SUBROUTINE checksets
        END INTERFACE 

        INTEGER, INTENT(out) :: kset,ierr
        
        INTEGER :: nn,i,j,m,nodein,foundset
        REAL(dp) :: mz2,zbot,xdiff,ydiff

        CHARACTER(70)  :: problemtype
        CHARACTER(120)  :: errmessage,solution
  
        kset = 0
        ierr = 0        

!     Calculate bounds so that the starting and ending columns always
!     fall outside of the search sphere "shadow"
        omini = nx+1
        omaxi = 0
        ominj = ny+1
        omaxj = 0 
        set = 0
        nset = 0
        nodecount = 0
        setflag = 0
        
        j1 = MAX(INT((ycen-rad)*invdelxy)-3,1)
        j2 = MIN(INT((ycen+rad)*invdelxy)+3,ny+1)
        i1 = MAX(INT((xcen-rad)*invdelxy)-3,1)
        i2 = MIN(INT((xcen+rad)*invdelxy)+3,nx+1)
        inscoop(i1:i2,j1:j2) = 0

        DO j = j1,j2
          ydiff = REAL(j-1,dp)*delxy - ycen
          DO i = i1,i2
            IF (i.lt.nx+1.and.j.lt.ny+1) THEN
              subset(i,j) = 0
            END IF            
            IF (inscoop(i,j).eq.0) THEN
              zb(i,j) = rnull             
              nodein = 0
              xdiff = REAL(i-1,dp)*delxy - xcen            
              mz2 = radsq-(xdiff*xdiff + ydiff*ydiff)
!     Check whether DEM node is intersected by spheroid. mz2 will
!     be less than zero if x or y of node is outside spheroid.   
!     An included node will serve as the seed for the
!     scan line floodfill routine to find all nodes in a subset.  
              IF (mz2 .gt. 0.0_dp) THEN
                zbot = zcen - sqrt(mz2)
!     Check whether DEM elevation is above bottom of spheroid at x,y location
                IF (zdemnodes(i,j).ge.zbot) THEN
                  nset = nset + 1             
                  nodein = 1          
                END IF                 
!     If too many subsets, return to Main and go to next search node.
                IF (nset.gt.nnset) THEN
                  IF (single.ne.0) THEN
                    errmessage = 'exceeded maximum number of subsets'
                    problemtype = 'determining sphere subsets'
                    solution = 'decrease DEM resolution'
                    Call WriteError(2,errmessage,problemtype,solution,'no ',0,' ') 
                    PRINT *,'         maximum number of subsets ("nnset") = ',nnset
                    WRITE (20,*) '        maximum number of subsets ("nnset") = ',nnset
                    CLOSE (20)
                    STOP
                  ELSE 
                    ierr = 1
                    IF (iradflag.ne.0) THEN
                      CALL SubsetError(ierr)
                      RETURN
                    END IF
                    nset = nset - 1
                  END IF
                END IF

!     If sphere loop being used to calculate ellipsoid fall directions, do not
!     find water subsets.              
                IF (nodein.eq.1) THEN
                  numset = nset
                  mset = numset
!     Use recursive floodfill routine to find all nodes in a subset.  Track whether
!     intersection results in truncated surface. Calculate slip base elevation and
!     save min and max bounds of all sets and number of intersected columns.
                  CALL Floodfill(i,j)
                END IF
              END IF
            END IF              
          END DO
        END DO
        
!     If no intersected nodes and not doing single run, return to Main to
!     recalculate radius.
        IF (nodecount.eq.0) THEN
          IF (single.ne.0) THEN
            errmessage = 'radius for single surface does not intersect grid'
            problemtype = 'determining sphere subsets'
            solution = 'check coordinates for center of sphere'
            Call WriteError(1,errmessage,problemtype,solution,'no ',0,' ')             
          ELSE
            RETURN
          END IF
        END IF
        
!     Expand bounds of all intersected nodes to take column calculations
!     into account. 
        j1 = MAX(ominj-1,1)
        j2 = MIN(omaxj,ny)
        i1 = MAX(omini-1,1)
        i2 = MIN(omaxi,nx)
!     Reinitialize subset bounds for column assignments
        mini = 0
        maxi = 0
        minj = 0
        maxj = 0
        set = 0
      
        DO j = j1,j2
          DO i = i1,i2
!     Determine set membership of each column and number of subsets. 
!     Track min and max array bounds of columns in each set.
            CALL checksets(i,j)
          END DO
        END DO
 
!     Check if more sets counted than valid sets from assigning columns.
!     Remove empty sets if no good sets after them and count how many good sets.

        nn = nset
        foundset = 0
        DO m = nn,1,-1
          IF (set(m).eq.0) THEN
            IF (m.eq.nn) THEN
              nset = nset - 1
            ELSE
              IF ((set(m+1).eq.0).and.(foundset.eq.0))&
                  nset = nset - 1
            END IF
            setflag(m) = -1  ! empty set
          ELSE
            foundset = 1
            kset = kset + 1
          END IF
        END DO

        END SUBROUTINE SphereSets
           
          
