        SUBROUTINE Subsets (kset,ierr)
        
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!       This subroutine determines the contiguous set of columns in the 
!       DEM that is intersected by the trial slip surface and contains
!       the ground-centric search node.
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!
!      Called by Scoops
!
!      VARIABLES
!
!        adjustdepthrad -- flag indicating radius is being adjusted to meet the 
!           maximum depth criterion within specified tolerance (dtol)
!        depthmax(nnset) -- maximum column depth in each subset
!        depthrad(nnset) -- estimated radius to meet minimum or maximum depth criterion
!        depthtargetmax -- maximum depth criterion for current column
!        depthtargetmin -- minimum depth criterion for current column
!        dmax -- maximum depth criterion, scalar if > 0, array if < 0
!        dmaxarray(nx,ny) -- array of maximum depths for each DEM column
!        dmin -- minimum depth criterion, scalar if > 0, array if < 0
!        dminarray(nx,ny) -- array of minimum depths for each DEM column
!        dtol -- tolerance on depth criterion for calculating initial radius to meet
!           minimum depth or for meeting maximum depth if required
!        goodset -- flag for finding valid subset
!        idepthflag -- flag indicating whether depth criteria entered as depth (1) 
!           or elevation using stratigraphic layer (2)
!        ierr -- error flag indicating initial radius problem (2) 
!           or too many subsets (1)
!        inn(nx,ny) -- number of contiguous nodes of each column bounded by spheroid,
!           equals -1 if truncated surface, -2 if column contains boundary node.
!        kset -- number of valid subsets
!        m -- subset number
!        maxddiff(nnset) -- maximum difference between subset column bottoms and maximum depth 
!           criterion (negative if subset column bottom falls above maximum depth)
!        mdepth -- array location of subset with slip base closest to the maximum depth target
!        maxdepthrad -- radius calculated in Subsets to hit maximum depth critera
!        mini(ntotset),maxi(ntotset) -- minimum and maximum i bounds of each subset
!        minj(ntotset),maxj(ntotset) -- minimum and maximum j bounds of each subset
!        nset -- number of land subsets found
!        omini,omaxi,ominj,omaxj -- minimum and maximum bounds of all sets included
!           in search spheroid
!        setflag(ntotset) -- indicates valid subset (0), or invalid by containing
!           truncated column (1), or adjacent to DEM boundary (2), or empty (-1)
!        single -- flag for calculating single slip surface
!        subset(nx,ny) -- indicates set membership of DEM columns
!        zmid(nx,ny) -- base of slip surface at midpoint of intersected column
!
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

        USE CommonData, ONLY: dp,rnull,nullhi
        USE GridData, ONLY: zdem,radsq,xcen,ycen,zcen,delxy,lastrad,rad
        USE SetData, ONLY: omini,omaxi,ominj,omaxj,mini,minj,maxi,maxj,&
                           setflag,nset,nnset
        USE ColumnData, ONLY: subset,inn,zmid
        USE FosData, ONLY: single,isizefiles,dr
        USE SearchData, ONLY: iradflag
        USE DepthData, ONLY: depthrad,minddiff,maxddiff,depthmax,dmin,dminarray,dmax,&
                            dmaxarray,idepthflag,adjustdepthrad,dtol,maxdepthrad,imaxdepth
        
        IMPLICIT NONE

        INTERFACE
          SUBROUTINE WriteError(errtype,problem,problemtype,solution,extraflag,extra1int,extra1ch)
            INTEGER, INTENT(in) :: errtype,extra1int
            CHARACTER(len=*), INTENT(in) :: extraflag,extra1ch,problemtype,problem,solution
          END SUBROUTINE WriteError
          
          SUBROUTINE SphereSets (kset,ierr)
            INTEGER, INTENT(out) :: kset,ierr
          END SUBROUTINE SphereSets
          
          SUBROUTINE SubsetError(ierr)
            INTEGER, INTENT(in) :: ierr 
          END SUBROUTINE SubsetError
        END INTERFACE

        INTEGER, INTENT(out) :: kset,ierr
        INTEGER :: i,j,m,goodset,mdepth

        CHARACTER(70)  :: problemtype
        CHARACTER(120)  :: errmessage,solution
        REAL(dp) :: xprime,yprime,xyprimesq,zz,adjustrad,diff,depthtargetmin,depthtargetmax
        REAL(dp) :: ztarget,zdiff,mxdepthrad(nnset),nextrad,zprime,zzdem
          
        ierr = 0   
        
        CALL SphereSets(kset,ierr)
!     If no valid sets return to Main to recalculate radius.
        IF (kset.eq.0) THEN
          nset = 0
          adjustdepthrad = 0
          IF (single.ne.0) THEN
            errmessage = 'no valid sets for single surface'
            problemtype = 'determining subsets' 
            solution = 'check coordinates for center of sphere'
            Call WriteError(1,errmessage,problemtype,solution,'no ',0,' ')             
          ELSE
            RETURN
          END IF
        END IF
        IF (ierr.gt.0) THEN
          CALL SubsetError(ierr)
          RETURN
        END IF
                        
!     Determine min and max bounds of columns in union of all sets.       
        omini = MINVAL(mini,MASK=(mini.gt.0))
        omaxi = MAXVAL(maxi)
        ominj = MINVAL(minj,MASK=(minj.gt.0))
        omaxj = MAXVAL(maxj)
        IF (idepthflag.gt.0) THEN
          maxddiff = rnull
          minddiff = nullhi
          depthrad = rnull
          mxdepthrad = rnull
          IF (dmax.ge.0.0_dp) depthtargetmax = dmax
          IF (dmin.ge.0.0_dp) depthtargetmin = dmin
        END IF
        IF (idepthflag.gt.0.or.isizefiles.ne.0) depthmax = rnull
  
        DO j = ominj,omaxj
          DO i = omini,omaxi
            m = subset(i,j)       
!     Find slip base elevation at column center.  
            IF (m.gt.0.and.inn(i,j).eq.4) THEN
              xprime = (REAL(i,dp)-.5_dp)*delxy - xcen
              yprime = (REAL(j,dp)-.5_dp)*delxy - ycen
              xyprimesq = xprime*xprime + yprime*yprime
              zz = radsq - xyprimesq
              zzdem = zdem(i,j)
              IF (zz.gt.0._dp) THEN
                zmid(i,j) = zcen - sqrt(zz)
              ELSE
                zmid(i,j) = zzdem
              END IF
              zprime = zcen - zmid(i,j)
              IF (idepthflag.gt.0.or.isizefiles.ne.0) &
!     Keep track of maximum column depth
                depthmax(m) = max(zzdem - zmid(i,j),depthmax(m))                  
              IF (idepthflag.gt.0) THEN
                IF (dmax.lt.0.0_dp) depthtargetmax = dmaxarray(i,j)
!     Find maximum distance between maximum depth criterion and column base
!     in subset if slip surface is below the depth criterion (positive maxddiff), 
!     or minimum distance if it is above (negative maxddiff).
                IF (depthtargetmax.gt.0._dp) THEN
                  ztarget = zcen - (zzdem - depthtargetmax)
                  zdiff = zprime - ztarget
                  IF (zdiff.gt.maxddiff(m)) THEN
                    maxddiff(m) = zdiff
!     If trying to hit maximum depth within tolerance find target radius
                    IF (imaxdepth.ne.0) THEN
                      mxdepthrad(m) = sqrt(ztarget*ztarget + xyprimesq)
                    END IF
                  END IF
                END IF
!     Find maximum distance between minimum depth criterion and column base
!     in subset if slip surface is above the depth criterion (positive minddiff), 
!     or minimum distance if it is below (negative minddiff).
                IF (dmin.lt.0.0_dp) depthtargetmin = dminarray(i,j)
                IF (depthtargetmin.ge.0._dp) THEN
                  ztarget = zcen - (zzdem - depthtargetmin)
                  zdiff = ztarget - zprime
                  IF (zdiff.lt.minddiff(m).or.minddiff(m).eq.nullhi) THEN
                    minddiff(m) = zdiff
!     If trying to hit initial radius based on depth find new radius
                    IF (iradflag.eq.0) THEN
                      depthrad(m) = sqrt(ztarget*ztarget + xyprimesq)
                    END IF
                  END IF
                END IF                    
              END IF
            END IF   
          END DO
        END DO

 
!     If trying to hit maximum depth criterion 
        IF (single.eq.0.and.imaxdepth.ne.0) THEN
          IF (iradflag.eq.1.and.(maxdepthrad.gt.0.0_dp.or.adjustdepthrad.gt.0)) THEN
!     Find array location of subset with slip base closest to the maximum depth target
!     to determine the subset bracketed by prior radius and r+dr.
            mdepth = MINLOC(abs(maxddiff),1,MASK=maxddiff.ne.rnull)
            adjustrad = mxdepthrad(mdepth)
            diff = maxddiff(mdepth)
!     lastrad is used for lattice search to check that the new radius would be bigger
!     than the last radius used to fit max depth for prior subset. 
            IF (adjustrad.gt.lastrad) THEN
              IF (adjustdepthrad.gt.4) THEN
  !     If don't hit good radius after 5 tries keep current radius and move on
                adjustdepthrad = -1
                lastrad = rad
                maxdepthrad = rnull
              ELSE
                IF (diff.gt.0._dp.or.-diff.gt.dtol) THEN
                  IF (diff.gt.0._dp.and.diff.lt.1.D-8) THEN
                    rad = rad - 1.D-8
                  ELSE
                    rad = adjustrad
                  END IF
                  adjustdepthrad = adjustdepthrad + 1
                ELSE
  !     New radius fits maximum depth tolerance.  
                  adjustdepthrad = -1
                  lastrad = rad
                  maxdepthrad = rnull
                END IF
              END IF
            ELSE 
              adjustdepthrad = 0
              maxdepthrad = rnull
            END IF 
          END IF
          
!     If tolerance is set for maximum depth criterion, check whether target subset radius
!     falls between last radius and next radius and find radius to meet maximum depth target.
!     If more than one target radius falls between last and next radii, pick smallest first.
!     If current radius hits depth target (adjustdepthrad = -1) don't check for other subsets
!     to hit boundary with next radius change because of issues with sets coalescing.  Later
!     subsets can be forced to hit depth target.
          IF (imaxdepth.ne.0.and.adjustdepthrad.eq.0) THEN
            nextrad = rad + dr
            maxdepthrad = rnull
            DO m = 1,nset
              IF (rad.lt.mxdepthrad(m).and.nextrad.gt.mxdepthrad(m).and.&
                -maxddiff(m).gt.dtol) THEN
                IF (maxdepthrad.lt.0.0_dp) THEN
                  maxdepthrad = mxdepthrad(m)
                ELSE
                  IF (mxdepthrad(m).lt.maxdepthrad) maxdepthrad = mxdepthrad(m)
                END IF
              END IF
            END DO
          END IF
        END IF
          
        IF (single.ne.0) THEN
!    Determine if any good sets
          goodset = 0         
          DO m = 1,nset
            IF (setflag(m).eq.0) THEN 
              goodset = 1
            END IF
          END DO
          IF (goodset.eq.0) THEN
            errmessage = 'no valid sets for single surface (contains boundary or truncated surface)'
            problemtype = 'determining subsets'
            solution = 'check coordinates for center of sphere'
            Call WriteError(1,errmessage,problemtype,solution,'no ',0,' ')             
          END IF
        END IF

        END SUBROUTINE Subsets
           
          
