      SUBROUTINE Volumes
        
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!     This subroutine calculates the volumes and areas of all  
!     subsets intersected by spheroid. Calculations assume planar
!     slip surface defined by intersection of search spheroid with 
!     column corners.
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!
!     Called by Scoops3
!
!     VARIABLES
!
!        area(nnset) -- surface area of subset
!        cellarea -- area of DEM grid cell (delxy*delxy)
!        colvol -- function that calculates volume of a trapezoidal column
!        colxy(nx,ny) -- length of each column side (delxy for full columns)
!        delxy -- DEM grid resolution (delta x, delta y)
!        dx1,dx2,dy1,dy2 -- x and y grid spacing of partial column relative to inside nodes
!        dz(4) -- depth of column from DEM surface to slip base
!        dzin -- for 3-node columns, depth of column at node connected to shortest 
!           column side
!        goodset -- flag indicating a volume was calculated for at least one subset
!        i,j -- DEM grid array location
!        inn(nx,ny) -- number of nodes of each column bounded by spheroid or
!           equals -1 if truncated surface, -2 if contains boundary node.
!        inscoop(nx,ny) -- indicates whether DEM column node is bounded by spheroid;
!        m -- set number
!        mini(ntotset),maxi(ntotset) -- minimum and maximum i bounds of each subset
!        minj(ntotset),maxj(ntotset) -- minimum and maximum j bounds of each subset
!        ncol(nnset) -- number of columns in subset
!        nodes -- count of included nodes used to calculate average DEM column elevation
!        nodesin -- number of nodes of each column bounded by spheroid
!        nonode -- flag for when there is no intersection between search
!           spheroid and slope line between outside and inside nodes or partial
!           column is truncated.
!           If nonode = 1, new node can not be calculated.
!           If nonode = 2, truncated column.
!        nset -- number of land subsets
!        nx -- number of DEM cells in x direction
!        ny -- number of DEM cells in y direction
!        onenode -- corner location of included column node for single node column
!        outnodes(2,nx,ny) -- corner location of column nodes which fall
!           outside the search spheroid
!        parea -- surface area of column
!        set(ntotset) -- number of nodes in a contiguous set
!        setflag(ntotset) -- indicates valid subset (0), or invalid by containing
!           truncated node (1), or adjacent to DEM boundary (2), or empty (-1)
!        single -- flag for calculating single slip surface
!        subset(nx,ny) -- indicates set membership of DEM columns
!        vol(nx,ny) -- volume of column in subset
!        volume(nnset) -- volume of subset
!        z(4) -- used to find average column length
!        zb(nx+1,ny+1) -- base of slip surface at column nodes
!        zbase(4) -- elevation of base of slip surface of each node in
!           column 
!        zdem(nx,ny) -- DEM elevations
!        ztop -- local variable for DEM column elevation
!
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

        USE CommonData, ONLY: dp
        USE GridData, ONLY: delxy,cellarea,zdem,inv3,zcen,zdemnodes,inv12,twoinv3
        USE SetData, ONLY: volume,area,mini,minj,maxi,maxj,set,setflag,&
                           ncol,goodset,mset,nset
        USE FosData, ONLY: single
        USE ColumnData, ONLY: x1,x2,y1,y2,dx1,dx2,dy1,dy2,nodesin,onenode,ztop,&
                           outnode1,outnode2,zbase,vol,parea,colxy,znodes,zmid,&
                           outnodes,inn,inscoop,subset,zb,nonode

        IMPLICIT NONE

        INTERFACE
          SUBROUTINE NewNodes
          END SUBROUTINE NewNodes
          
          SUBROUTINE WriteError(errtype,problem,problemtype,solution,extraflag,extra1int,extra1ch)
            INTEGER, INTENT(in) :: errtype,extra1int
            CHARACTER(len=*), INTENT(in) :: extraflag,extra1ch,problemtype,problem,solution
          END SUBROUTINE WriteError
          
          PURE REAL(dp) FUNCTION colvol(z)
            USE CommonData
            REAL(dp), INTENT(in) :: z(4)
          END FUNCTION colvol
      
          PURE REAL(dp) FUNCTION trivol(ddx1,ddx2,z)    
            USE CommonData
            REAL(dp), INTENT(in) :: ddx1,ddx2,z
          END FUNCTION trivol
        END INTERFACE

        INTEGER :: i,j,nn,inst(4),ii,zeroct

        REAL(dp) :: side1,side2,dzin,dz(4),volcol

        CHARACTER(70)  :: problemtype
        CHARACTER(120)  :: errmessage,solution

        goodset = 0
          
        DO mset = 1,nset
          area(mset) = 0.0_dp
          volume(mset) = 0.0_dp
          ncol(mset) = 0
          zeroct = 0
          IF (set(mset).ne.0) THEN
            DO j = minj(mset),maxj(mset)
              DO i = mini(mset),maxi(mset)
                IF (subset(i,j).eq.mset) THEN
                  nodesin = inn(i,j) 
                  IF (nodesin.eq.0) THEN
                    print *,'case where subset= mset but nodesin is 0'
                    subset(i,j) = 0
                    CYCLE
                  END IF
                  ztop = zdem(i,j)
                  IF (nodesin.lt.4) THEN
                    onenode = 0
                    nonode = 1
                    nn = 0
                    dx1 = delxy
                    dx2 = delxy
                    dy1 = delxy
                    dy2 = delxy
                    znodes(1) = zdemnodes(i,j)
                    znodes(2) = zdemnodes(i+1,j)
                    znodes(3) = zdemnodes(i+1,j+1)
                    znodes(4) = zdemnodes(i,j+1)
                    zbase(1) = zb(i,j)
                    zbase(2) = zb(i+1,j)
                    zbase(3) = zb(i+1,j+1)
                    zbase(4) = zb(i,j+1)
                    inst(1) = inscoop(i,j)
                    inst(2) = inscoop(i+1,j)
                    inst(3) = inscoop(i+1,j+1)
                    inst(4) = inscoop(i,j+1)
!     If DEM is above rotational center in a partial column then the set is truncated and disallowed.
                    IF (ztop.gt.zcen) THEN
                      IF (setflag(mset).ne.2) setflag(mset) = 1
                    END IF
!     Determine which column nodes are not in surface for partial columns.
                    DO ii = 1,4
                      IF (inst(ii).eq.mset) THEN
                        IF (nodesin.eq.1) THEN
                          onenode = ii
                          outnodes(1,i,j) = ii
                          nonode = 0
                          EXIT
                        END IF
                      ELSE
                        nn = nn+1      
                        IF (nn.gt.3) THEN ! no nodes are contained
                          nonode = 1                        
                          EXIT                          
                        END IF          
                        IF (nn.lt.3) outnodes(nn,i,j) = ii
                        nonode = 0 
                      END IF
                    END DO
                    IF (nodesin.gt.1) THEN
                      outnode1 = outnodes(1,i,j)
                      outnode2 = outnodes(2,i,j)
                    END IF
!     Find new coordinates for nodes in partially contained columns.             
                    IF (nonode.eq.0) THEN 
                      x1 = REAL(i-1,dp)*delxy
                      y1 = REAL(j-1,dp)*delxy
                      x2 = x1+delxy
                      y2 = y1+delxy
                      CALL newnodes
                    END IF
!     If new nodes could not be calculated due to precision errors
!     in initial calculation, throw out this column
                    IF (nonode.eq.1) THEN
                      subset(i,j) = 0
                      CYCLE
                    END IF
!     If two-node column changed to one-node due to precision errors
!     identify in node.
                    IF (nodesin.gt.1.and.onenode.gt.0) THEN
                      outnodes(1,i,j) = onenode
                      outnodes(2,i,j) = 0
                      inn(i,j) = 1
                      nodesin = 1
                      inst = 0
                      inst(onenode) = mset
                    END IF
                                                      
!     Calculate volumes of prismoid columns and projected areas of slip surface.
                    DO ii = 1,4
                      IF (inst(ii).eq.mset) THEN
                        dz(ii) = max(znodes(ii) - zbase(ii),0.0_dp)
                      ELSE
                        dz(ii) = 0.0_dp
                      END IF
                    END DO
                    IF (nodesin.gt.1) THEN            
                      volcol = colvol(dz)
                      IF (nodesin.eq.3) THEN
                        SELECT CASE (outnodes(1,i,j))
                         CASE (1)
                          IF (dx1.le.0.0_dp) THEN
                            side1 = dx1
                            side2 = dy1
                            dzin = dz(2)
                          ELSE
                            side1 = dy1
                            side2 = dx1
                            dzin = dz(4)
                          END IF
                         CASE (2)
                          IF (dx1.le.0.0_dp) THEN
                            side1 = dx1
                            side2 = dy2
                            dzin = dz(1)
                          ELSE
                            side1 = dy2
                            side2 = dx1
                            dzin = dz(3)
                          END IF
                         CASE (3)
                          IF (dx2.le.0.0_dp) THEN
                            side1 = dx2
                            side2 = dy2
                            dzin = dz(4)
                          ELSE
                            side1 = dy2
                            side2 = dx2
                            dzin = dz(2)
                          END IF
                         CASE (4)
                          IF (dx2.le.0.0_dp) THEN
                            side1 = dx2
                            side2 = dy1
                            dzin = dz(3)
                          ELSE
                            side1 = dy1
                            side2 = dx2
                            dzin = dz(1)
                          END IF
                        END SELECT                    
                        volcol = volcol + trivol(side1,side2,dzin)
                        parea = cellarea - 0.5_dp*(delxy + side1)*(delxy - side2)
                      ELSE 
                        parea = .250_dp * (dx1+dx2)*(dy1+dy2)
                      END IF
                    ELSE  !  only one node included (dx and dy outside spheroid are set to zero)
                      parea =  .5_dp * (dx1+dx2)*(dy1+dy2)
                      volcol = inv3*parea*dz(onenode)                  
                    END IF
                  ELSE  !  full column (nodesin = 4)
!     Prismoidal formula with central panel average base elevation estimated as spheroid
!     elevation at center of column.
                    zbase(1) = MIN(zb(i,j),ztop)
                    zbase(2) = MIN(zb(i+1,j),ztop)
                    zbase(3) = MIN(zb(i+1,j+1),ztop)
                    zbase(4) = MIN(zb(i,j+1),ztop)
                    volcol = (ztop - (twoinv3*zmid(i,j) + inv12*SUM(zbase))) * cellarea
                    parea = cellarea
                    IF (volcol.le.0._dp) THEN
                      zeroct = zeroct + 1
                      volcol = 0._dp
                    END IF
                  END IF
                  IF (volcol.gt.0._dp.and.parea.gt.0._dp) THEN                  
                    IF (nodesin.lt.4) THEN
                      colxy(1,i,j) = dx1
                      colxy(2,i,j) = dx2
                      colxy(3,i,j) = dy1
                      colxy(4,i,j) = dy2
                    END IF
                    volume(mset) = volume(mset) + volcol
                    area(mset) = area(mset)+ parea  ! for projected area
                    ncol(mset) = ncol(mset) + 1
                  ELSE
                    subset(i,j) = 0
                  END IF
                  vol(i,j) = volcol
                END IF  ! IF (subset(i,j).eq.mset)) 
              END DO  ! loop on i
            END DO  ! loop on j
          END IF  ! IF (set(mset).ne.0)
          
          IF (single.eq.0.or.setflag(mset).eq.0) THEN
            IF (volume(mset).gt.0.0_dp) THEN
              IF (zeroct.eq.0) THEN
                goodset = 1
              ELSE
!     If more than 1% of full columns has zero volume due to 'holes' in DEM disallow subset
                IF (REAL(zeroct)/REAL(ncol(mset)).gt.0.01) THEN
                  setflag(mset) = 3
                ELSE
                  goodset = 1
                END IF
              END IF
            END IF
          END IF
        END DO  ! loop on mset
        
        IF (goodset.eq.0) THEN
          IF (single.ne.0) THEN
            IF (setflag(mset).eq.1) THEN
              errmessage = 'no valid potential failure for single surface (surface is truncated)'
            ELSE
              errmessage = 'no valid potential failure for single surface (surface has too many holes)'
            END IF
            problemtype = 'calculating volume'
            solution = 'check coordinates for center of spheroid'
            Call WriteError(1,errmessage,problemtype,solution,'no ',0,' ')
          END IF
        END IF

        END SUBROUTINE volumes
        
             
      PURE REAL(dp) FUNCTION colvol(z)
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!     This function calculates the volume using prismoidal formula
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!
!     Called by volumes and watercolvol
!
!     VARIABLES
!
!        ddx1,ddx2,ddy1,ddy2 -- x and y grid spacing of partial column relative
!          to inside nodes
!        s0,s1,s2 -- area of two parallel sides and parallel center pane of 
!           trapezoidal column
!        z(4) -- depth of trapezoidal column at each corner
!
!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX      
        USE CommonData, ONLY: dp
        USE GridData, ONLY: delxyvol,delxy
        USE ColumnData, ONLY: dx1,dx2,dy1,dy2
        IMPLICIT NONE
        
        REAL(dp), INTENT(in) :: z(4)
        REAL(dp) :: s0,s1,s2
        
        IF ((dx1.eq.delxy.or.dx1.le.0.0_dp).and.(dx2.eq.delxy.or.dx2.le.0.0_dp)) THEN
          s0 = (z(1) + z(4)) * dy1
          s1 = SUM(z) * (dy1 + dy2)
          s2 = (z(2) + z(3)) * dy2            
        ELSE
          s0 = (z(1) + z(2)) * dx1
          s1 = SUM(z) * (dx1 + dx2)
          s2 = (z(3) + z(4)) * dx2            
        END IF
           
        colvol = (s0 + s1 + s2) * delxyvol
        
      END FUNCTION colvol
      
      PURE REAL(dp) FUNCTION trivol(ddx1,ddx2,z)
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!     This function calculates the volume of the additional pyramid added
!     to trapezoidal volume for columns with three nodes in.
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!
!     Called by volumes and watercolvol
!
!     VARIABLES
!
!        ddx1,ddx2 -- lengths of partial column sides that intersect spheroid
!        z -- depth of trapezoidal column at node connected to shortest side
!
!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX        
        USE CommonData, ONLY: dp
        USE GridData, ONLY: delxy,inv6
        IMPLICIT NONE
        
        REAL(dp), INTENT(in) :: ddx1,ddx2,z
           
        trivol = inv6 * (z * abs(ddx1)) * (delxy - ddx2)
        
      END FUNCTION trivol
